@@ -36,7 +36,7 @@ require_once "$IP/maintenance/Maintenance.php";
 class CountFancyCaptchas extends Maintenance {
 	public function __construct() {
 		parent::__construct();
-		$this->mDescription = "Counts the number of fancy aptchas in storage";
+		$this->addDescription( "Counts the number of fancy aptchas in storage" );
 		$this->requireExtension( "FancyCaptcha" );
 	}
 
