@@ -9,7 +9,7 @@ require "$IP/maintenance/Maintenance.php";
 class Update extends Maintenance {
 	public function __construct() {
 		parent::__construct();
-		$this->mDescription = 'Fetches translation updates to MediaWiki core, skins and extensions.';
+		$this->addDescription( 'Fetches translation updates to MediaWiki core, skins and extensions.' );
 		$this->addOption(
 			'repoid',
 			'Fetch translations from repositories identified by this',
@@ -27,7 +27,6 @@ class Update extends Maintenance {
 		ini_set( 'memory_limit', -1 );
 
 		global $IP;
-		global $wgExtensionMessagesFiles;
 		global $wgLocalisationUpdateRepositories;
 		global $wgLocalisationUpdateRepository;
 
@@ -40,7 +39,7 @@ class Update extends Maintenance {
 		$lc = Language::getLocalisationCache();
 		$messagesDirs = $lc->getMessagesDirs();
 
-		$finder = new LocalisationUpdate\Finder( $wgExtensionMessagesFiles, $messagesDirs, $IP );
+		$finder = new LocalisationUpdate\Finder( $messagesDirs, $IP );
 		$readerFactory = new LocalisationUpdate\ReaderFactory();
 		$fetcherFactory = new LocalisationUpdate\FetcherFactory();
 
@@ -52,13 +51,18 @@ class Update extends Maintenance {
 		}
 		$repos = $wgLocalisationUpdateRepositories[$repoid];
 
+		// output and error methods are protected, hence we add logInfo and logError
+		// public methods, that hopefully won't conflict in the future with the base class.
+		$logger = $this;
+
 		// Do it ;)
 		$updater = new LocalisationUpdate\Updater();
 		$updatedMessages = $updater->execute(
 			$finder,
 			$readerFactory,
 			$fetcherFactory,
-			$repos
+			$repos,
+			$logger
 		);
 
 		// Store it ;)
@@ -74,6 +78,14 @@ class Update extends Maintenance {
 		}
 		$this->output( "Saved $count new translations\n" );
 	}
+
+	public function logInfo( $msg ) {
+		$this->output( $msg . "\n" );
+	}
+
+	public function logError( $msg ) {
+		$this->error( $msg );
+	}
 }
 
 $maintClass = Update::class;
