@@ -17,10 +17,15 @@ use StringUtils;
 use StubObject;
 use Title;
 
+/**
+ * Parser function handlers
+ *
+ * @link https://www.mediawiki.org/wiki/Help:Extension:ParserFunctions
+ */
 class ParserFunctions {
-	public static $mExprParser;
-	public static $mTimeCache = [];
-	public static $mTimeChars = 0;
+	private static $mExprParser;
+	private static $mTimeCache = [];
+	private static $mTimeChars = 0;
 
 	/** ~10 seconds */
 	const MAX_TIME_CHARS = 6000;
@@ -30,7 +35,7 @@ class ParserFunctions {
 	 * We defer this until needed to avoid the loading of the code of this file
 	 * when no parser function is actually called.
 	 */
-	public static function registerClearHook() {
+	private static function registerClearHook() {
 		static $done = false;
 		if ( !$done ) {
 			global $wgHooks;
@@ -44,7 +49,7 @@ class ParserFunctions {
 	/**
 	 * @return ExprParser
 	 */
-	public static function &getExprParser() {
+	private static function &getExprParser() {
 		if ( !isset( self::$mExprParser ) ) {
 			self::$mExprParser = new ExprParser;
 		}
@@ -52,65 +57,65 @@ class ParserFunctions {
 	}
 
 	/**
+	 * {{#expr: expression }}
+	 *
+	 * @link https://www.mediawiki.org/wiki/Help:Extension:ParserFunctions##expr
+	 *
 	 * @param Parser $parser
 	 * @param string $expr
 	 * @return string
 	 */
-	public static function expr( $parser, $expr = '' ) {
+	public static function expr( Parser $parser, $expr = '' ) {
 		try {
 			return self::getExprParser()->doExpression( $expr );
 		} catch ( ExprError $e ) {
-			return '<strong class="error">' . htmlspecialchars( $e->getMessage() ) . '</strong>';
-		}
-	}
-
-	/**
-	 * @param Parser $parser
-	 * @param string $expr
-	 * @param string $then
-	 * @param string $else
-	 * @return string
-	 */
-	public static function ifexpr( $parser, $expr = '', $then = '', $else = '' ) {
-		try {
-			$ret = self::getExprParser()->doExpression( $expr );
-			if ( is_numeric( $ret ) ) {
-				$ret = (float)$ret;
-			}
-			if ( $ret ) {
-				return $then;
-			} else {
-				return $else;
-			}
-		} catch ( ExprError $e ) {
-			return '<strong class="error">' . htmlspecialchars( $e->getMessage() ) . '</strong>';
+			return '<strong class="error">' . htmlspecialchars( $e->getUserFriendlyMessage() ) . '</strong>';
 		}
 	}
 
 	/**
+	 * {{#ifexpr: expression | value if true | value if false }}
+	 *
+	 * @link https://www.mediawiki.org/wiki/Help:Extension:ParserFunctions##ifexpr
+	 *
 	 * @param Parser $parser
 	 * @param PPFrame $frame
 	 * @param array $args
 	 * @return string
 	 */
-	public static function ifexprObj( $parser, $frame, $args ) {
+	public static function ifexpr( Parser $parser, PPFrame $frame, array $args ) {
 		$expr = isset( $args[0] ) ? trim( $frame->expand( $args[0] ) ) : '';
 		$then = $args[1] ?? '';
 		$else = $args[2] ?? '';
-		$result = self::ifexpr( $parser, $expr, $then, $else );
+
+		try {
+			$result = self::getExprParser()->doExpression( $expr );
+			if ( is_numeric( $result ) ) {
+				$result = (float)$result;
+			}
+			$result = $result ? $then : $else;
+		} catch ( ExprError $e ) {
+			return '<strong class="error">' . htmlspecialchars( $e->getUserFriendlyMessage() ) . '</strong>';
+		}
+
 		if ( is_object( $result ) ) {
 			$result = trim( $frame->expand( $result ) );
 		}
+
 		return $result;
 	}
 
 	/**
+	 * {{#if: test string | value if test string is not empty | value if test string is empty }}
+	 *
+	 * @link https://www.mediawiki.org/wiki/Help:Extension:ParserFunctions##if
+	 *
 	 * @param Parser $parser
 	 * @param PPFrame $frame
 	 * @param array $args
 	 * @return string
 	 */
-	public static function ifObj( $parser, $frame, $args ) {
+	public static function if( Parser $parser, PPFrame $frame, array $args ) {
 		$test = isset( $args[0] ) ? trim( $frame->expand( $args[0] ) ) : '';
 		if ( $test !== '' ) {
 			return isset( $args[1] ) ? trim( $frame->expand( $args[1] ) ) : '';
@@ -120,12 +125,16 @@ class ParserFunctions {
 	}
 
 	/**
+	 * {{#ifeq: string 1 | string 2 | value if identical | value if different }}
+	 *
+	 * @link https://www.mediawiki.org/wiki/Help:Extension:ParserFunctions##ifeq
+	 *
 	 * @param Parser $parser
 	 * @param PPFrame $frame
 	 * @param array $args
 	 * @return string
 	 */
-	public static function ifeqObj( $parser, $frame, $args ) {
+	public static function ifeq( Parser $parser, PPFrame $frame, array $args ) {
 		$left = isset( $args[0] ) ? self::decodeTrimExpand( $args[0], $frame ) : '';
 		$right = isset( $args[1] ) ? self::decodeTrimExpand( $args[1], $frame ) : '';
 
@@ -139,50 +148,56 @@ class ParserFunctions {
 	}
 
 	/**
+	 * {{#iferror: test string | value if error | value if no error }}
+	 *
+	 * Error is when the input string contains an HTML object with class="error", as
+	 * generated by other parser functions such as #expr, #time and #rel2abs.
+	 *
+	 * @link https://www.mediawiki.org/wiki/Help:Extension:ParserFunctions##iferror
+	 *
 	 * @param Parser $parser
-	 * @param string $test
-	 * @param string $then
-	 * @param bool $else
-	 * @return bool|string
+	 * @param PPFrame $frame
+	 * @param array $args
+	 * @return string
 	 */
-	public static function iferror( $parser, $test = '', $then = '', $else = false ) {
+	public static function iferror( Parser $parser, PPFrame $frame, array $args ) {
+		$test = isset( $args[0] ) ? trim( $frame->expand( $args[0] ) ) : '';
+		$then = $args[1] ?? false;
+		$else = $args[2] ?? false;
+
 		if ( preg_match(
 			'/<(?:strong|span|p|div)\s(?:[^\s>]*\s+)*?class="(?:[^"\s>]*\s+)*?error(?:\s[^">]*)?"/',
 			$test )
 		) {
-			return $then;
+			$result = $then;
 		} elseif ( $else === false ) {
-			return $test;
+			$result = $test;
 		} else {
-			return $else;
+			$result = $else;
 		}
-	}
-
-	/**
-	 * @param Parser $parser
-	 * @param PPFrame $frame
-	 * @param array $args
-	 * @return string
-	 */
-	public static function iferrorObj( $parser, $frame, $args ) {
-		$test = isset( $args[0] ) ? trim( $frame->expand( $args[0] ) ) : '';
-		$then = $args[1] ?? false;
-		$else = $args[2] ?? false;
-		$result = self::iferror( $parser, $test, $then, $else );
 		if ( $result === false ) {
 			return '';
-		} else {
-			return trim( $frame->expand( $result ) );
 		}
+
+		return trim( $frame->expand( $result ) );
 	}
 
 	/**
+	 * {{#switch: comparison string
+	 * | case = result
+	 * | case = result
+	 * | ...
+	 * | default result
+	 * }}
+	 *
+	 * @link https://www.mediawiki.org/wiki/Help:Extension:ParserFunctions##switch
+	 *
 	 * @param Parser $parser
 	 * @param PPFrame $frame
 	 * @param array $args
 	 * @return string
 	 */
-	public static function switchObj( $parser, $frame, $args ) {
+	public static function switch( Parser $parser, PPFrame $frame, array $args ) {
 		if ( count( $args ) === 0 ) {
 			return '';
 		}
@@ -241,19 +256,23 @@ class ParserFunctions {
 	}
 
 	/**
+	 * {{#rel2abs: path }} or {{#rel2abs: path | base path }}
+	 *
 	 * Returns the absolute path to a subpage, relative to the current article
 	 * title. Treats titles as slash-separated paths.
 	 *
 	 * Following subpage link syntax instead of standard path syntax, an
 	 * initial slash is treated as a relative path, and vice versa.
 	 *
+	 * @link https://www.mediawiki.org/wiki/Help:Extension:ParserFunctions##rel2abs
+	 *
 	 * @param Parser $parser
 	 * @param string $to
 	 * @param string $from
 	 *
 	 * @return string
 	 */
-	public static function rel2abs( $parser, $to = '', $from = '' ) {
+	public static function rel2abs( Parser $parser, $to = '', $from = '' ) {
 		$from = trim( $from );
 		if ( $from === '' ) {
 			$from = $parser->getTitle()->getPrefixedText();
@@ -317,8 +336,8 @@ class ParserFunctions {
 	 *
 	 * @return string
 	 */
-	public static function ifexistCommon(
-		$parser, $frame, $titletext = '', $then = '', $else = ''
+	private static function ifexistInternal(
+		Parser $parser, PPFrame $frame, $titletext = '', $then = '', $else = ''
 	) {
 		$title = Title::newFromText( $titletext );
 		$parser->getContentLanguage()->findVariantLink( $titletext, $title, true );
@@ -377,17 +396,21 @@ class ParserFunctions {
 	}
 
 	/**
+	 * {{#ifexist: page title | value if exists | value if doesn't exist }}
+	 *
+	 * @link https://www.mediawiki.org/wiki/Help:Extension:ParserFunctions##ifexist
+	 *
 	 * @param Parser $parser
 	 * @param PPFrame $frame
 	 * @param array $args
 	 * @return string
 	 */
-	public static function ifexistObj( $parser, $frame, $args ) {
+	public static function ifexist( Parser $parser, PPFrame $frame, array $args ) {
 		$title = isset( $args[0] ) ? trim( $frame->expand( $args[0] ) ) : '';
 		$then = $args[1] ?? null;
 		$else = $args[2] ?? null;
 
-		$result = self::ifexistCommon( $parser, $frame, $title, $then, $else );
+		$result = self::ifexistInternal( $parser, $frame, $title, $then, $else );
 		if ( $result === null ) {
 			return '';
 		} else {
@@ -396,6 +419,8 @@ class ParserFunctions {
 	}
 
 	/**
+	 * Used by time() and localTime()
+	 *
 	 * @param Parser $parser
 	 * @param PPFrame|null $frame
 	 * @param string $format
@@ -404,8 +429,8 @@ class ParserFunctions {
 	 * @param string|bool $local
 	 * @return string
 	 */
-	public static function timeCommon(
-		$parser, $frame = null, $format = '', $date = '', $language = '', $local = false
+	private static function timeCommon(
+		Parser $parser, PPFrame $frame = null, $format = '', $date = '', $language = '', $local = false
 	) {
 		global $wgLocaltimezone;
 		self::registerClearHook();
@@ -420,9 +445,7 @@ class ParserFunctions {
 		}
 		if ( isset( self::$mTimeCache[$format][$cacheKey][$language][$local] ) ) {
 			$cachedVal = self::$mTimeCache[$format][$cacheKey][$language][$local];
-			if ( $useTTL
-				&& $cachedVal[1] !== null && $frame && is_callable( [ $frame, 'setTTL' ] )
-			) {
+			if ( $useTTL && $cachedVal[1] !== null && $frame ) {
 				$frame->setTTL( $cachedVal[1] );
 			}
 			return $cachedVal[0];
@@ -504,33 +527,25 @@ class ParserFunctions {
 			}
 		}
 		self::$mTimeCache[$format][$cacheKey][$language][$local] = [ $result, $ttl ];
-		if ( $useTTL && $ttl !== null && $frame && is_callable( [ $frame, 'setTTL' ] ) ) {
+		if ( $useTTL && $ttl !== null && $frame ) {
 			$frame->setTTL( $ttl );
 		}
 		return $result;
 	}
 
 	/**
-	 * @param Parser $parser
-	 * @param string $format
-	 * @param string $date
-	 * @param string $language
-	 * @param string|bool $local
-	 * @return string
-	 */
-	public static function time(
-		$parser, $format = '', $date = '', $language = '', $local = false
-	) {
-		return self::timeCommon( $parser, null, $format, $date, $language, $local );
-	}
-
-	/**
+	 * {{#time: format string }}
+	 * {{#time: format string | date/time object }}
+	 * {{#time: format string | date/time object | language code }}
+	 *
+	 * @link https://www.mediawiki.org/wiki/Help:Extension:ParserFunctions##time
+	 *
 	 * @param Parser $parser
 	 * @param PPFrame $frame
 	 * @param array $args
 	 * @return string
 	 */
-	public static function timeObj( $parser, $frame, $args ) {
+	public static function time( Parser $parser, PPFrame $frame, array $args ) {
 		$format = isset( $args[0] ) ? trim( $frame->expand( $args[0] ) ) : '';
 		$date = isset( $args[1] ) ? trim( $frame->expand( $args[1] ) ) : '';
 		$language = isset( $args[2] ) ? trim( $frame->expand( $args[2] ) ) : '';
@@ -539,23 +554,19 @@ class ParserFunctions {
 	}
 
 	/**
-	 * @param Parser $parser
-	 * @param string $format
-	 * @param string $date
-	 * @param string $language
-	 * @return string
-	 */
-	public static function localTime( $parser, $format = '', $date = '', $language = '' ) {
-		return self::timeCommon( $parser, null, $format, $date, $language, true );
-	}
-
-	/**
+	 * {{#timel: ... }}
+	 *
+	 * Identical to {{#time: ... }}, except that it uses the local time of the wiki
+	 * (as set in $wgLocaltimezone) when no date is given.
+	 *
+	 * @link https://www.mediawiki.org/wiki/Help:Extension:ParserFunctions##timel
+	 *
 	 * @param Parser $parser
 	 * @param PPFrame $frame
 	 * @param array $args
 	 * @return string
 	 */
-	public static function localTimeObj( $parser, $frame, $args ) {
+	public static function localTime( Parser $parser, PPFrame $frame, array $args ) {
 		$format = isset( $args[0] ) ? trim( $frame->expand( $args[0] ) ) : '';
 		$date = isset( $args[1] ) ? trim( $frame->expand( $args[1] ) ) : '';
 		$language = isset( $args[2] ) ? trim( $frame->expand( $args[2] ) ) : '';
@@ -566,13 +577,15 @@ class ParserFunctions {
 	 * Obtain a specified number of slash-separated parts of a title,
 	 * e.g. {{#titleparts:Hello/World|1}} => "Hello"
 	 *
+	 * @link https://www.mediawiki.org/wiki/Help:Extension:ParserFunctions##titleparts
+	 *
 	 * @param Parser $parser Parent parser
 	 * @param string $title Title to split
 	 * @param int $parts Number of parts to keep
 	 * @param int $offset Offset starting at 1
 	 * @return string
 	 */
-	public static function titleparts( $parser, $title = '', $parts = 0, $offset = 0 ) {
+	public static function titleparts( Parser $parser, $title = '', $parts = 0, $offset = 0 ) {
 		$parts = (int)$parts;
 		$offset = (int)$offset;
 		$ntitle = Title::newFromText( $title );
@@ -596,7 +609,8 @@ class ParserFunctions {
 	}
 
 	/**
-	 *  Verifies parameter is less than max string length.
+	 * Verifies parameter is less than max string length.
+	 *
 	 * @param string $text
 	 * @return bool
 	 */
@@ -606,7 +620,7 @@ class ParserFunctions {
 	}
 
 	/**
-	 * Generates error message.  Called when string is too long.
+	 * Generates error message. Called when string is too long.
 	 * @return string
 	 */
 	private static function tooLongError() {
@@ -619,11 +633,12 @@ class ParserFunctions {
 	 * {{#len:string}}
 	 *
 	 * Reports number of characters in string.
+	 *
 	 * @param Parser $parser
 	 * @param string $inStr
 	 * @return int
 	 */
-	public static function runLen( $parser, $inStr = '' ) {
+	public static function runLen( Parser $parser, $inStr = '' ) {
 		$inStr = $parser->killMarkers( (string)$inStr );
 		return mb_strlen( $inStr );
 	}
@@ -641,7 +656,7 @@ class ParserFunctions {
 	 * @param int $inOffset
 	 * @return int|string
 	 */
-	public static function runPos( $parser, $inStr = '', $inNeedle = '', $inOffset = 0 ) {
+	public static function runPos( Parser $parser, $inStr = '', $inNeedle = '', $inOffset = 0 ) {
 		$inStr = $parser->killMarkers( (string)$inStr );
 		$inNeedle = $parser->killMarkers( (string)$inNeedle );
 
@@ -674,7 +689,7 @@ class ParserFunctions {
 	 * @param int|string $inNeedle
 	 * @return int|string
 	 */
-	public static function runRPos( $parser, $inStr = '', $inNeedle = '' ) {
+	public static function runRPos( Parser $parser, $inStr = '', $inNeedle = '' ) {
 		$inStr = $parser->killMarkers( (string)$inStr );
 		$inNeedle = $parser->killMarkers( (string)$inNeedle );
 
@@ -713,7 +728,7 @@ class ParserFunctions {
 	 * @param int $inLength
 	 * @return string
 	 */
-	public static function runSub( $parser, $inStr = '', $inStart = 0, $inLength = 0 ) {
+	public static function runSub( Parser $parser, $inStr = '', $inStart = 0, $inLength = 0 ) {
 		$inStr = $parser->killMarkers( (string)$inStr );
 
 		if ( !self::checkLength( $inStr ) ) {
@@ -735,12 +750,13 @@ class ParserFunctions {
 	 * Returns number of occurrences of "substr" in "string".
 	 *
 	 * Note: If "substr" is empty, a single space is used.
+	 *
 	 * @param Parser $parser
 	 * @param string $inStr
 	 * @param string $inSubStr
 	 * @return int|string
 	 */
-	public static function runCount( $parser, $inStr = '', $inSubStr = '' ) {
+	public static function runCount( Parser $parser, $inStr = '', $inSubStr = '' ) {
 		$inStr = $parser->killMarkers( (string)$inStr );
 		$inSubStr = $parser->killMarkers( (string)$inSubStr );
 
@@ -766,6 +782,7 @@ class ParserFunctions {
 	 *
 	 * Note: Armored against replacements that would generate huge strings.
 	 * Note: If "from" is an empty string, single space is used instead.
+	 *
 	 * @param Parser $parser
 	 * @param string $inStr
 	 * @param string $inReplaceFrom
@@ -773,7 +790,7 @@ class ParserFunctions {
 	 * @param int $inLimit
 	 * @return mixed|string
 	 */
-	public static function runReplace( $parser, $inStr = '',
+	public static function runReplace( Parser $parser, $inStr = '',
 			$inReplaceFrom = '', $inReplaceTo = '', $inLimit = -1 ) {
 		global $wgPFStringLengthLimit;
 
@@ -829,6 +846,7 @@ class ParserFunctions {
 	 * Note: Negative position can be used to specify tokens from the end.
 	 * Note: If the divider is an empty string, single space is used instead.
 	 * Note: Empty string is returned if there are not enough exploded chunks.
+	 *
 	 * @param Parser $parser
 	 * @param string $inStr
 	 * @param string $inDiv
@@ -837,7 +855,7 @@ class ParserFunctions {
 	 * @return string
 	 */
 	public static function runExplode(
-		$parser, $inStr = '', $inDiv = '', $inPos = 0, $inLim = null
+		Parser $parser, $inStr = '', $inDiv = '', $inPos = 0, $inLim = null
 	) {
 		$inStr = $parser->killMarkers( (string)$inStr );
 		$inDiv = $parser->killMarkers( (string)$inDiv );
@@ -870,11 +888,12 @@ class ParserFunctions {
 	 * {{#urldecode:string}}
 	 *
 	 * Decodes URL-encoded (like%20that) strings.
+	 *
 	 * @param Parser $parser
 	 * @param string $inStr
 	 * @return string
 	 */
-	public static function runUrlDecode( $parser, $inStr = '' ) {
+	public static function runUrlDecode( Parser $parser, $inStr = '' ) {
 		$inStr = $parser->killMarkers( (string)$inStr );
 		if ( !self::checkLength( $inStr ) ) {
 			return self::tooLongError();
@@ -895,7 +914,7 @@ class ParserFunctions {
 	 *   but with char refs intact
 	 * @return string The trimmed, expanded and entity reference decoded version of the PPNode
 	 */
-	private static function decodeTrimExpand( $obj, $frame, &$trimExpanded = null ) {
+	private static function decodeTrimExpand( $obj, PPFrame $frame, &$trimExpanded = null ) {
 		$expanded = $frame->expand( $obj );
 		$trimExpanded = trim( $expanded );
 		return trim( Sanitizer::decodeCharReferences( $expanded ) );
