@@ -9,6 +9,10 @@ module.exports = function ( grunt ) {
 
 	grunt.initConfig( {
 		eslint: {
+			options: {
+				cache: true,
+				reportUnusedDisableDirectives: true
+			},
 			all: '.'
 		},
 		stylelint: {
