@@ -19,10 +19,11 @@ class TitleBlacklistHooks {
 	 * @param Title $title
 	 * @param User $user
 	 * @param string $action
-	 * @param array &$result
+	 * @param array|IApiMessage &$result
+	 *
 	 * @return bool
 	 */
-	public static function userCan( $title, $user, $action, &$result ) {
+	public static function onUserCan( $title, $user, $action, &$result ) {
 		# Some places check createpage, while others check create.
 		# As it stands, upload does createpage, but normalize both
 		# to the same action, to stop future similar bugs.
@@ -71,11 +72,10 @@ class TitleBlacklistHooks {
 	 * @param Title $title
 	 * @param int $oldid
 	 * @param array &$notices
-	 * @return true
 	 */
-	public static function displayBlacklistOverrideNotice( Title $title, $oldid, array &$notices ) {
+	public static function onDisplayBlacklistOverrideNotice( Title $title, $oldid, array &$notices ) {
 		if ( !RequestContext::getMain()->getUser()->isAllowed( 'tboverride' ) ) {
-			return true;
+			return;
 		}
 
 		$blacklisted = TitleBlacklist::singleton()->isBlacklisted(
@@ -83,18 +83,17 @@ class TitleBlacklistHooks {
 			$title->exists() ? 'edit' : 'create'
 		);
 		if ( !$blacklisted ) {
-			return true;
+			return;
 		}
 
 		$params = $blacklisted->getParams();
 		if ( isset( $params['autoconfirmed'] ) ) {
-			return true;
+			return;
 		}
 
 		$msg = wfMessage( 'titleblacklist-warning' );
-		$notices['titleblacklist'] = $msg->plaintextParams(
-			$blacklisted->getRaw() )->parseAsBlock();
-		return true;
+		$notices['titleblacklist'] = $msg->plaintextParams( $blacklisted->getRaw() )
+			->parseAsBlock();
 	}
 
 	/**
@@ -105,6 +104,7 @@ class TitleBlacklistHooks {
 	 * @param User $user
 	 * @param string $reason
 	 * @param Status $status
+	 *
 	 * @return bool
 	 */
 	public static function onMovePageCheckPermissions(
@@ -136,6 +136,7 @@ class TitleBlacklistHooks {
 	 * @param User $creatingUser
 	 * @param bool $override Should the test be skipped, if the user has sufficient privileges?
 	 * @param bool $log Log blacklist hits to Special:Log
+	 *
 	 * @return StatusValue
 	 */
 	public static function testUserName(
@@ -178,17 +179,16 @@ class TitleBlacklistHooks {
 	 * @param string $text
 	 * @param string $section
 	 * @param string &$error
-	 * @return true
 	 */
-	public static function validateBlacklist( $editor, $text, $section, &$error ) {
+	public static function onValidateBlacklist( $editor, $text, $section, &$error ) {
 		$title = $editor->getTitle();
 
 		if ( $title->getNamespace() == NS_MEDIAWIKI && $title->getDBkey() == 'Titleblacklist' ) {
 			$blackList = TitleBlacklist::singleton();
-			$bl = $blackList->parseBlacklist( $text, 'page' );
+			$bl = TitleBlacklist::parseBlacklist( $text, 'page' );
 			$ok = $blackList->validate( $bl );
-			if ( count( $ok ) == 0 ) {
-				return true;
+			if ( $ok === [] ) {
+				return;
 			}
 
 			$errmsg = wfMessage( 'titleblacklist-invalid' )->numParams( count( $ok ) )->text();
@@ -204,31 +204,26 @@ class TitleBlacklistHooks {
 
 			// $error will be displayed by the edit class
 		}
-		return true;
 	}
 
 	/**
 	 * PageContentSaveComplete hook
 	 *
-	 * @param WikiPage &$wikiPage
+	 * @param WikiPage $wikiPage
 	 * @param User &$user
 	 * @param Content $content
 	 * @param string $summary
 	 * @param bool $isminor
 	 * @param bool $iswatch
 	 * @param string $section
-	 *
-	 * @return true
 	 */
-	public static function clearBlacklist(
-		WikiPage &$wikiPage, &$user,
-		$content, $summary, $isminor, $iswatch, $section
+	public static function onClearBlacklist(
+		WikiPage $wikiPage, &$user, $content, $summary, $isminor, $iswatch, $section
 	) {
 		$title = $wikiPage->getTitle();
-		if ( $title->getNamespace() == NS_MEDIAWIKI && $title->getDBkey() == 'Titleblacklist' ) {
+		if ( $title->getNamespace() === NS_MEDIAWIKI && $title->getDBkey() == 'Titleblacklist' ) {
 			TitleBlacklist::singleton()->invalidate();
 		}
-		return true;
 	}
 
 	/**
@@ -258,12 +253,10 @@ class TitleBlacklistHooks {
 	 *
 	 * @param string $engine
 	 * @param array &$extraLibraries
-	 * @return bool
 	 */
-	public static function scribuntoExternalLibraries( $engine, array &$extraLibraries ) {
+	public static function onScribuntoExternalLibraries( $engine, array &$extraLibraries ) {
 		if ( $engine == 'lua' ) {
 			$extraLibraries['mw.ext.TitleBlacklist'] = 'Scribunto_LuaTitleBlacklistLibrary';
 		}
-		return true;
 	}
 }
