@@ -1,9 +1,9 @@
 /**
  * Toolbar module for wikiEditor
  */
-( function ( $, mw ) {
+( function () {
 
-	$.wikiEditor.modules.toolbar = {
+	var toolbarModule = {
 
 		/**
 		 * API accessible functions
@@ -12,9 +12,7 @@
 			addToToolbar: function ( context, data ) {
 
 				var section, type, i, group, $group, $section, $sections,
-					$tabs, tool, $pages, $index, page, $table, $characters, actions,
-					$divSections, $visibleSection,
-					smooth = true;
+					$tabs, tool, $pages, $index, page, $table, $characters, actions;
 
 				for ( type in data ) {
 					switch ( type ) {
@@ -25,7 +23,7 @@
 								if ( section === 'main' ) {
 									// Section
 									context.modules.toolbar.$toolbar.prepend(
-										$.wikiEditor.modules.toolbar.fn.buildSection(
+										toolbarModule.fn.buildSection(
 											context, section, data[ type ][ section ]
 										)
 									);
@@ -33,11 +31,11 @@
 								}
 								// Section
 								$sections.append(
-									$.wikiEditor.modules.toolbar.fn.buildSection( context, section, data[ type ][ section ] )
+									toolbarModule.fn.buildSection( context, section, data[ type ][ section ] )
 								);
 								// Tab
 								$tabs.append(
-									$.wikiEditor.modules.toolbar.fn.buildTab( context, section, data[ type ][ section ] )
+									toolbarModule.fn.buildTab( context, section, data[ type ][ section ] )
 								);
 							}
 							break;
@@ -49,10 +47,9 @@
 							for ( group in data[ type ] ) {
 								// Group
 								$section.append(
-									$.wikiEditor.modules.toolbar.fn.buildGroup( context, group, data[ type ][ group ] )
+									toolbarModule.fn.buildGroup( context, group, data[ type ][ group ] )
 								);
 							}
-							smooth = false;
 							break;
 						case 'tools':
 							if ( !( 'section' in data && 'group' in data ) ) {
@@ -64,12 +61,11 @@
 							);
 							for ( tool in data[ type ] ) {
 								// Tool
-								$group.append( $.wikiEditor.modules.toolbar.fn.buildTool( context, tool, data[ type ][ tool ] ) );
+								$group.append( toolbarModule.fn.buildTool( context, tool, data[ type ][ tool ] ) );
 							}
 							if ( $group.children().length ) {
 								$group.removeClass( 'empty' );
 							}
-							smooth = false;
 							break;
 						case 'pages':
 							if ( !( 'section' in data ) ) {
@@ -83,14 +79,13 @@
 							);
 							for ( page in data[ type ] ) {
 								// Page
-								$pages.append( $.wikiEditor.modules.toolbar.fn.buildPage( context, page, data[ type ][ page ] ) );
+								$pages.append( toolbarModule.fn.buildPage( context, page, data[ type ][ page ] ) );
 								// Index
 								$index.append(
-									$.wikiEditor.modules.toolbar.fn.buildBookmark( context, page, data[ type ][ page ] )
+									toolbarModule.fn.buildBookmark( context, page, data[ type ][ page ] )
 								);
 							}
-							$.wikiEditor.modules.toolbar.fn.updateBookletSelection( context, data.section, $pages, $index );
-							smooth = false;
+							toolbarModule.fn.updateBookletSelection( context, data.section, $pages, $index );
 							break;
 						case 'rows':
 							if ( !( 'section' in data && 'page' in data ) ) {
@@ -102,9 +97,8 @@
 							);
 							for ( i = 0; i < data.rows.length; i++ ) {
 								// Row
-								$table.append( $.wikiEditor.modules.toolbar.fn.buildRow( context, data.rows[ i ] ) );
+								$table.append( toolbarModule.fn.buildRow( context, data.rows[ i ] ) );
 							}
-							smooth = false;
 							break;
 						case 'characters':
 							if ( !( 'section' in data && 'page' in data ) ) {
@@ -118,37 +112,24 @@
 							for ( i = 0; i < data.characters.length; i++ ) {
 								// Character
 								$characters.append(
-									$( $.wikiEditor.modules.toolbar.fn.buildCharacter( data.characters[ i ], actions ) )
-										.mousedown( function ( e ) {
+									$( toolbarModule.fn.buildCharacter( data.characters[ i ], actions ) )
+										.on( 'mousedown', function ( e ) {
 											// No dragging!
 											e.preventDefault();
 											return false;
 										} )
-										.click( function ( e ) {
-											$.wikiEditor.modules.toolbar.fn.doAction( $( this ).parent().data( 'context' ),
+										.on( 'click', function ( e ) {
+											toolbarModule.fn.doAction( $( this ).parent().data( 'context' ),
 												$( this ).parent().data( 'actions' )[ $( this ).attr( 'rel' ) ] );
 											e.preventDefault();
 											return false;
 										} )
 								);
 							}
-							smooth = false;
 							break;
 						default: break;
 					}
 				}
-
-				// Fix div.section size after adding things; if smooth is true uses a smooth
-				// animation, otherwise just change height (breaking any ongoing animation)
-				$divSections = context.modules.toolbar.$toolbar.find( 'div.sections' );
-				$visibleSection = $divSections.find( '.section-visible' );
-				if ( $visibleSection.length ) {
-					if ( smooth ) {
-						$divSections.animate( { height: $visibleSection.outerHeight() }, 'fast' );
-					} else {
-						$divSections.height( $visibleSection.outerHeight() );
-					}
-				}
 			},
 			removeFromToolbar: function ( context, data ) {
 				var index, $group, tab, target, group;
@@ -179,7 +160,7 @@
 						} else {
 							// Just a page, remove the index too!
 							context.modules.toolbar.$toolbar.find( index ).remove();
-							$.wikiEditor.modules.toolbar.fn.updateBookletSelection(
+							toolbarModule.fn.updateBookletSelection(
 								context,
 								data.section,
 								context.modules.toolbar.$toolbar.find( target ),
@@ -202,18 +183,6 @@
 			}
 		},
 
-		/**
-		 * Event handlers
-		 */
-		evt: {
-			/**
-			 * @param {Object} context
-			 */
-			resize: function ( context ) {
-				context.$ui.find( '.sections' ).height( context.$ui.find( '.sections .section-visible' ).outerHeight() );
-			}
-		},
-
 		/**
 		 * Internally used functions
 		 */
@@ -231,7 +200,7 @@
 				context.modules.toolbar.$toolbar = $( '<div>' )
 					.addClass( 'wikiEditor-ui-toolbar' )
 					.attr( 'id', 'wikiEditor-ui-toolbar' );
-				$.wikiEditor.modules.toolbar.fn.build( context, config );
+				toolbarModule.fn.build( context, config );
 				context.$ui.find( '.wikiEditor-ui-top' ).append( context.modules.toolbar.$toolbar );
 			},
 			/**
@@ -245,6 +214,9 @@
 				switch ( action.type ) {
 					case 'replace':
 					case 'encapsulate':
+						if ( context.$textarea.prop( 'readonly' ) ) {
+							break;
+						}
 						parts = {
 							pre: $.wikiEditor.autoMsg( action.options, 'pre' ),
 							peri: $.wikiEditor.autoMsg( action.options, 'peri' ),
@@ -279,7 +251,7 @@
 			},
 			buildGroup: function ( context, id, group ) {
 				var $label, empty, tool,
-					$group = $( '<div>' ).attr( { 'class': 'group group-' + id, rel: id } ),
+					$group = $( '<div>' ).attr( { class: 'group group-' + id, rel: id } ),
 					label = $.wikiEditor.autoMsg( group, 'label' );
 				if ( label ) {
 					$label = $( '<span>' )
@@ -290,7 +262,7 @@
 				empty = true;
 				if ( 'tools' in group ) {
 					for ( tool in group.tools ) {
-						tool = $.wikiEditor.modules.toolbar.fn.buildTool( context, tool, group.tools[ tool ] );
+						tool = toolbarModule.fn.buildTool( context, tool, group.tools[ tool ] );
 						if ( tool ) {
 							// Consider a group with only hidden tools empty as well
 							// .is( ':visible' ) always returns false because tool is not attached to the DOM yet
@@ -340,7 +312,7 @@
 									title: label,
 									rel: id,
 									role: 'button',
-									'class': 'tool tool-button'
+									class: 'tool tool-button'
 								} )
 								.text( label );
 							if ( tool.icon ) {
@@ -363,20 +335,20 @@
 							$button
 								.data( 'action', tool.action )
 								.data( 'context', context )
-								.mousedown( function ( e ) {
+								.on( 'mousedown', function ( e ) {
 									// No dragging!
 									e.preventDefault();
 									return false;
 								} );
 							if ( $button.data( 'ooui' ) ) {
 								$button.data( 'ooui' ).on( 'click', function () {
-									$.wikiEditor.modules.toolbar.fn.doAction(
+									toolbarModule.fn.doAction(
 										context, tool.action
 									);
 								} );
 							} else {
 								$button.on( 'click', function ( e ) {
-									$.wikiEditor.modules.toolbar.fn.doAction(
+									toolbarModule.fn.doAction(
 										context, tool.action
 									);
 									e.preventDefault();
@@ -387,7 +359,7 @@
 						return $button;
 					case 'select':
 						$select = $( '<div>' )
-							.attr( { rel: id, 'class': 'tool tool-select' } );
+							.attr( { rel: id, class: 'tool tool-select' } );
 						$options = $( '<div>' ).addClass( 'options' );
 						if ( 'list' in tool ) {
 							for ( option in tool.list ) {
@@ -396,21 +368,17 @@
 									$( '<a>' )
 										.data( 'action', tool.list[ option ].action )
 										.data( 'context', context )
-										.mousedown( function ( e ) {
+										.on( 'mousedown', function ( e ) {
 											// No dragging!
 											e.preventDefault();
 											return false;
 										} )
-										.click( function ( e ) {
-											$.wikiEditor.modules.toolbar.fn.doAction(
+										.on( 'click', function ( e ) {
+											toolbarModule.fn.doAction(
 												$( this ).data( 'context' ), $( this ).data( 'action' ), $( this )
 											);
 											// Hide the dropdown
-											// Sanity check: if this somehow gets called while the dropdown
-											// is hidden, don't show it
-											if ( $( this ).parent().is( ':visible' ) ) {
-												$( this ).parent().animate( { opacity: 'toggle' }, 'fast' );
-											}
+											$( this ).closest( '.tool-select' ).removeClass( 'options-shown' );
 											e.preventDefault();
 											return false;
 										} )
@@ -420,23 +388,23 @@
 								);
 							}
 						}
-						$select.append( $( '<div>' ).addClass( 'menu' ).append( $options ) );
 						$select.append( $( '<a>' )
 							.addClass( 'label' )
 							.text( label )
 							.data( 'options', $options )
 							.attr( 'href', '#' )
-							.mousedown( function ( e ) {
+							.on( 'mousedown', function ( e ) {
 								// No dragging!
 								e.preventDefault();
 								return false;
 							} )
-							.click( function ( e ) {
-								$( this ).data( 'options' ).animate( { opacity: 'toggle' }, 'fast' );
+							.on( 'click', function ( e ) {
+								$( this ).data( 'options' ).closest( '.tool-select' ).toggleClass( 'options-shown' );
 								e.preventDefault();
 								return false;
 							} )
 						);
+						$select.append( $( '<div>' ).addClass( 'menu' ).append( $options ) );
 						return $select;
 					default:
 						return null;
@@ -448,12 +416,12 @@
 					.text( label )
 					.attr( 'rel', id )
 					.data( 'context', context )
-					.mousedown( function ( e ) {
+					.on( 'mousedown', function ( e ) {
 						// No dragging!
 						e.preventDefault();
 						return false;
 					} )
-					.click( function ( event ) {
+					.on( 'click', function ( event ) {
 						var section;
 						$( this ).parent().parent().find( '.page' ).hide();
 						$( this ).parent().parent().find( '.page-' + $( this ).attr( 'rel' ) ).show().trigger( 'loadPage' );
@@ -472,15 +440,15 @@
 			},
 			buildPage: function ( context, id, page, deferLoad ) {
 				var $page = $( '<div>' ).attr( {
-					'class': 'page page-' + id,
+					class: 'page page-' + id,
 					rel: id
 				} );
 				if ( deferLoad ) {
 					$page.one( 'loadPage', function () {
-						$.wikiEditor.modules.toolbar.fn.reallyBuildPage( context, id, page, $page );
+						toolbarModule.fn.reallyBuildPage( context, id, page, $page );
 					} );
 				} else {
-					$.wikiEditor.modules.toolbar.fn.reallyBuildPage( context, id, page, $page );
+					toolbarModule.fn.reallyBuildPage( context, id, page, $page );
 				}
 				return $page;
 			},
@@ -492,11 +460,11 @@
 						html =
 							'<table class="table-' + id + '">';
 						if ( 'headings' in page ) {
-							html += $.wikiEditor.modules.toolbar.fn.buildHeading( context, page.headings );
+							html += toolbarModule.fn.buildHeading( context, page.headings );
 						}
 						if ( 'rows' in page ) {
 							for ( i = 0; i < page.rows.length; i++ ) {
-								html += $.wikiEditor.modules.toolbar.fn.buildRow( context, page.rows[ i ] );
+								html += toolbarModule.fn.buildRow( context, page.rows[ i ] );
 							}
 						}
 						$page.html( html + '</table>' );
@@ -519,18 +487,18 @@
 						if ( 'characters' in page ) {
 							html = '';
 							for ( i = 0; i < page.characters.length; i++ ) {
-								html += $.wikiEditor.modules.toolbar.fn.buildCharacter( page.characters[ i ], actions );
+								html += toolbarModule.fn.buildCharacter( page.characters[ i ], actions );
 							}
 							$characters
 								.html( html )
 								.children()
-								.mousedown( function ( e ) {
+								.on( 'mousedown', function ( e ) {
 									// No dragging!
 									e.preventDefault();
 									return false;
 								} )
-								.click( function ( e ) {
-									$.wikiEditor.modules.toolbar.fn.doAction(
+								.on( 'click', function ( e ) {
+									toolbarModule.fn.doAction(
 										$( this ).parent().data( 'context' ),
 										$( this ).parent().data( 'actions' )[ $( this ).attr( 'rel' ) ],
 										$( this )
@@ -621,15 +589,15 @@
 						} )
 						.text( $.wikiEditor.autoMsg( section, 'label' ) )
 						.data( 'context', context )
-						.mouseup( function () {
-							$( this ).blur();
+						.on( 'mouseup', function () {
+							$( this ).trigger( 'blur' );
 						} )
-						.mousedown( function ( e ) {
+						.on( 'mousedown', function ( e ) {
 							// No dragging!
 							e.preventDefault();
 							return false;
 						} )
-						.click( function ( e ) {
+						.on( 'click', function ( e ) {
 							// We have to set aria-pressed over here, as NVDA wont recognize it
 							// if we do it in the below .each as it seems
 							$( this ).attr( 'aria-pressed', 'true' );
@@ -639,55 +607,35 @@
 								}
 							} );
 							$sections = $( this ).data( 'context' ).$ui.find( '.sections' );
-							$section =
-								$( this ).data( 'context' ).$ui.find( '.section-' + $( this ).parent().attr( 'rel' ) );
+							$section = $sections.find( '.section-' + $( this ).parent().attr( 'rel' ) );
 							show = !$section.hasClass( 'section-visible' );
-							$section.parent().find( '.section-visible' )
-								.css( 'position', 'absolute' )
+							$sections.find( '.section-visible' )
 								.attr( 'aria-expanded', 'false' )
 								.removeClass( 'section-visible' )
-								.animate( { opacity: 0 }, 'fast', 'linear', function () {
-									$( this ).addClass( 'section-hidden' ).css( 'position', 'static' );
-								} );
+								.addClass( 'section-hidden' );
 
 							$( this ).parent().parent().find( 'a' ).removeClass( 'current' );
-							$sections.css( 'overflow', 'hidden' );
 							if ( show ) {
 								$section
-									.stop()
 									.removeClass( 'section-hidden' )
 									.attr( 'aria-expanded', 'true' )
-									.animate( { opacity: 100.0 }, 'fast', 'linear', function () {
-										$( this ).addClass( 'section-visible' );
-										context.fn.trigger( 'resize' );
-									} );
-								$sections
-									.animate( { height: $section.outerHeight() }, $section.outerHeight() * 2, function () {
-										$( this ).css( 'overflow', 'visible' ).css( 'height', 'auto' );
-										context.fn.trigger( 'resize' );
-									} );
+									.addClass( 'section-visible' );
 								$( this ).addClass( 'current' );
-							} else {
-								$sections
-									.stop()
-									.css( 'height', $section.outerHeight() )
-									.animate( { height: 0 }, $section.outerHeight() * 2, function () {
-										$( this ).css( { overflow: 'visible' } );
-										context.fn.trigger( 'resize' );
-									} );
 							}
+
 							// Save the currently visible section
 							$.cookie(
 								'wikiEditor-' + $( this ).data( 'context' ).instance + '-toolbar-section',
 								show ? $section.attr( 'rel' ) : null,
 								{ expires: 30, path: '/' }
 							);
+
 							e.preventDefault();
 							return false;
 						} );
 				return $( '<span>' )
 					.attr( {
-						'class': 'tab tab-' + id,
+						class: 'tab tab-' + id,
 						rel: id
 					} )
 					.append( $link );
@@ -695,14 +643,14 @@
 			buildSection: function ( context, id, section ) {
 				var selected, show,
 					$section = $( '<div>' ).attr( {
-						'class': section.type + ' section section-' + id,
+						class: section.type + ' section section-' + id,
 						rel: id,
 						id: 'wikiEditor-section-' + id
 					} );
 				selected = $.cookie( 'wikiEditor-' + context.instance + '-toolbar-section' );
 				show = selected === id;
 
-				$.wikiEditor.modules.toolbar.fn.reallyBuildSection( context, id, section, $section, section.deferLoad );
+				toolbarModule.fn.reallyBuildSection( context, id, section, $section, section.deferLoad );
 
 				// Show or hide section
 				if ( id !== 'main' ) {
@@ -724,7 +672,7 @@
 						if ( 'groups' in section ) {
 							for ( group in section.groups ) {
 								$section.append(
-									$.wikiEditor.modules.toolbar.fn.buildGroup( context, group, section.groups[ group ] )
+									toolbarModule.fn.buildGroup( context, group, section.groups[ group ] )
 								);
 							}
 						}
@@ -735,15 +683,15 @@
 						if ( 'pages' in section ) {
 							for ( page in section.pages ) {
 								$pages.append(
-									$.wikiEditor.modules.toolbar.fn.buildPage( context, page, section.pages[ page ], deferLoad )
+									toolbarModule.fn.buildPage( context, page, section.pages[ page ], deferLoad )
 								);
 								$index.append(
-									$.wikiEditor.modules.toolbar.fn.buildBookmark( context, page, section.pages[ page ] )
+									toolbarModule.fn.buildBookmark( context, page, section.pages[ page ] )
 								);
 							}
 						}
 						$section.append( $index ).append( $pages );
-						$.wikiEditor.modules.toolbar.fn.updateBookletSelection( context, id, $pages, $index );
+						toolbarModule.fn.updateBookletSelection( context, id, $pages, $index );
 						break;
 				}
 			},
@@ -766,45 +714,28 @@
 				$selectedIndex.addClass( 'current' );
 			},
 			build: function ( context, config ) {
-				var section, sectionQueue,
+				var section,
 					$tabs = $( '<div>' ).addClass( 'tabs' ).appendTo( context.modules.toolbar.$toolbar ),
 					$sections = $( '<div>' ).addClass( 'sections' ).appendTo( context.modules.toolbar.$toolbar );
 				context.modules.toolbar.$toolbar.append( $( '<div>' ).css( 'clear', 'both' ) );
-				sectionQueue = [];
 				for ( section in config ) {
 					if ( section === 'main' ) {
 						context.modules.toolbar.$toolbar.prepend(
-							$.wikiEditor.modules.toolbar.fn.buildSection( context, section, config[ section ] )
+							toolbarModule.fn.buildSection( context, section, config[ section ] )
 						);
 					} else {
-						sectionQueue.push( {
-							$sections: $sections,
-							context: context,
-							id: section,
-							config: config[ section ]
-						} );
-						$tabs.append( $.wikiEditor.modules.toolbar.fn.buildTab( context, section, config[ section ] ) );
+						$sections.append( toolbarModule.fn.buildSection( context, section, config[ section ] ) );
+						$tabs.append( toolbarModule.fn.buildTab( context, section, config[ section ] ) );
 					}
 				}
-				$.eachAsync( sectionQueue, {
-					bulk: 0,
-					end: function () {
-						context.$textarea.trigger( 'wikiEditor-toolbar-doneInitialSections' );
-					},
-					loop: function ( i, s ) {
-						var $section;
-						s.$sections.append( $.wikiEditor.modules.toolbar.fn.buildSection( s.context, s.id, s.config ) );
-						$section = s.$sections.find( '.section-visible' );
-						if ( $section.length ) {
-							$sections.animate( { height: $section.outerHeight() }, $section.outerHeight() * 2, function () {
-								context.fn.trigger( 'resize' );
-							} );
-						}
-					}
+				setTimeout( function () {
+					context.$textarea.trigger( 'wikiEditor-toolbar-doneInitialSections' );
 				} );
 			}
 		}
 
 	};
 
-}( jQuery, mediaWiki ) );
+	module.exports = toolbarModule;
+
+}() );
