@@ -21,6 +21,7 @@
  */
 use MediaWiki\Linker\LinkTarget;
 use MediaWiki\MediaWikiServices;
+use MediaWiki\Revision\RevisionRecord;
 
 /**
  * Some internal bits split of from Skin.php. These functions are used
@@ -41,12 +42,12 @@ class Linker {
 	/**
 	 * This function returns an HTML link to the given target.  It serves a few
 	 * purposes:
-	 *   1) If $target is a Title, the correct URL to link to will be figured
+	 *   1) If $target is a LinkTarget, the correct URL to link to will be figured
 	 *      out automatically.
 	 *   2) It automatically adds the usual classes for various types of link
 	 *      targets: "new" for red links, "stub" for short articles, etc.
 	 *   3) It escapes all attribute values safely so there's no risk of XSS.
-	 *   4) It provides a default tooltip if the target is a Title (the page
+	 *   4) It provides a default tooltip if the target is a LinkTarget (the page
 	 *      name of the target).
 	 * link() replaces the old functions in the makeLink() family.
 	 *
@@ -57,7 +58,7 @@ class Linker {
 	 *   change to support Images, literal URLs, etc.
 	 * @param string $html The HTML contents of the <a> element, i.e.,
 	 *   the link text.  This is raw HTML and will not be escaped.  If null,
-	 *   defaults to the prefixed text of the Title; or if the Title is just a
+	 *   defaults to the prefixed text of the LinkTarget; or if the LinkTarget is just a
 	 *   fragment, the contents of the fragment.
 	 * @param array $customAttribs A key => value array of extra HTML attributes,
 	 *   such as title and class.  (href is ignored.)  Classes will be
@@ -89,12 +90,6 @@ class Linker {
 			return "<!-- ERROR -->$html";
 		}
 
-		if ( is_string( $query ) ) {
-			// some functions withing core using this still hand over query strings
-			wfDeprecated( __METHOD__ . ' with parameter $query as string (should be array)', '1.20' );
-			$query = wfCgiToArray( $query );
-		}
-
 		$services = MediaWikiServices::getInstance();
 		$options = (array)$options;
 		if ( $options ) {
@@ -136,7 +131,7 @@ class Linker {
 	 * @since 1.16.3
 	 * @deprecated since 1.28, use MediaWiki\Linker\LinkRenderer instead
 	 * @see Linker::link
-	 * @param Title $target
+	 * @param LinkTarget $target
 	 * @param string $html
 	 * @param array $customAttribs
 	 * @param array $query
@@ -157,7 +152,7 @@ class Linker {
 	 * make*LinkObj static functions, but $query is not used.
 	 *
 	 * @since 1.16.3
-	 * @param Title $nt
+	 * @param LinkTarget $nt
 	 * @param string $html [optional]
 	 * @param string $query [optional]
 	 * @param string $trail [optional]
@@ -166,6 +161,7 @@ class Linker {
 	 * @return string
 	 */
 	public static function makeSelfLinkObj( $nt, $html = '', $query = '', $trail = '', $prefix = '' ) {
+		$nt = Title::newFromLinkTarget( $nt );
 		$ret = "<a class=\"mw-selflink selflink\">{$prefix}{$html}</a>{$trail}";
 		if ( !Hooks::run( 'SelfLinkBegin', [ $nt, &$html, &$trail, &$prefix, &$ret ] ) ) {
 			return $ret;
@@ -190,7 +186,7 @@ class Linker {
 	 */
 	public static function getInvalidTitleDescription( IContextSource $context, $namespace, $title ) {
 		// First we check whether the namespace exists or not.
-		if ( MWNamespace::exists( $namespace ) ) {
+		if ( MediaWikiServices::getInstance()->getNamespaceInfo()->exists( $namespace ) ) {
 			if ( $namespace == NS_MAIN ) {
 				$name = $context->msg( 'blanknamespace' )->text();
 			} else {
@@ -272,7 +268,7 @@ class Linker {
 	 * HTML that that syntax inserts in the page.
 	 *
 	 * @param Parser $parser
-	 * @param Title $title Title object of the file (not the currently viewed page)
+	 * @param LinkTarget $title LinkTarget object of the file (not the currently viewed page)
 	 * @param File $file File object, or false if it doesn't exist
 	 * @param array $frameParams Associative array of parameters external to the media handler.
 	 *     Boolean parameters are indicated by presence or absence, the value is arbitrary and
@@ -291,7 +287,7 @@ class Linker {
 	 *          class           HTML for image classes. Plain text.
 	 *          caption         HTML for image caption.
 	 *          link-url        URL to link to
-	 *          link-title      Title object to link to
+	 *          link-title      LinkTarget object to link to
 	 *          link-target     Value for the target attribute, only with link-url
 	 *          no-link         Boolean, suppress description link
 	 *          targetlang      (optional) Target language code, see Parser::getTargetLanguage()
@@ -304,10 +300,11 @@ class Linker {
 	 * @since 1.20
 	 * @return string HTML for an image, with links, wrappers, etc.
 	 */
-	public static function makeImageLink( Parser $parser, Title $title,
+	public static function makeImageLink( Parser $parser, LinkTarget $title,
 		$file, $frameParams = [], $handlerParams = [], $time = false,
 		$query = "", $widthOption = null
 	) {
+		$title = Title::newFromLinkTarget( $title );
 		$res = null;
 		$dummy = new DummyLinker;
 		if ( !Hooks::run( 'ImageBeforeProduceHTML', [ &$dummy, &$title,
@@ -483,7 +480,7 @@ class Linker {
 
 	/**
 	 * Make HTML for a thumbnail including image, border and caption
-	 * @param Title $title
+	 * @param LinkTarget $title
 	 * @param File|bool $file File object or false if it doesn't exist
 	 * @param string $label
 	 * @param string $alt
@@ -493,7 +490,7 @@ class Linker {
 	 * @param string $manualthumb
 	 * @return string
 	 */
-	public static function makeThumbLinkObj( Title $title, $file, $label = '', $alt = '',
+	public static function makeThumbLinkObj( LinkTarget $title, $file, $label = '', $alt = '',
 		$align = 'right', $params = [], $framed = false, $manualthumb = ""
 	) {
 		$frameParams = [
@@ -511,7 +508,7 @@ class Linker {
 	}
 
 	/**
-	 * @param Title $title
+	 * @param LinkTarget $title
 	 * @param File $file
 	 * @param array $frameParams
 	 * @param array $handlerParams
@@ -519,7 +516,7 @@ class Linker {
 	 * @param string $query
 	 * @return string
 	 */
-	public static function makeThumbLink2( Title $title, $file, $frameParams = [],
+	public static function makeThumbLink2( LinkTarget $title, $file, $frameParams = [],
 		$handlerParams = [], $time = false, $query = ""
 	) {
 		$exists = $file && $file->exists();
@@ -553,7 +550,8 @@ class Linker {
 				# Use manually specified thumbnail
 				$manual_title = Title::makeTitleSafe( NS_FILE, $frameParams['manualthumb'] );
 				if ( $manual_title ) {
-					$manual_img = wfFindFile( $manual_title );
+					$manual_img = MediaWikiServices::getInstance()->getRepoGroup()
+						->findFile( $manual_title );
 					if ( $manual_img ) {
 						$thumb = $manual_img->getUnscaledThumb( $handlerParams );
 						$manualthumb = true;
@@ -585,7 +583,7 @@ class Linker {
 		# ThumbnailImage::toHtml() already adds page= onto the end of DjVu URLs
 		# So we don't need to pass it here in $query. However, the URL for the
 		# zoom icon still needs it, so we make a unique query for it. See T16771
-		$url = $title->getLocalURL( $query );
+		$url = Title::newFromLinkTarget( $title )->getLocalURL( $query );
 		if ( $page ) {
 			$url = wfAppendQuery( $url, [ 'page' => $page ] );
 		}
@@ -668,7 +666,7 @@ class Linker {
 	 * Make a "broken" link to an image
 	 *
 	 * @since 1.16.3
-	 * @param Title $title
+	 * @param LinkTarget $title
 	 * @param string $label Link label (plain text)
 	 * @param string $query Query string
 	 * @param string $unused1 Unused parameter kept for b/c
@@ -679,56 +677,62 @@ class Linker {
 	public static function makeBrokenImageLinkObj( $title, $label = '',
 		$query = '', $unused1 = '', $unused2 = '', $time = false
 	) {
-		if ( !$title instanceof Title ) {
-			wfWarn( __METHOD__ . ': Requires $title to be a Title object.' );
+		if ( !$title instanceof LinkTarget ) {
+			wfWarn( __METHOD__ . ': Requires $title to be a LinkTarget object.' );
 			return "<!-- ERROR -->" . htmlspecialchars( $label );
 		}
 
+		$title = Title::castFromLinkTarget( $title );
+
 		global $wgEnableUploads, $wgUploadMissingFileUrl, $wgUploadNavigationUrl;
 		if ( $label == '' ) {
 			$label = $title->getPrefixedText();
 		}
-		$encLabel = htmlspecialchars( $label );
-		$currentExists = $time ? ( wfFindFile( $title ) != false ) : false;
+		$repoGroup = MediaWikiServices::getInstance()->getRepoGroup();
+		$currentExists = $time
+			&& $repoGroup->findFile( $title ) !== false;
 
 		if ( ( $wgUploadMissingFileUrl || $wgUploadNavigationUrl || $wgEnableUploads )
 			&& !$currentExists
 		) {
-			$redir = RepoGroup::singleton()->getLocalRepo()->checkRedirect( $title );
-
-			if ( $redir ) {
-				// We already know it's a redirect, so mark it
-				// accordingly
+			if ( $repoGroup->getLocalRepo()->checkRedirect( $title ) ) {
+				// We already know it's a redirect, so mark it accordingly
 				return self::link(
 					$title,
-					$encLabel,
+					htmlspecialchars( $label ),
 					[ 'class' => 'mw-redirect' ],
 					wfCgiToArray( $query ),
 					[ 'known', 'noclasses' ]
 				);
 			}
 
-			$href = self::getUploadUrl( $title, $query );
-
-			return '<a href="' . htmlspecialchars( $href ) . '" class="new" title="' .
-				htmlspecialchars( $title->getPrefixedText(), ENT_QUOTES ) . '">' .
-				$encLabel . '</a>';
+			return Html::element( 'a', [
+					'href' => self::getUploadUrl( $title, $query ),
+					'class' => 'new',
+					'title' => $title->getPrefixedText()
+				], $label );
 		}
 
-		return self::link( $title, $encLabel, [], wfCgiToArray( $query ), [ 'known', 'noclasses' ] );
+		return self::link(
+			$title,
+			htmlspecialchars( $label ),
+			[],
+			wfCgiToArray( $query ),
+			[ 'known', 'noclasses' ]
+		);
 	}
 
 	/**
 	 * Get the URL to upload a certain file
 	 *
 	 * @since 1.16.3
-	 * @param Title $destFile Title object of the file to upload
+	 * @param LinkTarget $destFile LinkTarget object of the file to upload
 	 * @param string $query Urlencoded query string to prepend
 	 * @return string Urlencoded URL
 	 */
 	protected static function getUploadUrl( $destFile, $query = '' ) {
 		global $wgUploadMissingFileUrl, $wgUploadNavigationUrl;
-		$q = 'wpDestFile=' . $destFile->getPartialURL();
+		$q = 'wpDestFile=' . Title::castFromLinkTarget( $destFile )->getPartialURL();
 		if ( $query != '' ) {
 			$q .= '&' . $query;
 		}
@@ -750,13 +754,15 @@ class Linker {
 	 * Create a direct link to a given uploaded file.
 	 *
 	 * @since 1.16.3
-	 * @param Title $title
+	 * @param LinkTarget $title
 	 * @param string $html Pre-sanitized HTML
-	 * @param string $time MW timestamp of file creation time
+	 * @param string|false $time MW timestamp of file creation time
 	 * @return string HTML
 	 */
 	public static function makeMediaLinkObj( $title, $html = '', $time = false ) {
-		$img = wfFindFile( $title, [ 'time' => $time ] );
+		$img = MediaWikiServices::getInstance()->getRepoGroup()->findFile(
+			$title, [ 'time' => $time ]
+		);
 		return self::makeMediaLinkFile( $title, $img, $html );
 	}
 
@@ -765,14 +771,14 @@ class Linker {
 	 * This will make a broken link if $file is false.
 	 *
 	 * @since 1.16.3
-	 * @param Title $title
+	 * @param LinkTarget $title
 	 * @param File|bool $file File object or false
 	 * @param string $html Pre-sanitized HTML
 	 * @return string HTML
 	 *
 	 * @todo Handle invalid or missing images better.
 	 */
-	public static function makeMediaLinkFile( Title $title, $file, $html = '' ) {
+	public static function makeMediaLinkFile( LinkTarget $title, $file, $html = '' ) {
 		if ( $file && $file->exists() ) {
 			$url = $file->getUrl();
 			$class = 'internal';
@@ -794,7 +800,7 @@ class Linker {
 		];
 
 		if ( !Hooks::run( 'LinkerMakeMediaLinkFile',
-			[ $title, $file, &$html, &$attribs, &$ret ] ) ) {
+			[ Title::castFromLinkTarget( $title ), $file, &$html, &$attribs, &$ret ] ) ) {
 			wfDebug( "Hook LinkerMakeMediaLinkFile changed the output of link "
 				. "with url {$url} and text {$html} to {$ret}\n", true );
 			return $ret;
@@ -835,7 +841,7 @@ class Linker {
 	 * @param-taint $linktype escapes_html
 	 * @param array $attribs Array of extra attributes to <a>
 	 * @param-taint $attribs escapes_html
-	 * @param Title|null $title Title object used for title specific link attributes
+	 * @param LinkTarget|null $title LinkTarget object used for title specific link attributes
 	 * @param-taint $title none
 	 * @return string
 	 */
@@ -885,11 +891,17 @@ class Linker {
 	 * Make user link (or user contributions for unregistered users)
 	 * @param int $userId User id in database.
 	 * @param string $userName User name in database.
-	 * @param string $altUserName Text to display instead of the user name (optional)
+	 * @param string|false $altUserName Text to display instead of the user name (optional)
 	 * @return string HTML fragment
 	 * @since 1.16.3. $altUserName was added in 1.19.
 	 */
 	public static function userLink( $userId, $userName, $altUserName = false ) {
+		if ( $userName === '' || $userName === false || $userName === null ) {
+			wfDebug( __METHOD__ . ' received an empty username. Are there database errors ' .
+				'that need to be fixed?' );
+			return wfMessage( 'empty-username' )->parse();
+		}
+
 		$classes = 'mw-userlink';
 		$page = null;
 		if ( $userId == 0 ) {
@@ -902,7 +914,7 @@ class Linker {
 			}
 			$classes .= ' mw-anonuserlink'; // Separate link class for anons (T45179)
 		} else {
-			$page = Title::makeTitle( NS_USER, $userName );
+			$page = new TitleValue( NS_USER, strtr( $userName, ' ', '_' ) );
 		}
 
 		// Wrap the output with <bdi> tags for directionality isolation
@@ -932,6 +944,12 @@ class Linker {
 		$userId, $userText, $redContribsWhenNoEdits = false, $flags = 0, $edits = null,
 		$useParentheses = true
 	) {
+		if ( $userText === '' ) {
+			wfDebug( __METHOD__ . ' received an empty username. Are there database errors ' .
+				'that need to be fixed?' );
+			return ' ' . wfMessage( 'empty-username' )->parse();
+		}
+
 		global $wgUser, $wgDisableAnonTalk, $wgLang;
 		$talkable = !( $wgDisableAnonTalk && $userId == 0 );
 		$blockable = !( $flags & self::TOOL_LINKS_NOBLOCK );
@@ -963,7 +981,9 @@ class Linker {
 
 			$items[] = self::link( $contribsPage, wfMessage( 'contribslink' )->escaped(), $attribs );
 		}
-		if ( $blockable && $wgUser->isAllowed( 'block' ) ) {
+		$userCanBlock = MediaWikiServices::getInstance()->getPermissionManager()
+			->userHasRight( $wgUser, 'block' );
+		if ( $blockable && $userCanBlock ) {
 			$items[] = self::blockLink( $userId, $userText );
 		}
 
@@ -1014,8 +1034,14 @@ class Linker {
 	 * @return string HTML fragment with user talk link
 	 */
 	public static function userTalkLink( $userId, $userText ) {
-		$userTalkPage = Title::makeTitle( NS_USER_TALK, $userText );
-		$moreLinkAttribs['class'] = 'mw-usertoollinks-talk';
+		if ( $userText === '' ) {
+			wfDebug( __METHOD__ . ' received an empty username. Are there database errors ' .
+				'that need to be fixed?' );
+			return wfMessage( 'empty-username' )->parse();
+		}
+
+		$userTalkPage = new TitleValue( NS_USER_TALK, strtr( $userText, ' ', '_' ) );
+		$moreLinkAttribs = [ 'class' => 'mw-usertoollinks-talk' ];
 
 		return self::link( $userTalkPage,
 			wfMessage( 'talkpagelinktext' )->escaped(),
@@ -1030,8 +1056,14 @@ class Linker {
 	 * @return string HTML fragment with block link
 	 */
 	public static function blockLink( $userId, $userText ) {
+		if ( $userText === '' ) {
+			wfDebug( __METHOD__ . ' received an empty username. Are there database errors ' .
+				'that need to be fixed?' );
+			return wfMessage( 'empty-username' )->parse();
+		}
+
 		$blockPage = SpecialPage::getTitleFor( 'Block', $userText );
-		$moreLinkAttribs['class'] = 'mw-usertoollinks-block';
+		$moreLinkAttribs = [ 'class' => 'mw-usertoollinks-block' ];
 
 		return self::link( $blockPage,
 			wfMessage( 'blocklink' )->escaped(),
@@ -1045,8 +1077,14 @@ class Linker {
 	 * @return string HTML fragment with e-mail user link
 	 */
 	public static function emailLink( $userId, $userText ) {
+		if ( $userText === '' ) {
+			wfLogWarning( __METHOD__ . ' received an empty username. Are there database errors ' .
+				'that need to be fixed?' );
+			return wfMessage( 'empty-username' )->parse();
+		}
+
 		$emailPage = SpecialPage::getTitleFor( 'Emailuser', $userText );
-		$moreLinkAttribs['class'] = 'mw-usertoollinks-mail';
+		$moreLinkAttribs = [ 'class' => 'mw-usertoollinks-mail' ];
 		return self::link( $emailPage,
 			wfMessage( 'emaillink' )->escaped(),
 			$moreLinkAttribs
@@ -1061,15 +1099,15 @@ class Linker {
 	 * @return string HTML fragment
 	 */
 	public static function revUserLink( $rev, $isPublic = false ) {
-		if ( $rev->isDeleted( Revision::DELETED_USER ) && $isPublic ) {
+		if ( $rev->isDeleted( RevisionRecord::DELETED_USER ) && $isPublic ) {
 			$link = wfMessage( 'rev-deleted-user' )->escaped();
-		} elseif ( $rev->userCan( Revision::DELETED_USER ) ) {
-			$link = self::userLink( $rev->getUser( Revision::FOR_THIS_USER ),
-				$rev->getUserText( Revision::FOR_THIS_USER ) );
+		} elseif ( $rev->userCan( RevisionRecord::DELETED_USER ) ) {
+			$link = self::userLink( $rev->getUser( RevisionRecord::FOR_THIS_USER ),
+				$rev->getUserText( RevisionRecord::FOR_THIS_USER ) );
 		} else {
 			$link = wfMessage( 'rev-deleted-user' )->escaped();
 		}
-		if ( $rev->isDeleted( Revision::DELETED_USER ) ) {
+		if ( $rev->isDeleted( RevisionRecord::DELETED_USER ) ) {
 			return '<span class="history-deleted">' . $link . '</span>';
 		}
 		return $link;
@@ -1084,18 +1122,23 @@ class Linker {
 	 * @return string HTML
 	 */
 	public static function revUserTools( $rev, $isPublic = false, $useParentheses = true ) {
-		if ( $rev->isDeleted( Revision::DELETED_USER ) && $isPublic ) {
-			$link = wfMessage( 'rev-deleted-user' )->escaped();
-		} elseif ( $rev->userCan( Revision::DELETED_USER ) ) {
-			$userId = $rev->getUser( Revision::FOR_THIS_USER );
-			$userText = $rev->getUserText( Revision::FOR_THIS_USER );
-			$link = self::userLink( $userId, $userText )
-				. self::userToolLinks( $userId, $userText, false, 0, null,
-					$useParentheses );
-		} else {
+		if ( $rev->userCan( RevisionRecord::DELETED_USER ) &&
+			( !$rev->isDeleted( RevisionRecord::DELETED_USER ) || !$isPublic )
+		) {
+			$userId = $rev->getUser( RevisionRecord::FOR_THIS_USER );
+			$userText = $rev->getUserText( RevisionRecord::FOR_THIS_USER );
+			if ( $userId || (string)$userText !== '' ) {
+				$link = self::userLink( $userId, $userText )
+					. self::userToolLinks( $userId, $userText, false, 0, null,
+						$useParentheses );
+			}
+		}
+
+		if ( !isset( $link ) ) {
 			$link = wfMessage( 'rev-deleted-user' )->escaped();
 		}
-		if ( $rev->isDeleted( Revision::DELETED_USER ) ) {
+
+		if ( $rev->isDeleted( RevisionRecord::DELETED_USER ) ) {
 			return ' <span class="history-deleted mw-userlink">' . $link . '</span>';
 		}
 		return $link;
@@ -1111,8 +1154,8 @@ class Linker {
 	 * @since 1.16.3. $wikiId added in 1.26
 	 *
 	 * @param string $comment
-	 * @param Title|null $title Title object (to generate link to the section in autocomment)
-	 *  or null
+	 * @param LinkTarget|null $title LinkTarget object (to generate link to the section in
+	 *  autocomment) or null
 	 * @param bool $local Whether section links should refer to local page
 	 * @param string|null $wikiId Id (as used by WikiMap) of the wiki to generate links to.
 	 *  For use with external changes.
@@ -1142,7 +1185,7 @@ class Linker {
 	 * Called by Linker::formatComment.
 	 *
 	 * @param string $comment Comment text
-	 * @param Title|null $title An optional title object used to links to sections
+	 * @param LinkTarget|null $title An optional LinkTarget object used to links to sections
 	 * @param bool $local Whether section links should refer to local page
 	 * @param string|null $wikiId Id of the wiki to link to (if not the local wiki),
 	 *  as used by WikiMap.
@@ -1178,7 +1221,8 @@ class Linker {
 
 				Hooks::run(
 					'FormatAutocomments',
-					[ &$comment, $pre, $auto, $post, $title, $local, $wikiId ]
+					[ &$comment, $pre, $auto, $post, Title::castFromLinkTarget( $title ), $local,
+					$wikiId ]
 				);
 
 				if ( $comment === null ) {
@@ -1197,16 +1241,22 @@ class Linker {
 						$sectionText = str_replace( '[[', '&#91;[', $auto );
 
 						$section = substr( Parser::guessSectionNameFromStrippedText( $section ), 1 );
-						if ( $local ) {
-							$sectionTitle = Title::makeTitleSafe( NS_MAIN, '', $section );
-						} else {
-							$sectionTitle = Title::makeTitleSafe( $title->getNamespace(),
-								$title->getDBkey(), $section );
-						}
-						if ( $sectionTitle ) {
+						// Support: HHVM (T222857)
+						// The guessSectionNameFromStrippedText method returns a non-empty string
+						// that starts with "#". Before PHP 7 (and still on HHVM) substr() would
+						// return false if the start offset is the end of the string.
+						// On PHP 7+, it gracefully returns empty string instead.
+						if ( $section !== '' && $section !== false ) {
+							if ( $local ) {
+								$sectionTitle = new TitleValue( NS_MAIN, '', $section );
+							} else {
+								$sectionTitle = $title->createFragmentTarget( $section );
+							}
 							$auto = Linker::makeCommentLink(
-								$sectionTitle, $wgLang->getArrow() . $wgLang->getDirMark() . $sectionText,
-								$wikiId, 'noclasses'
+								$sectionTitle,
+								$wgLang->getArrow() . $wgLang->getDirMark() . $sectionText,
+								$wikiId,
+								'noclasses'
 							);
 						}
 					}
@@ -1242,7 +1292,7 @@ class Linker {
 	 * 	function is html, $comment must be sanitized for use as html. You probably want
 	 * 	to pass $comment through Sanitizer::escapeHtmlAllowEntities() before calling
 	 * 	this function.
-	 * @param Title|null $title An optional title object used to links to sections
+	 * @param LinkTarget|null $title An optional LinkTarget object used to links to sections
 	 * @param bool $local Whether section links should refer to local page
 	 * @param string|null $wikiId Id of the wiki to link to (if not the local wiki),
 	 *  as used by WikiMap.
@@ -1268,9 +1318,14 @@ class Linker {
 				([^[]*) # 3. link trail (the text up until the next link)
 			/x',
 			function ( $match ) use ( $title, $local, $wikiId ) {
-				$medians = '(?:' . preg_quote( MWNamespace::getCanonicalName( NS_MEDIA ), '/' ) . '|';
+				$services = MediaWikiServices::getInstance();
+
+				$medians = '(?:';
 				$medians .= preg_quote(
-					MediaWikiServices::getInstance()->getContentLanguage()->getNsText( NS_MEDIA ),
+					$services->getNamespaceInfo()->getCanonicalName( NS_MEDIA ), '/' );
+				$medians .= '|';
+				$medians .= preg_quote(
+					$services->getContentLanguage()->getNsText( NS_MEDIA ),
 					'/'
 				) . '):';
 
@@ -1307,7 +1362,7 @@ class Linker {
 					}
 					if ( $match[1] !== false && $match[1] !== '' ) {
 						if ( preg_match(
-							MediaWikiServices::getInstance()->getContentLanguage()->linkTrail(),
+							$services->getContentLanguage()->linkTrail(),
 							$match[3],
 							$submatch
 						) ) {
@@ -1321,8 +1376,11 @@ class Linker {
 						$linkText = $text;
 						$linkTarget = Linker::normalizeSubpageLink( $title, $match[1], $linkText );
 
-						$target = Title::newFromText( $linkTarget );
-						if ( $target ) {
+						Title::newFromText( $linkTarget );
+						try {
+							$target = $services->getTitleParser()->
+								parseTitle( $linkTarget );
+
 							if ( $target->getText() == '' && !$target->isExternal()
 								&& !$local && $title
 							) {
@@ -1330,6 +1388,8 @@ class Linker {
 							}
 
 							$thelink = Linker::makeCommentLink( $target, $linkText . $inside, $wikiId ) . $trail;
+						} catch ( MalformedTitleException $e ) {
+							// Fall through
 						}
 					}
 				}
@@ -1350,7 +1410,7 @@ class Linker {
 	}
 
 	/**
-	 * Generates a link to the given Title
+	 * Generates a link to the given LinkTarget
 	 *
 	 * @note This is only public for technical reasons. It's not intended for use outside Linker.
 	 *
@@ -1371,8 +1431,9 @@ class Linker {
 					$wikiId,
 					$linkTarget->getNamespace() === 0
 						? $linkTarget->getDBkey()
-						: MWNamespace::getCanonicalName( $linkTarget->getNamespace() ) . ':'
-							. $linkTarget->getDBkey(),
+						: MediaWikiServices::getInstance()->getNamespaceInfo()->
+							getCanonicalName( $linkTarget->getNamespace() ) .
+							':' . $linkTarget->getDBkey(),
 					$linkTarget->getFragment()
 				),
 				$text,
@@ -1386,7 +1447,7 @@ class Linker {
 	}
 
 	/**
-	 * @param Title $contextTitle
+	 * @param LinkTarget $contextTitle
 	 * @param string $target
 	 * @param string &$text
 	 * @return string
@@ -1407,7 +1468,10 @@ class Linker {
 
 		# Some namespaces don't allow subpages,
 		# so only perform processing if subpages are allowed
-		if ( $contextTitle && MWNamespace::hasSubpages( $contextTitle->getNamespace() ) ) {
+		if (
+			$contextTitle && MediaWikiServices::getInstance()->getNamespaceInfo()->
+			hasSubpages( $contextTitle->getNamespace() )
+		) {
 			$hash = strpos( $target, '#' );
 			if ( $hash !== false ) {
 				$suffix = substr( $target, $hash );
@@ -1417,6 +1481,8 @@ class Linker {
 			}
 			# T9425
 			$target = trim( $target );
+			$contextPrefixedText = MediaWikiServices::getInstance()->getTitleFormatter()->
+				getPrefixedText( $contextTitle );
 			# Look at the first character
 			if ( $target != '' && $target[0] === '/' ) {
 				# / at end means we don't want the slash to be shown
@@ -1428,7 +1494,7 @@ class Linker {
 					$noslash = substr( $target, 1 );
 				}
 
-				$ret = $contextTitle->getPrefixedText() . '/' . trim( $noslash ) . $suffix;
+				$ret = $contextPrefixedText . '/' . trim( $noslash ) . $suffix;
 				if ( $text === '' ) {
 					$text = $target . $suffix;
 				} # this might be changed for ugliness reasons
@@ -1441,7 +1507,7 @@ class Linker {
 					$nodotdot = substr( $nodotdot, 3 );
 				}
 				if ( $dotdotcount > 0 ) {
-					$exploded = explode( '/', $contextTitle->getPrefixedText() );
+					$exploded = explode( '/', $contextPrefixedText );
 					if ( count( $exploded ) > $dotdotcount ) { # not allowed to go below top level page
 						$ret = implode( '/', array_slice( $exploded, 0, -$dotdotcount ) );
 						# / at the end means don't show full path
@@ -1470,7 +1536,8 @@ class Linker {
 	 *
 	 * @since 1.16.3. $wikiId added in 1.26
 	 * @param string $comment
-	 * @param Title|null $title Title object (to generate link to section in autocomment) or null
+	 * @param LinkTarget|null $title LinkTarget object (to generate link to section in autocomment)
+	 *  or null
 	 * @param bool $local Whether section links should refer to local page
 	 * @param string|null $wikiId Id (as used by WikiMap) of the wiki to generate links to.
 	 *  For use with external changes.
@@ -1510,18 +1577,18 @@ class Linker {
 	public static function revComment( Revision $rev, $local = false, $isPublic = false,
 		$useParentheses = true
 	) {
-		if ( $rev->getComment( Revision::RAW ) == "" ) {
+		if ( $rev->getComment( RevisionRecord::RAW ) == "" ) {
 			return "";
 		}
-		if ( $rev->isDeleted( Revision::DELETED_COMMENT ) && $isPublic ) {
+		if ( $rev->isDeleted( RevisionRecord::DELETED_COMMENT ) && $isPublic ) {
 			$block = " <span class=\"comment\">" . wfMessage( 'rev-deleted-comment' )->escaped() . "</span>";
-		} elseif ( $rev->userCan( Revision::DELETED_COMMENT ) ) {
-			$block = self::commentBlock( $rev->getComment( Revision::FOR_THIS_USER ),
+		} elseif ( $rev->userCan( RevisionRecord::DELETED_COMMENT ) ) {
+			$block = self::commentBlock( $rev->getComment( RevisionRecord::FOR_THIS_USER ),
 				$rev->getTitle(), $local, null, $useParentheses );
 		} else {
 			$block = " <span class=\"comment\">" . wfMessage( 'rev-deleted-comment' )->escaped() . "</span>";
 		}
-		if ( $rev->isDeleted( Revision::DELETED_COMMENT ) ) {
+		if ( $rev->isDeleted( RevisionRecord::DELETED_COMMENT ) ) {
 			return " <span class=\"history-deleted comment\">$block</span>";
 		}
 		return $block;
@@ -1606,16 +1673,11 @@ class Linker {
 	 *
 	 * @since 1.16.3
 	 * @param string $toc Html of the Table Of Contents
-	 * @param string|Language|bool|null $lang Language for the toc title, defaults to user language.
-	 *  The types string and bool are deprecated.
+	 * @param Language|null $lang Language for the toc title, defaults to user language
 	 * @return string Full html of the TOC
 	 */
-	public static function tocList( $toc, $lang = null ) {
+	public static function tocList( $toc, Language $lang = null ) {
 		$lang = $lang ?? RequestContext::getMain()->getLanguage();
-		if ( !$lang instanceof Language ) {
-			wfDeprecated( __METHOD__ . ' with type other than Language for $lang', '1.33' );
-			$lang = wfGetLangObj( $lang );
-		}
 
 		$title = wfMessage( 'toc' )->inLanguage( $lang )->escaped();
 
@@ -1648,11 +1710,10 @@ class Linker {
 	 *
 	 * @since 1.16.3. $lang added in 1.17
 	 * @param array $tree Return value of ParserOutput::getSections()
-	 * @param string|Language|bool|null $lang Language for the toc title, defaults to user language.
-	 *  The types string and bool are deprecated.
+	 * @param Language|null $lang Language for the toc title, defaults to user language
 	 * @return string HTML fragment
 	 */
-	public static function generateTOC( $tree, $lang = null ) {
+	public static function generateTOC( $tree, Language $lang = null ) {
 		$toc = '';
 		$lastLevel = 0;
 		foreach ( $tree as $section ) {
@@ -1820,10 +1881,10 @@ class Linker {
 		$editCount = 0;
 		$moreRevs = false;
 		foreach ( $res as $row ) {
-			if ( $rev->getUserText( Revision::RAW ) != $row->rev_user_text ) {
+			if ( $rev->getUserText( RevisionRecord::RAW ) != $row->rev_user_text ) {
 				if ( $verify &&
-					( $row->rev_deleted & Revision::DELETED_TEXT
-						|| $row->rev_deleted & Revision::DELETED_USER
+					( $row->rev_deleted & RevisionRecord::DELETED_TEXT
+						|| $row->rev_deleted & RevisionRecord::DELETED_USER
 				) ) {
 					// If the user or the text of the revision we might rollback
 					// to is deleted in some way we can't rollback. Similar to
@@ -1851,7 +1912,7 @@ class Linker {
 	 * @since 1.16.3. $context added in 1.20. $editCount added in 1.21
 	 * @param Revision $rev
 	 * @param IContextSource|null $context Context to use or null for the main context.
-	 * @param int $editCount Number of edits that would be reverted
+	 * @param int|false $editCount Number of edits that would be reverted
 	 * @return string HTML fragment
 	 */
 	public static function buildRollbackLink( $rev, IContextSource $context = null,
@@ -2043,24 +2104,28 @@ class Linker {
 	 *
 	 * @param User $user
 	 * @param Revision $rev
-	 * @param Title $title
+	 * @param LinkTarget $title
 	 * @return string HTML fragment
 	 */
-	public static function getRevDeleteLink( User $user, Revision $rev, Title $title ) {
-		$canHide = $user->isAllowed( 'deleterevision' );
-		if ( !$canHide && !( $rev->getVisibility() && $user->isAllowed( 'deletedhistory' ) ) ) {
+	public static function getRevDeleteLink( User $user, Revision $rev, LinkTarget $title ) {
+		$permissionManager = MediaWikiServices::getInstance()->getPermissionManager();
+		$canHide = $permissionManager->userHasRight( $user, 'deleterevision' );
+		$canHideHistory = $permissionManager->userHasRight( $user, 'deletedhistory' );
+		if ( !$canHide && !( $rev->getVisibility() && $canHideHistory ) ) {
 			return '';
 		}
 
-		if ( !$rev->userCan( Revision::DELETED_RESTRICTED, $user ) ) {
+		if ( !$rev->userCan( RevisionRecord::DELETED_RESTRICTED, $user ) ) {
 			return self::revDeleteLinkDisabled( $canHide ); // revision was hidden from sysops
 		}
+		$prefixedDbKey = MediaWikiServices::getInstance()->getTitleFormatter()->
+			getPrefixedDBkey( $title );
 		if ( $rev->getId() ) {
 			// RevDelete links using revision ID are stable across
 			// page deletion and undeletion; use when possible.
 			$query = [
 				'type' => 'revision',
-				'target' => $title->getPrefixedDBkey(),
+				'target' => $prefixedDbKey,
 				'ids' => $rev->getId()
 			];
 		} else {
@@ -2068,12 +2133,12 @@ class Linker {
 			// We have to refer to these by timestamp, ick!
 			$query = [
 				'type' => 'archive',
-				'target' => $title->getPrefixedDBkey(),
+				'target' => $prefixedDbKey,
 				'ids' => $rev->getTimestamp()
 			];
 		}
 		return self::revDeleteLink( $query,
-			$rev->isDeleted( Revision::DELETED_RESTRICTED ), $canHide );
+			$rev->isDeleted( RevisionRecord::DELETED_RESTRICTED ), $canHide );
 	}
 
 	/**
