@@ -29,11 +29,10 @@ use MediaWiki\Revision\RevisionStore;
 use MediaWiki\Revision\RevisionStoreRecord;
 use MediaWiki\Revision\SlotRecord;
 use MediaWiki\Storage\SqlBlobStore;
+use Wikimedia\Assert\Assert;
 use Wikimedia\Rdbms\IDatabase;
 use MediaWiki\Linker\LinkTarget;
 use MediaWiki\MediaWikiServices;
-use Wikimedia\Rdbms\ResultWrapper;
-use Wikimedia\Rdbms\FakeResultWrapper;
 
 /**
  * @deprecated since 1.31, use RevisionRecord, RevisionStore, and BlobStore instead.
@@ -61,8 +60,13 @@ class Revision implements IDBAccessObject {
 	/**
 	 * @return RevisionStore
 	 */
-	protected static function getRevisionStore() {
-		return MediaWikiServices::getInstance()->getRevisionStore();
+	protected static function getRevisionStore( $wiki = false ) {
+		if ( $wiki ) {
+			return MediaWikiServices::getInstance()->getRevisionStoreFactory()
+				->getRevisionStore( $wiki );
+		} else {
+			return MediaWikiServices::getInstance()->getRevisionStore();
+		}
 	}
 
 	/**
@@ -85,6 +89,7 @@ class Revision implements IDBAccessObject {
 	 * @return SqlBlobStore
 	 */
 	protected static function getBlobStore( $wiki = false ) {
+		// @phan-suppress-next-line PhanAccessMethodInternal
 		$store = MediaWikiServices::getInstance()
 			->getBlobStoreFactory()
 			->newSqlBlobStore( $wiki );
@@ -113,7 +118,7 @@ class Revision implements IDBAccessObject {
 	 */
 	public static function newFromId( $id, $flags = 0 ) {
 		$rec = self::getRevisionLookup()->getRevisionById( $id, $flags );
-		return $rec === null ? null : new Revision( $rec, $flags );
+		return $rec ? new Revision( $rec, $flags ) : null;
 	}
 
 	/**
@@ -132,7 +137,7 @@ class Revision implements IDBAccessObject {
 	 */
 	public static function newFromTitle( LinkTarget $linkTarget, $id = 0, $flags = 0 ) {
 		$rec = self::getRevisionLookup()->getRevisionByTitle( $linkTarget, $id, $flags );
-		return $rec === null ? null : new Revision( $rec, $flags );
+		return $rec ? new Revision( $rec, $flags ) : null;
 	}
 
 	/**
@@ -151,7 +156,7 @@ class Revision implements IDBAccessObject {
 	 */
 	public static function newFromPageId( $pageId, $revId = 0, $flags = 0 ) {
 		$rec = self::getRevisionLookup()->getRevisionByPageId( $pageId, $revId, $flags );
-		return $rec === null ? null : new Revision( $rec, $flags );
+		return $rec ? new Revision( $rec, $flags ) : null;
 	}
 
 	/**
@@ -238,7 +243,7 @@ class Revision implements IDBAccessObject {
 	public static function loadFromId( $db, $id ) {
 		wfDeprecated( __METHOD__, '1.31' ); // no known callers
 		$rec = self::getRevisionStore()->loadRevisionFromId( $db, $id );
-		return $rec === null ? null : new Revision( $rec );
+		return $rec ? new Revision( $rec ) : null;
 	}
 
 	/**
@@ -255,7 +260,7 @@ class Revision implements IDBAccessObject {
 	 */
 	public static function loadFromPageId( $db, $pageid, $id = 0 ) {
 		$rec = self::getRevisionStore()->loadRevisionFromPageId( $db, $pageid, $id );
-		return $rec === null ? null : new Revision( $rec );
+		return $rec ? new Revision( $rec ) : null;
 	}
 
 	/**
@@ -272,7 +277,7 @@ class Revision implements IDBAccessObject {
 	 */
 	public static function loadFromTitle( $db, $title, $id = 0 ) {
 		$rec = self::getRevisionStore()->loadRevisionFromTitle( $db, $title, $id );
-		return $rec === null ? null : new Revision( $rec );
+		return $rec ? new Revision( $rec ) : null;
 	}
 
 	/**
@@ -290,218 +295,7 @@ class Revision implements IDBAccessObject {
 	 */
 	public static function loadFromTimestamp( $db, $title, $timestamp ) {
 		$rec = self::getRevisionStore()->loadRevisionFromTimestamp( $db, $title, $timestamp );
-		return $rec === null ? null : new Revision( $rec );
-	}
-
-	/**
-	 * Return a wrapper for a series of database rows to
-	 * fetch all of a given page's revisions in turn.
-	 * Each row can be fed to the constructor to get objects.
-	 *
-	 * @param LinkTarget $title
-	 * @return ResultWrapper
-	 * @deprecated Since 1.28, no callers in core nor in known extensions. No-op since 1.31.
-	 */
-	public static function fetchRevision( LinkTarget $title ) {
-		wfDeprecated( __METHOD__, '1.31' );
-		return new FakeResultWrapper( [] );
-	}
-
-	/**
-	 * Return the value of a select() JOIN conds array for the user table.
-	 * This will get user table rows for logged-in users.
-	 * @since 1.19
-	 * @deprecated since 1.31, use RevisionStore::getQueryInfo( [ 'user' ] ) instead.
-	 * @return array
-	 */
-	public static function userJoinCond() {
-		global $wgActorTableSchemaMigrationStage;
-
-		wfDeprecated( __METHOD__, '1.31' );
-		if ( $wgActorTableSchemaMigrationStage & SCHEMA_COMPAT_READ_NEW ) {
-			// If code is using this instead of self::getQueryInfo(), there's
-			// no way the join it's trying to do can work once the old fields
-			// aren't being used anymore.
-			throw new BadMethodCallException(
-				'Cannot use ' . __METHOD__
-					. ' when $wgActorTableSchemaMigrationStage has SCHEMA_COMPAT_READ_NEW'
-			);
-		}
-
-		return [ 'LEFT JOIN', [ 'rev_user != 0', 'user_id = rev_user' ] ];
-	}
-
-	/**
-	 * Return the value of a select() page conds array for the page table.
-	 * This will assure that the revision(s) are not orphaned from live pages.
-	 * @since 1.19
-	 * @deprecated since 1.31, use RevisionStore::getQueryInfo( [ 'page' ] ) instead.
-	 * @return array
-	 */
-	public static function pageJoinCond() {
-		wfDeprecated( __METHOD__, '1.31' );
-		return [ 'INNER JOIN', [ 'page_id = rev_page' ] ];
-	}
-
-	/**
-	 * Return the list of revision fields that should be selected to create
-	 * a new revision.
-	 * @deprecated since 1.31, use RevisionStore::getQueryInfo() instead.
-	 * @return array
-	 */
-	public static function selectFields() {
-		global $wgContentHandlerUseDB, $wgActorTableSchemaMigrationStage;
-		global $wgMultiContentRevisionSchemaMigrationStage;
-
-		if ( $wgActorTableSchemaMigrationStage & SCHEMA_COMPAT_READ_NEW ) {
-			// If code is using this instead of self::getQueryInfo(), there's a
-			// decent chance it's going to try to directly access
-			// $row->rev_user or $row->rev_user_text and we can't give it
-			// useful values here once those aren't being used anymore.
-			throw new BadMethodCallException(
-				'Cannot use ' . __METHOD__
-					. ' when $wgActorTableSchemaMigrationStage has SCHEMA_COMPAT_READ_NEW'
-			);
-		}
-
-		if ( !( $wgMultiContentRevisionSchemaMigrationStage & SCHEMA_COMPAT_WRITE_OLD ) ) {
-			// If code is using this instead of self::getQueryInfo(), there's a
-			// decent chance it's going to try to directly access
-			// $row->rev_text_id or $row->rev_content_model and we can't give it
-			// useful values here once those aren't being written anymore,
-			// and may not exist at all.
-			throw new BadMethodCallException(
-				'Cannot use ' . __METHOD__ . ' when $wgMultiContentRevisionSchemaMigrationStage '
-				. 'does not have SCHEMA_COMPAT_WRITE_OLD set.'
-			);
-		}
-
-		wfDeprecated( __METHOD__, '1.31' );
-
-		$fields = [
-			'rev_id',
-			'rev_page',
-			'rev_text_id',
-			'rev_timestamp',
-			'rev_user_text',
-			'rev_user',
-			'rev_actor' => 'NULL',
-			'rev_minor_edit',
-			'rev_deleted',
-			'rev_len',
-			'rev_parent_id',
-			'rev_sha1',
-		];
-
-		$fields += CommentStore::getStore()->getFields( 'rev_comment' );
-
-		if ( $wgContentHandlerUseDB ) {
-			$fields[] = 'rev_content_format';
-			$fields[] = 'rev_content_model';
-		}
-
-		return $fields;
-	}
-
-	/**
-	 * Return the list of revision fields that should be selected to create
-	 * a new revision from an archive row.
-	 * @deprecated since 1.31, use RevisionStore::getArchiveQueryInfo() instead.
-	 * @return array
-	 */
-	public static function selectArchiveFields() {
-		global $wgContentHandlerUseDB, $wgActorTableSchemaMigrationStage;
-		global $wgMultiContentRevisionSchemaMigrationStage;
-
-		if ( $wgActorTableSchemaMigrationStage & SCHEMA_COMPAT_READ_NEW ) {
-			// If code is using this instead of self::getQueryInfo(), there's a
-			// decent chance it's going to try to directly access
-			// $row->ar_user or $row->ar_user_text and we can't give it
-			// useful values here once those aren't being used anymore.
-			throw new BadMethodCallException(
-				'Cannot use ' . __METHOD__
-					. ' when $wgActorTableSchemaMigrationStage has SCHEMA_COMPAT_READ_NEW'
-			);
-		}
-
-		if ( !( $wgMultiContentRevisionSchemaMigrationStage & SCHEMA_COMPAT_WRITE_OLD ) ) {
-			// If code is using this instead of self::getQueryInfo(), there's a
-			// decent chance it's going to try to directly access
-			// $row->ar_text_id or $row->ar_content_model and we can't give it
-			// useful values here once those aren't being written anymore,
-			// and may not exist at all.
-			throw new BadMethodCallException(
-				'Cannot use ' . __METHOD__ . ' when $wgMultiContentRevisionSchemaMigrationStage '
-				. 'does not have SCHEMA_COMPAT_WRITE_OLD set.'
-			);
-		}
-
-		wfDeprecated( __METHOD__, '1.31' );
-
-		$fields = [
-			'ar_id',
-			'ar_page_id',
-			'ar_rev_id',
-			'ar_text_id',
-			'ar_timestamp',
-			'ar_user_text',
-			'ar_user',
-			'ar_actor' => 'NULL',
-			'ar_minor_edit',
-			'ar_deleted',
-			'ar_len',
-			'ar_parent_id',
-			'ar_sha1',
-		];
-
-		$fields += CommentStore::getStore()->getFields( 'ar_comment' );
-
-		if ( $wgContentHandlerUseDB ) {
-			$fields[] = 'ar_content_format';
-			$fields[] = 'ar_content_model';
-		}
-		return $fields;
-	}
-
-	/**
-	 * Return the list of text fields that should be selected to read the
-	 * revision text
-	 * @deprecated since 1.31, use RevisionStore::getQueryInfo( [ 'text' ] ) instead.
-	 * @return array
-	 */
-	public static function selectTextFields() {
-		wfDeprecated( __METHOD__, '1.31' );
-		return [
-			'old_text',
-			'old_flags'
-		];
-	}
-
-	/**
-	 * Return the list of page fields that should be selected from page table
-	 * @deprecated since 1.31, use RevisionStore::getQueryInfo( [ 'page' ] ) instead.
-	 * @return array
-	 */
-	public static function selectPageFields() {
-		wfDeprecated( __METHOD__, '1.31' );
-		return [
-			'page_namespace',
-			'page_title',
-			'page_id',
-			'page_latest',
-			'page_is_redirect',
-			'page_len',
-		];
-	}
-
-	/**
-	 * Return the list of user fields that should be selected from user table
-	 * @deprecated since 1.31, use RevisionStore::getQueryInfo( [ 'user' ] ) instead.
-	 * @return array
-	 */
-	public static function selectUserFields() {
-		wfDeprecated( __METHOD__, '1.31' );
-		return [ 'user_name' ];
+		return $rec ? new Revision( $rec ) : null;
 	}
 
 	/**
@@ -554,7 +348,7 @@ class Revision implements IDBAccessObject {
 	 * @param int $queryFlags
 	 * @param Title|null $title
 	 *
-	 * @access private
+	 * @private
 	 */
 	function __construct( $row, $queryFlags = 0, Title $title = null ) {
 		global $wgUser;
@@ -584,6 +378,8 @@ class Revision implements IDBAccessObject {
 				'$row must be a row object, an associative array, or a RevisionRecord'
 			);
 		}
+
+		Assert::postcondition( $this->mRecord !== null, 'Failed to construct a RevisionRecord' );
 	}
 
 	/**
@@ -838,18 +634,17 @@ class Revision implements IDBAccessObject {
 		$user = $this->mRecord->getUser( $audience, $user );
 		return $user ? $user->getName() : '';
 	}
+
 	/**
-	 * Fetch revision comment if it's available to the specified audience.
-	 * If the specified audience does not have access to the comment, an
-	 * empty string will be returned.
-	 *
 	 * @param int $audience One of:
 	 *   Revision::FOR_PUBLIC       to be displayed to all users
 	 *   Revision::FOR_THIS_USER    to be displayed to the given user
 	 *   Revision::RAW              get the text regardless of permissions
 	 * @param User|null $user User object to check for, only if FOR_THIS_USER is passed
 	 *   to the $audience parameter
-	 * @return string
+	 *
+	 * @return string|null Returns null if the specified audience does not have access to the
+	 *  comment.
 	 */
 	function getComment( $audience = self::FOR_PUBLIC, User $user = null ) {
 		global $wgUser;
@@ -1017,9 +812,8 @@ class Revision implements IDBAccessObject {
 	 * @return Revision|null
 	 */
 	public function getPrevious() {
-		$title = $this->getTitle();
-		$rec = self::getRevisionLookup()->getPreviousRevision( $this->mRecord, $title );
-		return $rec === null ? null : new Revision( $rec, self::READ_NORMAL, $title );
+		$rec = self::getRevisionLookup()->getPreviousRevision( $this->mRecord );
+		return $rec ? new Revision( $rec, self::READ_NORMAL, $this->getTitle() ) : null;
 	}
 
 	/**
@@ -1028,18 +822,24 @@ class Revision implements IDBAccessObject {
 	 * @return Revision|null
 	 */
 	public function getNext() {
-		$title = $this->getTitle();
-		$rec = self::getRevisionLookup()->getNextRevision( $this->mRecord, $title );
-		return $rec === null ? null : new Revision( $rec, self::READ_NORMAL, $title );
+		$rec = self::getRevisionLookup()->getNextRevision( $this->mRecord );
+		return $rec ? new Revision( $rec, self::READ_NORMAL, $this->getTitle() ) : null;
 	}
 
 	/**
 	 * Get revision text associated with an old or archive row
 	 *
-	 * Both the flags and the text field must be included. Including the old_id
+	 * If the text field is not included, this uses RevisionStore to load the appropriate slot
+	 * and return its serialized content. This is the default backwards-compatibility behavior
+	 * when reading from the MCR aware database schema is enabled. For this to work, either
+	 * the revision ID or the page ID must be included in the row.
+	 *
+	 * When using the old text field, the flags field must also be set. Including the old_id
 	 * field will activate cache usage as long as the $wiki parameter is not set.
 	 *
-	 * @param stdClass $row The text data
+	 * @deprecated since 1.32, use RevisionStore::newRevisionFromRow instead.
+	 *
+	 * @param stdClass $row The text data. If a falsy value is passed instead, false is returned.
 	 * @param string $prefix Table prefix (default 'old_')
 	 * @param string|bool $wiki The name of the wiki to load the revision text from
 	 *   (same as the wiki $row was loaded from) or false to indicate the local
@@ -1048,21 +848,53 @@ class Revision implements IDBAccessObject {
 	 * @return string|false Text the text requested or false on failure
 	 */
 	public static function getRevisionText( $row, $prefix = 'old_', $wiki = false ) {
+		global $wgMultiContentRevisionSchemaMigrationStage;
+
+		if ( !$row ) {
+			return false;
+		}
+
 		$textField = $prefix . 'text';
 		$flagsField = $prefix . 'flags';
 
+		if ( isset( $row->$textField ) ) {
+			if ( !( $wgMultiContentRevisionSchemaMigrationStage & SCHEMA_COMPAT_WRITE_OLD ) ) {
+				// The text field was read, but it's no longer being populated!
+				// We could gloss over this by using the text when it's there and loading
+				// if when it's not, but it seems preferable to complain loudly about a
+				// query that is no longer guaranteed to work reliably.
+				throw new LogicException(
+					'Cannot use ' . __METHOD__ . ' with the ' . $textField . ' field when'
+					. ' $wgMultiContentRevisionSchemaMigrationStage does not include'
+					. ' SCHEMA_COMPAT_WRITE_OLD. The field may not be populated for all revisions!'
+				);
+			}
+
+			$text = $row->$textField;
+		} else {
+			// Missing text field, we are probably looking at the MCR-enabled DB schema.
+
+			if ( !( $wgMultiContentRevisionSchemaMigrationStage & SCHEMA_COMPAT_WRITE_OLD ) ) {
+				// This method should no longer be used with the new schema. Ideally, we
+				// would already trigger a deprecation warning when SCHEMA_COMPAT_READ_NEW is set.
+				wfDeprecated( __METHOD__ . ' (MCR without SCHEMA_COMPAT_WRITE_OLD)', '1.32' );
+			}
+
+			$store = self::getRevisionStore( $wiki );
+			$rev = $prefix === 'ar_'
+				? $store->newRevisionFromArchiveRow( $row )
+				: $store->newRevisionFromRow( $row );
+
+			$content = $rev->getContent( SlotRecord::MAIN );
+			return $content ? $content->serialize() : false;
+		}
+
 		if ( isset( $row->$flagsField ) ) {
 			$flags = explode( ',', $row->$flagsField );
 		} else {
 			$flags = [];
 		}
 
-		if ( isset( $row->$textField ) ) {
-			$text = $row->$textField;
-		} else {
-			return false;
-		}
-
 		$cacheKey = isset( $row->old_id )
 			? SqlBlobStore::makeAddressFromTextId( $row->old_id )
 			: null;
@@ -1136,9 +968,7 @@ class Revision implements IDBAccessObject {
 		$rec = self::getRevisionStore()->insertRevisionOn( $this->mRecord, $dbw );
 
 		$this->mRecord = $rec;
-
-		// Avoid PHP 7.1 warning of passing $this by reference
-		$revision = $this;
+		Assert::postcondition( $this->mRecord !== null, 'Failed to acquire a RevisionRecord' );
 
 		return $rec->getId();
 	}
@@ -1175,7 +1005,7 @@ class Revision implements IDBAccessObject {
 
 		$comment = CommentStoreComment::newUnsavedComment( $summary, null );
 
-		$title = Title::newFromID( $pageId, Title::GAID_FOR_UPDATE );
+		$title = Title::newFromID( $pageId, Title::READ_LATEST );
 		if ( $title === null ) {
 			return null;
 		}
@@ -1228,13 +1058,13 @@ class Revision implements IDBAccessObject {
 	/**
 	 * Get rev_timestamp from rev_id, without loading the rest of the row
 	 *
-	 * @param Title $title
+	 * @param Title $title (ignored since 1.34)
 	 * @param int $id
 	 * @param int $flags
 	 * @return string|bool False if not found
 	 */
 	static function getTimestampFromId( $title, $id, $flags = 0 ) {
-		return self::getRevisionStore()->getTimestampFromId( $title, $id, $flags );
+		return self::getRevisionStore()->getTimestampFromId( $id, $flags );
 	}
 
 	/**
