@@ -63,7 +63,7 @@ class ApiQueryImageInfo extends ApiQueryBase {
 				$this->dieWithError( [ 'apierror-bad-badfilecontexttitle', $p ], 'invalid-title' );
 			}
 		} else {
-			$badFileContextTitle = false;
+			$badFileContextTitle = null;
 		}
 
 		$pageIds = $this->getPageSet()->getGoodAndMissingTitlesByNamespace();
@@ -110,7 +110,8 @@ class ApiQueryImageInfo extends ApiQueryBase {
 				if ( !isset( $images[$title] ) ) {
 					if ( isset( $prop['uploadwarning'] ) || isset( $prop['badfile'] ) ) {
 						// uploadwarning and badfile need info about non-existing files
-						$images[$title] = wfLocalFile( $title );
+						$images[$title] = MediaWikiServices::getInstance()->getRepoGroup()
+							->getLocalRepo()->newFile( $title );
 						// Doesn't exist, so set an empty image repository
 						$info['imagerepository'] = '';
 					} else {
@@ -143,7 +144,8 @@ class ApiQueryImageInfo extends ApiQueryBase {
 					$info['imagerepository'] = $img->getRepoName();
 				}
 				if ( isset( $prop['badfile'] ) ) {
-					$info['badfile'] = (bool)wfIsBadImage( $title, $badFileContextTitle );
+					$info['badfile'] = (bool)MediaWikiServices::getInstance()->getBadFileLookup()
+						->isBadFile( $title, $badFileContextTitle );
 				}
 
 				$fit = $result->addValue( [ 'query', 'pages' ], (int)$pageId, $info );
@@ -385,9 +387,13 @@ class ApiQueryImageInfo extends ApiQueryBase {
 		$vals = [
 			ApiResult::META_TYPE => 'assoc',
 		];
+
+		// Some information will be unavailable if the file does not exist. T221812
+		$exists = $file->exists();
+
 		// Timestamp is shown even if the file is revdelete'd in interface
 		// so do same here.
-		if ( isset( $prop['timestamp'] ) ) {
+		if ( isset( $prop['timestamp'] ) && $exists ) {
 			$vals['timestamp'] = wfTimestamp( TS_ISO_8601, $file->getTimestamp() );
 		}
 
@@ -406,7 +412,7 @@ class ApiQueryImageInfo extends ApiQueryBase {
 		$user = isset( $prop['user'] );
 		$userid = isset( $prop['userid'] );
 
-		if ( $user || $userid ) {
+		if ( ( $user || $userid ) && $exists ) {
 			if ( $file->isDeleted( File::DELETED_USER ) ) {
 				$vals['userhidden'] = true;
 				$anyHidden = true;
@@ -426,7 +432,7 @@ class ApiQueryImageInfo extends ApiQueryBase {
 
 		// This is shown even if the file is revdelete'd in interface
 		// so do same here.
-		if ( isset( $prop['size'] ) || isset( $prop['dimensions'] ) ) {
+		if ( ( isset( $prop['size'] ) || isset( $prop['dimensions'] ) ) && $exists ) {
 			$vals['size'] = (int)$file->getSize();
 			$vals['width'] = (int)$file->getWidth();
 			$vals['height'] = (int)$file->getHeight();
@@ -447,7 +453,7 @@ class ApiQueryImageInfo extends ApiQueryBase {
 		$pcomment = isset( $prop['parsedcomment'] );
 		$comment = isset( $prop['comment'] );
 
-		if ( $pcomment || $comment ) {
+		if ( ( $pcomment || $comment ) && $exists ) {
 			if ( $file->isDeleted( File::DELETED_COMMENT ) ) {
 				$vals['commenthidden'] = true;
 				$anyHidden = true;
@@ -498,7 +504,7 @@ class ApiQueryImageInfo extends ApiQueryBase {
 		}
 
 		if ( $url ) {
-			if ( $file->exists() ) {
+			if ( $exists ) {
 				if ( !is_null( $thumbParams ) ) {
 					$mto = $file->transform( $thumbParams );
 					self::$transformCount++;
@@ -521,12 +527,12 @@ class ApiQueryImageInfo extends ApiQueryBase {
 							$vals['thumbmime'] = $mime;
 						}
 					} elseif ( $mto && $mto->isError() ) {
+						/** @var MediaTransformError $mto */
+						'@phan-var MediaTransformError $mto';
 						$vals['thumberror'] = $mto->toText();
 					}
 				}
 				$vals['url'] = wfExpandUrl( $file->getFullUrl(), PROTO_CURRENT );
-			} else {
-				$vals['filemissing'] = true;
 			}
 			$vals['descriptionurl'] = wfExpandUrl( $file->getDescriptionUrl(), PROTO_CURRENT );
 
@@ -536,11 +542,15 @@ class ApiQueryImageInfo extends ApiQueryBase {
 			}
 		}
 
-		if ( $sha1 ) {
+		if ( !$exists ) {
+			$vals['filemissing'] = true;
+		}
+
+		if ( $sha1 && $exists ) {
 			$vals['sha1'] = Wikimedia\base_convert( $file->getSha1(), 36, 16, 40 );
 		}
 
-		if ( $meta ) {
+		if ( $meta && $exists ) {
 			Wikimedia\suppressWarnings();
 			$metadata = unserialize( $file->getMetadata() );
 			Wikimedia\restoreWarnings();
@@ -549,17 +559,18 @@ class ApiQueryImageInfo extends ApiQueryBase {
 			}
 			$vals['metadata'] = $metadata ? static::processMetaData( $metadata, $result ) : null;
 		}
-		if ( $commonmeta ) {
+		if ( $commonmeta && $exists ) {
 			$metaArray = $file->getCommonMetaArray();
 			$vals['commonmetadata'] = $metaArray ? static::processMetaData( $metaArray, $result ) : [];
 		}
 
-		if ( $extmetadata ) {
+		if ( $extmetadata && $exists ) {
 			// Note, this should return an array where all the keys
 			// start with a letter, and all the values are strings.
 			// Thus there should be no issue with format=xml.
 			$format = new FormatMetadata;
 			$format->setSingleLanguage( !$opts['multilang'] );
+			// @phan-suppress-next-line PhanUndeclaredMethod
 			$format->getContext()->setLanguage( $opts['language'] );
 			$extmetaArray = $format->fetchExtendedMetadata( $file );
 			if ( $opts['extmetadatafilter'] ) {
@@ -570,19 +581,21 @@ class ApiQueryImageInfo extends ApiQueryBase {
 			$vals['extmetadata'] = $extmetaArray;
 		}
 
-		if ( $mime ) {
+		if ( $mime && $exists ) {
 			$vals['mime'] = $file->getMimeType();
 		}
 
-		if ( $mediatype ) {
+		if ( $mediatype && $exists ) {
 			$vals['mediatype'] = $file->getMediaType();
 		}
 
 		if ( $archive && $file->isOld() ) {
+			/** @var OldLocalFile $file */
+			'@phan-var OldLocalFile $file';
 			$vals['archivename'] = $file->getArchiveName();
 		}
 
-		if ( $bitdepth ) {
+		if ( $bitdepth && $exists ) {
 			$vals['bitdepth'] = $file->getBitDepth();
 		}
 
@@ -756,58 +769,6 @@ class ApiQueryImageInfo extends ApiQueryBase {
 		);
 	}
 
-	/**
-	 * Returns array key value pairs of properties and their descriptions
-	 *
-	 * @deprecated since 1.25
-	 * @param string $modulePrefix
-	 * @return array
-	 */
-	private static function getProperties( $modulePrefix = '' ) {
-		return [
-			'timestamp' => ' timestamp     - Adds timestamp for the uploaded version',
-			'user' => ' user          - Adds the user who uploaded the image version',
-			'userid' => ' userid        - Add the user ID that uploaded the image version',
-			'comment' => ' comment       - Comment on the version',
-			'parsedcomment' => ' parsedcomment - Parse the comment on the version',
-			'canonicaltitle' => ' canonicaltitle - Adds the canonical title of the image file',
-			'url' => ' url           - Gives URL to the image and the description page',
-			'size' => ' size          - Adds the size of the image in bytes, ' .
-				'its height and its width. Page count and duration are added if applicable',
-			'dimensions' => ' dimensions    - Alias for size', // B/C with Allimages
-			'sha1' => ' sha1          - Adds SHA-1 hash for the image',
-			'mime' => ' mime          - Adds MIME type of the image',
-			'thumbmime' => ' thumbmime     - Adds MIME type of the image thumbnail' .
-				' (requires url and param ' . $modulePrefix . 'urlwidth)',
-			'mediatype' => ' mediatype     - Adds the media type of the image',
-			'metadata' => ' metadata      - Lists Exif metadata for the version of the image',
-			'commonmetadata' => ' commonmetadata - Lists file format generic metadata ' .
-				'for the version of the image',
-			'extmetadata' => ' extmetadata   - Lists formatted metadata combined ' .
-				'from multiple sources. Results are HTML formatted.',
-			'archivename' => ' archivename   - Adds the file name of the archive ' .
-				'version for non-latest versions',
-			'bitdepth' => ' bitdepth      - Adds the bit depth of the version',
-			'uploadwarning' => ' uploadwarning - Used by the Special:Upload page to ' .
-				'get information about an existing file. Not intended for use outside MediaWiki core',
-		];
-	}
-
-	/**
-	 * Returns the descriptions for the properties provided by getPropertyNames()
-	 *
-	 * @deprecated since 1.25
-	 * @param array $filter List of properties to filter out
-	 * @param string $modulePrefix
-	 * @return array
-	 */
-	public static function getPropertyDescriptions( $filter = [], $modulePrefix = '' ) {
-		return array_merge(
-			[ 'What image information to get:' ],
-			array_values( array_diff_key( static::getProperties( $modulePrefix ), array_flip( $filter ) ) )
-		);
-	}
-
 	protected function getExamplesMessages() {
 		return [
 			'action=query&titles=File:Albert%20Einstein%20Head.jpg&prop=imageinfo'
