@@ -45,6 +45,9 @@ class CustomUppercaseCollation extends NumericUppercaseCollation {
 	/** @var array $puaSubset List of private use area codes */
 	private $puaSubset;
 
+	/** @var array */
+	private $firstLetters;
+
 	/**
 	 * @note This assumes $alphabet does not contain U+F3000-U+F3FFF
 	 *
