@@ -117,12 +117,34 @@ class FileBackendGroup {
 			}
 			$name = $config['name'];
 			if ( isset( $this->backends[$name] ) ) {
-				throw new LogicException( "Backend with name `{$name}` already registered." );
+				throw new LogicException( "Backend with name '$name' already registered." );
 			} elseif ( !isset( $config['class'] ) ) {
-				throw new InvalidArgumentException( "Backend with name `{$name}` has no class." );
+				throw new InvalidArgumentException( "Backend with name '$name' has no class." );
 			}
 			$class = $config['class'];
 
+			if ( isset( $config['domainId'] ) ) {
+				$domainId = $config['domainId'];
+			} elseif ( isset( $config['wikiId'] ) ) {
+				$domainId = $config['wikiId']; // b/c
+			} else {
+				// Only use the raw database/prefix for backwards compatibility
+				$ld = WikiMap::getCurrentWikiDbDomain();
+				$domainId = strlen( $ld->getTablePrefix() )
+					? "{$ld->getDatabase()}-{$ld->getTablePrefix()}"
+					: $ld->getDatabase();
+				// If the local wiki ID and local domain ID do not match, probably due to a
+				// non-default schema, issue a warning. A non-default schema indicates that
+				// it might be used to disambiguate different wikis.
+				$wikiId = WikiMap::getWikiIdFromDbDomain( $ld );
+				if ( $ld->getSchema() !== null && $domainId !== $wikiId ) {
+					wfWarn(
+						"\$wgFileBackend entry '$name' should have 'domainId' set.\n" .
+						"Legacy default 'domainId' is '$domainId' but wiki ID is '$wikiId'."
+					);
+				}
+			}
+			$config['domainId'] = $domainId;
 			$config['readOnly'] = $config['readOnly'] ?? $readOnlyReason;
 
 			unset( $config['class'] ); // backend won't need this
@@ -148,6 +170,7 @@ class FileBackendGroup {
 
 			$class = $config['class'];
 			if ( $class === FileBackendMultiWrite::class ) {
+				// @todo How can we test this? What's the intended use-case?
 				foreach ( $config['backends'] as $index => $beConfig ) {
 					if ( isset( $beConfig['template'] ) ) {
 						// Config is just a modified version of a registered backend's.
@@ -172,40 +195,40 @@ class FileBackendGroup {
 	 */
 	public function config( $name ) {
 		if ( !isset( $this->backends[$name] ) ) {
-			throw new InvalidArgumentException( "No backend defined with the name `$name`." );
+			throw new InvalidArgumentException( "No backend defined with the name '$name'." );
 		}
-		$class = $this->backends[$name]['class'];
 
 		$config = $this->backends[$name]['config'];
-		$config['class'] = $class;
-		if ( isset( $config['domainId'] ) ) {
-			$domain = $config['domainId'];
-		} else {
-			// @FIXME: this does not include the domain for b/c but it ideally should
-			$domain = $config['wikiId'] ?? wfWikiID();
-		}
-		// Set default parameter values
-		$config += [
-			'domainId' => $domain, // e.g. "my_wiki-en_"
-			'mimeCallback' => [ $this, 'guessMimeInternal' ],
-			'obResetFunc' => 'wfResetOutputBuffers',
-			'streamMimeFunc' => [ StreamFile::class, 'contentTypeFromPath' ],
-			'tmpDirectory' => wfTempDir(),
-			'statusWrapper' => [ Status::class, 'wrap' ],
-			'wanCache' => MediaWikiServices::getInstance()->getMainWANObjectCache(),
-			'srvCache' => ObjectCache::getLocalServerInstance( 'hash' ),
-			'logger' => LoggerFactory::getInstance( 'FileOperation' ),
-			'profiler' => function ( $section ) {
-				return Profiler::instance()->scopedProfileIn( $section );
-			}
-		];
-		$config['lockManager'] =
-			LockManagerGroup::singleton( $domain )->get( $config['lockManager'] );
-		$config['fileJournal'] = isset( $config['fileJournal'] )
-			? FileJournal::factory( $config['fileJournal'], $name )
-			: FileJournal::factory( [ 'class' => NullFileJournal::class ], $name );
+		$services = MediaWikiServices::getInstance();
 
-		return $config;
+		return array_merge(
+			// Default backend parameters
+			[
+				'mimeCallback' => [ $this, 'guessMimeInternal' ],
+				'obResetFunc' => 'wfResetOutputBuffers',
+				'streamMimeFunc' => [ StreamFile::class, 'contentTypeFromPath' ],
+				'tmpFileFactory' => $services->getTempFSFileFactory(),
+				'statusWrapper' => [ Status::class, 'wrap' ],
+				'wanCache' => $services->getMainWANObjectCache(),
+				'srvCache' => ObjectCache::getLocalServerInstance( 'hash' ),
+				'logger' => LoggerFactory::getInstance( 'FileOperation' ),
+				'profiler' => function ( $section ) {
+					return Profiler::instance()->scopedProfileIn( $section );
+				}
+			],
+			// Configured backend parameters
+			$config,
+			// Resolved backend parameters
+			[
+				'class' => $this->backends[$name]['class'],
+				'lockManager' =>
+					LockManagerGroup::singleton( $config['domainId'] )
+						->get( $config['lockManager'] ),
+				'fileJournal' => isset( $config['fileJournal'] )
+					? FileJournal::factory( $config['fileJournal'], $name )
+					: FileJournal::factory( [ 'class' => NullFileJournal::class ], $name )
+			]
+		);
 	}
 
 	/**
@@ -239,7 +262,8 @@ class FileBackendGroup {
 		if ( !$type && $fsPath ) {
 			$type = $magic->guessMimeType( $fsPath, false );
 		} elseif ( !$type && strlen( $content ) ) {
-			$tmpFile = TempFSFile::factory( 'mime_', '', wfTempDir() );
+			$tmpFile = MediaWikiServices::getInstance()->getTempFSFileFactory()
+				->newTempFSFile( 'mime_', '' );
 			file_put_contents( $tmpFile->getPath(), $content );
 			$type = $magic->guessMimeType( $tmpFile->getPath(), false );
 		}
