@@ -35,6 +35,9 @@ class RepoGroup {
 	/** @var FileRepo[] */
 	protected $foreignRepos;
 
+	/** @var WANObjectCache */
+	protected $wanCache;
+
 	/** @var bool */
 	protected $reposInitialised = false;
 
@@ -44,69 +47,63 @@ class RepoGroup {
 	/** @var array */
 	protected $foreignInfo;
 
-	/** @var ProcessCacheLRU */
+	/** @var MapCacheLRU */
 	protected $cache;
 
-	/** @var RepoGroup */
-	protected static $instance;
-
 	/** Maximum number of cache items */
 	const MAX_CACHE_SIZE = 500;
 
 	/**
-	 * Get a RepoGroup instance. At present only one instance of RepoGroup is
-	 * needed in a MediaWiki invocation, this may change in the future.
+	 * @deprecated since 1.34, use MediaWikiServices::getRepoGroup
 	 * @return RepoGroup
 	 */
 	static function singleton() {
-		if ( self::$instance ) {
-			return self::$instance;
-		}
-		global $wgLocalFileRepo, $wgForeignFileRepos;
-		/** @var array $wgLocalFileRepo */
-		self::$instance = new RepoGroup( $wgLocalFileRepo, $wgForeignFileRepos );
-
-		return self::$instance;
+		return MediaWikiServices::getInstance()->getRepoGroup();
 	}
 
 	/**
-	 * Destroy the singleton instance, so that a new one will be created next
-	 * time singleton() is called.
+	 * @deprecated since 1.34, use MediaWikiTestCase::overrideMwServices() or similar. This will
+	 * cause bugs if you don't reset all other services that depend on this one at the same time.
 	 */
 	static function destroySingleton() {
-		self::$instance = null;
+		MediaWikiServices::getInstance()->resetServiceForTesting( 'RepoGroup' );
 	}
 
 	/**
-	 * Set the singleton instance to a given object
-	 * Used by extensions which hook into the Repo chain.
-	 * It's not enough to just create a superclass ... you have
-	 * to get people to call into it even though all they know is RepoGroup::singleton()
-	 *
+	 * @deprecated since 1.34, use MediaWikiTestCase::setService, this can mess up state of other
+	 *   tests
 	 * @param RepoGroup $instance
 	 */
 	static function setSingleton( $instance ) {
-		self::$instance = $instance;
+		$services = MediaWikiServices::getInstance();
+		$services->disableService( 'RepoGroup' );
+		$services->redefineService( 'RepoGroup',
+			function () use ( $instance ) {
+				return $instance;
+			}
+		);
 	}
 
 	/**
-	 * Construct a group of file repositories.
+	 * Construct a group of file repositories. Do not call this -- use
+	 * MediaWikiServices::getRepoGroup.
 	 *
 	 * @param array $localInfo Associative array for local repo's info
 	 * @param array $foreignInfo Array of repository info arrays.
 	 *   Each info array is an associative array with the 'class' member
 	 *   giving the class name. The entire array is passed to the repository
 	 *   constructor as the first parameter.
+	 * @param WANObjectCache $wanCache
 	 */
-	function __construct( $localInfo, $foreignInfo ) {
+	function __construct( $localInfo, $foreignInfo, $wanCache ) {
 		$this->localInfo = $localInfo;
 		$this->foreignInfo = $foreignInfo;
 		$this->cache = new MapCacheLRU( self::MAX_CACHE_SIZE );
+		$this->wanCache = $wanCache;
 	}
 
 	/**
 	 * Search repositories for an image.
-	 * You can also use wfFindFile() to do this.
 	 *
 	 * @param Title|string $title Title object or string
 	 * @param array $options Associative array of options:
@@ -118,6 +115,7 @@ class RepoGroup {
 	 *                   user is allowed to view them. Otherwise, such files will not
 	 *                   be found.
 	 *   latest:         If true, load from the latest available data into File objects
+	 * @phan-param array{time?:mixed,ignoreRedirect?:bool,private?:bool,latest?:bool} $options
 	 * @return File|bool False if title is not found
 	 */
 	function findFile( $title, $options = [] ) {
@@ -419,8 +417,7 @@ class RepoGroup {
 	protected function newRepo( $info ) {
 		$class = $info['class'];
 
-		$cache = MediaWikiServices::getInstance()->getMainWANObjectCache();
-		$info['wanCache'] = $cache;
+		$info['wanCache'] = $this->wanCache;
 
 		return new $class( $info );
 	}
