@@ -76,15 +76,15 @@ class HTMLInfoField extends HTMLFormField {
 	}
 
 	/**
-	 * @param mixed $value
+	 * @param mixed $value If not FieldLayout or subclass has been deprecated.
 	 * @return OOUI\FieldLayout
 	 * @since 1.32
 	 */
 	public function getOOUI( $value ) {
 		if ( !empty( $this->mParams['rawrow'] ) ) {
 			if ( !( $value instanceof OOUI\FieldLayout ) ) {
-				wfDeprecated( "'default' parameter as a string when using 'rawrow' " .
-					"(must be a FieldLayout or subclass)", '1.32' );
+				wfDeprecated( __METHOD__ . ": 'default' parameter as a string when using" .
+					"'rawrow' (must be a FieldLayout or subclass)", '1.32' );
 			}
 			return $value;
 		}
