@@ -20,10 +20,11 @@
  * @file
  */
 use Wikimedia\Rdbms\IDatabase;
-use Wikimedia\Rdbms\DBConnRef;
+use Wikimedia\Rdbms\Database;
 use Wikimedia\Rdbms\DBConnectionError;
 use Wikimedia\Rdbms\DBError;
 use MediaWiki\MediaWikiServices;
+use Wikimedia\Rdbms\IMaintainableDatabase;
 use Wikimedia\ScopedCallback;
 
 /**
@@ -38,14 +39,17 @@ class JobQueueDB extends JobQueue {
 	const MAX_JOB_RANDOM = 2147483647; // integer; 2^31 - 1, used for job_random
 	const MAX_OFFSET = 255; // integer; maximum number of rows to skip
 
-	/** @var WANObjectCache */
-	protected $cache;
+	/** @var IMaintainableDatabase|DBError|null */
+	protected $conn;
 
-	/** @var bool|string Name of an external DB cluster. False if not set */
-	protected $cluster = false;
+	/** @var array|null Server configuration array */
+	protected $server;
+	/** @var string|null Name of an external DB cluster or null for the local DB cluster */
+	protected $cluster;
 
 	/**
 	 * Additional parameters include:
+	 *   - server  : Server configuration array for Database::factory. Overrides "cluster".
 	 *   - cluster : The name of an external cluster registered via LBFactory.
 	 *               If not specified, the primary DB cluster for the wiki will be used.
 	 *               This can be overridden with a custom cluster so that DB handles will
@@ -55,8 +59,11 @@ class JobQueueDB extends JobQueue {
 	protected function __construct( array $params ) {
 		parent::__construct( $params );
 
-		$this->cluster = $params['cluster'] ?? false;
-		$this->cache = ObjectCache::getMainWANInstance();
+		if ( isset( $params['server'] ) ) {
+			$this->server = $params['server'];
+		} elseif ( isset( $params['cluster'] ) && is_string( $params['cluster'] ) ) {
+			$this->cluster = $params['cluster'];
+		}
 	}
 
 	protected function supportedOrders() {
@@ -73,12 +80,14 @@ class JobQueueDB extends JobQueue {
 	 */
 	protected function doIsEmpty() {
 		$dbr = $this->getReplicaDB();
+		/** @noinspection PhpUnusedLocalVariableInspection */
+		$scope = $this->getScopedNoTrxFlag( $dbr );
 		try {
 			$found = $dbr->selectField( // unclaimed job
 				'job', '1', [ 'job_cmd' => $this->type, 'job_token' => '' ], __METHOD__
 			);
 		} catch ( DBError $e ) {
-			$this->throwDBException( $e );
+			throw $this->getDBException( $e );
 		}
 
 		return !$found;
@@ -91,21 +100,23 @@ class JobQueueDB extends JobQueue {
 	protected function doGetSize() {
 		$key = $this->getCacheKey( 'size' );
 
-		$size = $this->cache->get( $key );
+		$size = $this->wanCache->get( $key );
 		if ( is_int( $size ) ) {
 			return $size;
 		}
 
+		$dbr = $this->getReplicaDB();
+		/** @noinspection PhpUnusedLocalVariableInspection */
+		$scope = $this->getScopedNoTrxFlag( $dbr );
 		try {
-			$dbr = $this->getReplicaDB();
 			$size = (int)$dbr->selectField( 'job', 'COUNT(*)',
 				[ 'job_cmd' => $this->type, 'job_token' => '' ],
 				__METHOD__
 			);
 		} catch ( DBError $e ) {
-			$this->throwDBException( $e );
+			throw $this->getDBException( $e );
 		}
-		$this->cache->set( $key, $size, self::CACHE_TTL_SHORT );
+		$this->wanCache->set( $key, $size, self::CACHE_TTL_SHORT );
 
 		return $size;
 	}
@@ -121,21 +132,23 @@ class JobQueueDB extends JobQueue {
 
 		$key = $this->getCacheKey( 'acquiredcount' );
 
-		$count = $this->cache->get( $key );
+		$count = $this->wanCache->get( $key );
 		if ( is_int( $count ) ) {
 			return $count;
 		}
 
 		$dbr = $this->getReplicaDB();
+		/** @noinspection PhpUnusedLocalVariableInspection */
+		$scope = $this->getScopedNoTrxFlag( $dbr );
 		try {
 			$count = (int)$dbr->selectField( 'job', 'COUNT(*)',
 				[ 'job_cmd' => $this->type, "job_token != {$dbr->addQuotes( '' )}" ],
 				__METHOD__
 			);
 		} catch ( DBError $e ) {
-			$this->throwDBException( $e );
+			throw $this->getDBException( $e );
 		}
-		$this->cache->set( $key, $count, self::CACHE_TTL_SHORT );
+		$this->wanCache->set( $key, $count, self::CACHE_TTL_SHORT );
 
 		return $count;
 	}
@@ -152,12 +165,14 @@ class JobQueueDB extends JobQueue {
 
 		$key = $this->getCacheKey( 'abandonedcount' );
 
-		$count = $this->cache->get( $key );
+		$count = $this->wanCache->get( $key );
 		if ( is_int( $count ) ) {
 			return $count;
 		}
 
 		$dbr = $this->getReplicaDB();
+		/** @noinspection PhpUnusedLocalVariableInspection */
+		$scope = $this->getScopedNoTrxFlag( $dbr );
 		try {
 			$count = (int)$dbr->selectField( 'job', 'COUNT(*)',
 				[
@@ -168,10 +183,10 @@ class JobQueueDB extends JobQueue {
 				__METHOD__
 			);
 		} catch ( DBError $e ) {
-			$this->throwDBException( $e );
+			throw $this->getDBException( $e );
 		}
 
-		$this->cache->set( $key, $count, self::CACHE_TTL_SHORT );
+		$this->wanCache->set( $key, $count, self::CACHE_TTL_SHORT );
 
 		return $count;
 	}
@@ -185,6 +200,8 @@ class JobQueueDB extends JobQueue {
 	 */
 	protected function doBatchPush( array $jobs, $flags ) {
 		$dbw = $this->getMasterDB();
+		/** @noinspection PhpUnusedLocalVariableInspection */
+		$scope = $this->getScopedNoTrxFlag( $dbw );
 		// In general, there will be two cases here:
 		// a) sqlite; DB connection is probably a regular round-aware handle.
 		// If the connection is busy with a transaction, then defer the job writes
@@ -206,6 +223,7 @@ class JobQueueDB extends JobQueue {
 	/**
 	 * This function should *not* be called outside of JobQueueDB
 	 *
+	 * @suppress SecurityCheck-SQLInjection Bug in phan-taint-check handling bulk inserts
 	 * @param IDatabase $dbw
 	 * @param IJobSpecification[] $jobs
 	 * @param int $flags
@@ -214,7 +232,7 @@ class JobQueueDB extends JobQueue {
 	 * @return void
 	 */
 	public function doBatchPushInternal( IDatabase $dbw, array $jobs, $flags, $method ) {
-		if ( !count( $jobs ) ) {
+		if ( $jobs === [] ) {
 			return;
 		}
 
@@ -254,12 +272,12 @@ class JobQueueDB extends JobQueue {
 			foreach ( array_chunk( $rows, 50 ) as $rowBatch ) {
 				$dbw->insert( 'job', $rowBatch, $method );
 			}
-			JobQueue::incrStats( 'inserts', $this->type, count( $rows ) );
-			JobQueue::incrStats( 'dupe_inserts', $this->type,
+			$this->incrStats( 'inserts', $this->type, count( $rows ) );
+			$this->incrStats( 'dupe_inserts', $this->type,
 				count( $rowSet ) + count( $rowList ) - count( $rows )
 			);
 		} catch ( DBError $e ) {
-			$this->throwDBException( $e );
+			throw $this->getDBException( $e );
 		}
 		if ( $flags & self::QOS_ATOMIC ) {
 			$dbw->endAtomic( $method );
@@ -268,19 +286,16 @@ class JobQueueDB extends JobQueue {
 
 	/**
 	 * @see JobQueue::doPop()
-	 * @return Job|bool
+	 * @return RunnableJob|bool
 	 */
 	protected function doPop() {
 		$dbw = $this->getMasterDB();
-		try {
-			$autoTrx = $dbw->getFlag( DBO_TRX ); // get current setting
-			$dbw->clearFlag( DBO_TRX ); // make each query its own transaction
-			$scopedReset = new ScopedCallback( function () use ( $dbw, $autoTrx ) {
-				$dbw->setFlag( $autoTrx ? DBO_TRX : 0 ); // restore old setting
-			} );
+		/** @noinspection PhpUnusedLocalVariableInspection */
+		$scope = $this->getScopedNoTrxFlag( $dbw );
 
+		$job = false; // job popped off
+		try {
 			$uuid = wfRandomString( 32 ); // pop attempt
-			$job = false; // job popped off
 			do { // retry when our row is invalid or deleted as a duplicate
 				// Try to reserve a row in the DB...
 				if ( in_array( $this->order, [ 'fifo', 'timestamp' ] ) ) {
@@ -294,13 +309,10 @@ class JobQueueDB extends JobQueue {
 				if ( !$row ) {
 					break; // nothing to do
 				}
-				JobQueue::incrStats( 'pops', $this->type );
+				$this->incrStats( 'pops', $this->type );
+
 				// Get the job object from the row...
-				$title = Title::makeTitle( $row->job_namespace, $row->job_title );
-				$job = Job::factory( $row->job_cmd, $title,
-					self::extractBlob( $row->job_params ), $row->job_id );
-				$job->metadata['id'] = $row->job_id;
-				$job->metadata['timestamp'] = $row->job_timestamp;
+				$job = $this->jobFromRow( $row );
 				break; // done
 			} while ( true );
 
@@ -310,7 +322,7 @@ class JobQueueDB extends JobQueue {
 				$this->recycleAndDeleteStaleJobs();
 			}
 		} catch ( DBError $e ) {
-			$this->throwDBException( $e );
+			throw $this->getDBException( $e );
 		}
 
 		return $job;
@@ -326,10 +338,11 @@ class JobQueueDB extends JobQueue {
 	 */
 	protected function claimRandom( $uuid, $rand, $gte ) {
 		$dbw = $this->getMasterDB();
+		/** @noinspection PhpUnusedLocalVariableInspection */
+		$scope = $this->getScopedNoTrxFlag( $dbw );
 		// Check cache to see if the queue has <= OFFSET items
-		$tinyQueue = $this->cache->get( $this->getCacheKey( 'small' ) );
+		$tinyQueue = $this->wanCache->get( $this->getCacheKey( 'small' ) );
 
-		$row = false; // the row acquired
 		$invertedDirection = false; // whether one job_random direction was already scanned
 		// This uses a replication safe method for acquiring jobs. One could use UPDATE+LIMIT
 		// instead, but that either uses ORDER BY (in which case it deadlocks in MySQL) or is
@@ -368,7 +381,7 @@ class JobQueueDB extends JobQueue {
 				);
 				if ( !$row ) {
 					$tinyQueue = true; // we know the queue must have <= MAX_OFFSET rows
-					$this->cache->set( $this->getCacheKey( 'small' ), 1, 30 );
+					$this->wanCache->set( $this->getCacheKey( 'small' ), 1, 30 );
 					continue; // use job_random
 				}
 			}
@@ -403,13 +416,15 @@ class JobQueueDB extends JobQueue {
 	 */
 	protected function claimOldest( $uuid ) {
 		$dbw = $this->getMasterDB();
+		/** @noinspection PhpUnusedLocalVariableInspection */
+		$scope = $this->getScopedNoTrxFlag( $dbw );
 
 		$row = false; // the row acquired
 		do {
 			if ( $dbw->getType() === 'mysql' ) {
 				// Per https://bugs.mysql.com/bug.php?id=6980, we can't use subqueries on the
 				// same table being changed in an UPDATE query in MySQL (gives Error: 1093).
-				// Oracle and Postgre have no such limitation. However, MySQL offers an
+				// Postgres has no such limitation. However, MySQL offers an
 				// alternative here by supporting ORDER BY + LIMIT for UPDATE queries.
 				$dbw->query( "UPDATE {$dbw->tableName( 'job' )} " .
 					"SET " .
@@ -458,29 +473,29 @@ class JobQueueDB extends JobQueue {
 
 	/**
 	 * @see JobQueue::doAck()
-	 * @param Job $job
+	 * @param RunnableJob $job
 	 * @throws MWException
 	 */
-	protected function doAck( Job $job ) {
-		if ( !isset( $job->metadata['id'] ) ) {
+	protected function doAck( RunnableJob $job ) {
+		$id = $job->getMetadata( 'id' );
+		if ( $id === null ) {
 			throw new MWException( "Job of type '{$job->getType()}' has no ID." );
 		}
 
 		$dbw = $this->getMasterDB();
+		/** @noinspection PhpUnusedLocalVariableInspection */
+		$scope = $this->getScopedNoTrxFlag( $dbw );
 		try {
-			$autoTrx = $dbw->getFlag( DBO_TRX ); // get current setting
-			$dbw->clearFlag( DBO_TRX ); // make each query its own transaction
-			$scopedReset = new ScopedCallback( function () use ( $dbw, $autoTrx ) {
-				$dbw->setFlag( $autoTrx ? DBO_TRX : 0 ); // restore old setting
-			} );
-
 			// Delete a row with a single DELETE without holding row locks over RTTs...
-			$dbw->delete( 'job',
-				[ 'job_cmd' => $this->type, 'job_id' => $job->metadata['id'] ], __METHOD__ );
+			$dbw->delete(
+				'job',
+				[ 'job_cmd' => $this->type, 'job_id' => $id ],
+				__METHOD__
+			);
 
-			JobQueue::incrStats( 'acks', $this->type );
+			$this->incrStats( 'acks', $this->type );
 		} catch ( DBError $e ) {
-			$this->throwDBException( $e );
+			throw $this->getDBException( $e );
 		}
 	}
 
@@ -491,29 +506,17 @@ class JobQueueDB extends JobQueue {
 	 * @return bool
 	 */
 	protected function doDeduplicateRootJob( IJobSpecification $job ) {
-		$params = $job->getParams();
-		if ( !isset( $params['rootJobSignature'] ) ) {
-			throw new MWException( "Cannot register root job; missing 'rootJobSignature'." );
-		} elseif ( !isset( $params['rootJobTimestamp'] ) ) {
-			throw new MWException( "Cannot register root job; missing 'rootJobTimestamp'." );
-		}
-		$key = $this->getRootJobCacheKey( $params['rootJobSignature'] );
-		// Callers should call JobQueueGroup::push() before this method so that if the insert
-		// fails, the de-duplication registration will be aborted. Since the insert is
-		// deferred till "transaction idle", do the same here, so that the ordering is
+		// Callers should call JobQueueGroup::push() before this method so that if the
+		// insert fails, the de-duplication registration will be aborted. Since the insert
+		// is deferred till "transaction idle", do the same here, so that the ordering is
 		// maintained. Having only the de-duplication registration succeed would cause
 		// jobs to become no-ops without any actual jobs that made them redundant.
 		$dbw = $this->getMasterDB();
-		$cache = $this->dupCache;
+		/** @noinspection PhpUnusedLocalVariableInspection */
+		$scope = $this->getScopedNoTrxFlag( $dbw );
 		$dbw->onTransactionCommitOrIdle(
-			function () use ( $cache, $params, $key ) {
-				$timestamp = $cache->get( $key ); // current last timestamp of this job
-				if ( $timestamp && $timestamp >= $params['rootJobTimestamp'] ) {
-					return true; // a newer version of this root job was enqueued
-				}
-
-				// Update the timestamp of the last root job started at the location...
-				return $cache->set( $key, $params['rootJobTimestamp'], JobQueueDB::ROOTJOB_TTL );
+			function () use ( $job ) {
+				parent::doDeduplicateRootJob( $job );
 			},
 			__METHOD__
 		);
@@ -527,10 +530,12 @@ class JobQueueDB extends JobQueue {
 	 */
 	protected function doDelete() {
 		$dbw = $this->getMasterDB();
+		/** @noinspection PhpUnusedLocalVariableInspection */
+		$scope = $this->getScopedNoTrxFlag( $dbw );
 		try {
 			$dbw->delete( 'job', [ 'job_cmd' => $this->type ] );
 		} catch ( DBError $e ) {
-			$this->throwDBException( $e );
+			throw $this->getDBException( $e );
 		}
 
 		return true;
@@ -541,8 +546,15 @@ class JobQueueDB extends JobQueue {
 	 * @return void
 	 */
 	protected function doWaitForBackups() {
+		if ( $this->server ) {
+			return; // not using LBFactory instance
+		}
+
 		$lbFactory = MediaWikiServices::getInstance()->getDBLoadBalancerFactory();
-		$lbFactory->waitForReplication( [ 'wiki' => $this->wiki, 'cluster' => $this->cluster ] );
+		$lbFactory->waitForReplication( [
+			'domain' => $this->domain,
+			'cluster' => is_string( $this->cluster ) ? $this->cluster : false
+		] );
 	}
 
 	/**
@@ -550,7 +562,7 @@ class JobQueueDB extends JobQueue {
 	 */
 	protected function doFlushCaches() {
 		foreach ( [ 'size', 'acquiredcount' ] as $type ) {
-			$this->cache->delete( $this->getCacheKey( $type ) );
+			$this->wanCache->delete( $this->getCacheKey( $type ) );
 		}
 	}
 
@@ -576,34 +588,34 @@ class JobQueueDB extends JobQueue {
 	 */
 	protected function getJobIterator( array $conds ) {
 		$dbr = $this->getReplicaDB();
+		/** @noinspection PhpUnusedLocalVariableInspection */
+		$scope = $this->getScopedNoTrxFlag( $dbr );
 		try {
 			return new MappedIterator(
 				$dbr->select( 'job', self::selectFields(), $conds ),
 				function ( $row ) {
-					$job = Job::factory(
-						$row->job_cmd,
-						Title::makeTitle( $row->job_namespace, $row->job_title ),
-						strlen( $row->job_params ) ? unserialize( $row->job_params ) : []
-					);
-					$job->metadata['id'] = $row->job_id;
-					$job->metadata['timestamp'] = $row->job_timestamp;
-
-					return $job;
+					return $this->jobFromRow( $row );
 				}
 			);
 		} catch ( DBError $e ) {
-			$this->throwDBException( $e );
+			throw $this->getDBException( $e );
 		}
 	}
 
 	public function getCoalesceLocationInternal() {
-		return $this->cluster
-			? "DBCluster:{$this->cluster}:{$this->wiki}"
-			: "LBFactory:{$this->wiki}";
+		if ( $this->server ) {
+			return null; // not using the LBFactory instance
+		}
+
+		return is_string( $this->cluster )
+			? "DBCluster:{$this->cluster}:{$this->domain}"
+			: "LBFactory:{$this->domain}";
 	}
 
 	protected function doGetSiblingQueuesWithJobs( array $types ) {
 		$dbr = $this->getReplicaDB();
+		/** @noinspection PhpUnusedLocalVariableInspection */
+		$scope = $this->getScopedNoTrxFlag( $dbr );
 		// @note: this does not check whether the jobs are claimed or not.
 		// This is useful so JobQueueGroup::pop() also sees queues that only
 		// have stale jobs. This lets recycleAndDeleteStaleJobs() re-enqueue
@@ -621,6 +633,9 @@ class JobQueueDB extends JobQueue {
 
 	protected function doGetSiblingQueueSizes( array $types ) {
 		$dbr = $this->getReplicaDB();
+		/** @noinspection PhpUnusedLocalVariableInspection */
+		$scope = $this->getScopedNoTrxFlag( $dbr );
+
 		$res = $dbr->select( 'job', [ 'job_cmd', 'COUNT(*) AS count' ],
 			[ 'job_cmd' => $types ], __METHOD__, [ 'GROUP BY' => 'job_cmd' ] );
 
@@ -641,6 +656,8 @@ class JobQueueDB extends JobQueue {
 		$now = time();
 		$count = 0; // affected rows
 		$dbw = $this->getMasterDB();
+		/** @noinspection PhpUnusedLocalVariableInspection */
+		$scope = $this->getScopedNoTrxFlag( $dbw );
 
 		try {
 			if ( !$dbw->lock( "jobqueue-recycle-{$this->type}", __METHOD__, 1 ) ) {
@@ -673,15 +690,14 @@ class JobQueueDB extends JobQueue {
 					$dbw->update( 'job',
 						[
 							'job_token' => '',
-							'job_token_timestamp' => $dbw->timestamp( $now ) ], // time of release
-						[
-							'job_id' => $ids ],
+							'job_token_timestamp' => $dbw->timestamp( $now ) // time of release
+						],
+						[ 'job_id' => $ids, "job_token != ''" ],
 						__METHOD__
 					);
 					$affected = $dbw->affectedRows();
 					$count += $affected;
-					JobQueue::incrStats( 'recycles', $this->type, $affected );
-					$this->aggr->notifyQueueNonEmpty( $this->wiki, $this->type );
+					$this->incrStats( 'recycles', $this->type, $affected );
 				}
 			}
 
@@ -707,12 +723,12 @@ class JobQueueDB extends JobQueue {
 				$dbw->delete( 'job', [ 'job_id' => $ids ], __METHOD__ );
 				$affected = $dbw->affectedRows();
 				$count += $affected;
-				JobQueue::incrStats( 'abandons', $this->type, $affected );
+				$this->incrStats( 'abandons', $this->type, $affected );
 			}
 
 			$dbw->unlock( "jobqueue-recycle-{$this->type}", __METHOD__ );
 		} catch ( DBError $e ) {
-			$this->throwDBException( $e );
+			throw $this->getDBException( $e );
 		}
 
 		return $count;
@@ -727,8 +743,8 @@ class JobQueueDB extends JobQueue {
 		return [
 			// Fields that describe the nature of the job
 			'job_cmd' => $job->getType(),
-			'job_namespace' => $job->getTitle()->getNamespace(),
-			'job_title' => $job->getTitle()->getDBkey(),
+			'job_namespace' => $job->getParams()['namespace'] ?? NS_SPECIAL,
+			'job_title' => $job->getParams()['title'] ?? '',
 			'job_params' => self::makeBlob( $job->getParams() ),
 			// Additional job metadata
 			'job_timestamp' => $db->timestamp(),
@@ -742,7 +758,7 @@ class JobQueueDB extends JobQueue {
 
 	/**
 	 * @throws JobQueueConnectionError
-	 * @return DBConnRef
+	 * @return IDatabase
 	 */
 	protected function getReplicaDB() {
 		try {
@@ -754,7 +770,7 @@ class JobQueueDB extends JobQueue {
 
 	/**
 	 * @throws JobQueueConnectionError
-	 * @return DBConnRef
+	 * @return IMaintainableDatabase
 	 */
 	protected function getMasterDB() {
 		try {
@@ -766,20 +782,56 @@ class JobQueueDB extends JobQueue {
 
 	/**
 	 * @param int $index (DB_REPLICA/DB_MASTER)
-	 * @return DBConnRef
+	 * @return IMaintainableDatabase
 	 */
 	protected function getDB( $index ) {
-		$lbFactory = MediaWikiServices::getInstance()->getDBLoadBalancerFactory();
-		$lb = ( $this->cluster !== false )
-			? $lbFactory->getExternalLB( $this->cluster )
-			: $lbFactory->getMainLB( $this->wiki );
+		if ( $this->server ) {
+			if ( $this->conn instanceof IDatabase ) {
+				return $this->conn;
+			} elseif ( $this->conn instanceof DBError ) {
+				throw $this->conn;
+			}
 
-		return ( $lb->getServerType( $lb->getWriterIndex() ) !== 'sqlite' )
-			// Keep a separate connection to avoid contention and deadlocks;
-			// However, SQLite has the opposite behavior due to DB-level locking.
-			? $lb->getConnectionRef( $index, [], $this->wiki, $lb::CONN_TRX_AUTOCOMMIT )
-			// Jobs insertion will be defered until the PRESEND stage to reduce contention.
-			: $lb->getConnectionRef( $index, [], $this->wiki );
+			try {
+				$this->conn = Database::factory( $this->server['type'], $this->server );
+			} catch ( DBError $e ) {
+				$this->conn = $e;
+				throw $e;
+			}
+
+			return $this->conn;
+		} else {
+			$lbFactory = MediaWikiServices::getInstance()->getDBLoadBalancerFactory();
+			$lb = is_string( $this->cluster )
+				? $lbFactory->getExternalLB( $this->cluster )
+				: $lbFactory->getMainLB( $this->domain );
+
+			if ( $lb->getServerType( $lb->getWriterIndex() ) !== 'sqlite' ) {
+				// Keep a separate connection to avoid contention and deadlocks;
+				// However, SQLite has the opposite behavior due to DB-level locking.
+				$flags = $lb::CONN_TRX_AUTOCOMMIT;
+			} else {
+				// Jobs insertion will be defered until the PRESEND stage to reduce contention.
+				$flags = 0;
+			}
+
+			return $lb->getMaintenanceConnectionRef( $index, [], $this->domain, $flags );
+		}
+	}
+
+	/**
+	 * @param IDatabase $db
+	 * @return ScopedCallback
+	 */
+	private function getScopedNoTrxFlag( IDatabase $db ) {
+		$autoTrx = $db->getFlag( DBO_TRX ); // get current setting
+		$db->clearFlag( DBO_TRX ); // make each query its own transaction
+
+		return new ScopedCallback( function () use ( $db, $autoTrx ) {
+			if ( $autoTrx ) {
+				$db->setFlag( DBO_TRX ); // restore old setting
+			}
+		} );
 	}
 
 	/**
@@ -789,8 +841,13 @@ class JobQueueDB extends JobQueue {
 	private function getCacheKey( $property ) {
 		$cluster = is_string( $this->cluster ) ? $this->cluster : 'main';
 
-		return $this->cache->makeGlobalKey(
-			'jobqueue', $this->wiki, $cluster, $this->type, $property );
+		return $this->wanCache->makeGlobalKey(
+			'jobqueue',
+			$this->domain,
+			$cluster,
+			$this->type,
+			$property
+		);
 	}
 
 	/**
@@ -806,23 +863,30 @@ class JobQueueDB extends JobQueue {
 	}
 
 	/**
-	 * @param string $blob
-	 * @return bool|mixed
+	 * @param stdClass $row
+	 * @return RunnableJob|null
 	 */
-	protected static function extractBlob( $blob ) {
-		if ( (string)$blob !== '' ) {
-			return unserialize( $blob );
-		} else {
-			return false;
+	protected function jobFromRow( $row ) {
+		$params = ( (string)$row->job_params !== '' ) ? unserialize( $row->job_params ) : [];
+		if ( !is_array( $params ) ) { // this shouldn't happen
+			throw new UnexpectedValueException(
+				"Could not unserialize job with ID '{$row->job_id}'." );
 		}
+
+		$params += [ 'namespace' => $row->job_namespace, 'title' => $row->job_title ];
+		$job = $this->factoryJob( $row->job_cmd, $params );
+		$job->setMetadata( 'id', $row->job_id );
+		$job->setMetadata( 'timestamp', $row->job_timestamp );
+
+		return $job;
 	}
 
 	/**
 	 * @param DBError $e
-	 * @throws JobQueueError
+	 * @return JobQueueError
 	 */
-	protected function throwDBException( DBError $e ) {
-		throw new JobQueueError( get_class( $e ) . ": " . $e->getMessage() );
+	protected function getDBException( DBError $e ) {
+		return new JobQueueError( get_class( $e ) . ": " . $e->getMessage() );
 	}
 
 	/**
