@@ -98,6 +98,7 @@ class FormatMetadata extends ContextSource {
 	 *   Exif::getFilteredData() or BitmapMetadataHandler )
 	 * @return array
 	 * @since 1.23
+	 * @suppress PhanTypeArraySuspiciousNullable
 	 */
 	public function makeFormattedData( $tags ) {
 		$resolutionunit = !isset( $tags['ResolutionUnit'] ) || $tags['ResolutionUnit'] == 2 ? 2 : 3;
@@ -490,8 +491,16 @@ class FormatMetadata extends ContextSource {
 
 					case 'CustomRendered':
 						switch ( $val ) {
-							case 0:
-							case 1:
+							case 0: /* normal */
+							case 1: /* custom */
+								/* The following are unofficial Apple additions */
+							case 2: /* HDR (no original saved) */
+							case 3: /* HDR (original saved) */
+							case 4: /* Original (for HDR) */
+								/* Yes 5 is not present ;) */
+							case 6: /* Panorama */
+							case 7: /* Portrait HDR */
+							case 8: /* Portrait */
 								$val = $this->exifMsg( $tag, $val );
 								break;
 							default:
@@ -1181,7 +1190,7 @@ class FormatMetadata extends ContextSource {
 		$langName = Language::fetchLanguageName( $lowLang );
 		if ( $langName === '' ) {
 			// try just the base language name. (aka en-US -> en ).
-			list( $langPrefix ) = explode( '-', $lowLang, 2 );
+			$langPrefix = explode( '-', $lowLang, 2 )[0];
 			$langName = Language::fetchLanguageName( $langPrefix );
 			if ( $langName === '' ) {
 				// give up.
@@ -1397,11 +1406,16 @@ class FormatMetadata extends ContextSource {
 	 * Format a coordinate value, convert numbers from floating point
 	 * into degree minute second representation.
 	 *
-	 * @param int $coord Degrees, minutes and seconds
-	 * @param string $type Latitude or longitude (for if its a NWS or E)
-	 * @return mixed A floating point number or whatever we were fed
+	 * @param float|string $coord Expected to be a number or numeric string in degrees
+	 * @param string $type "latitude" or "longitude"
+	 * @return string
 	 */
-	private function formatCoords( $coord, $type ) {
+	private function formatCoords( $coord, string $type ) {
+		if ( !is_numeric( $coord ) ) {
+			wfDebugLog( 'exif', __METHOD__ . ": \"$coord\" is not a number" );
+			return (string)$coord;
+		}
+
 		$ref = '';
 		if ( $coord < 0 ) {
 			$nCoord = -$coord;
@@ -1411,7 +1425,7 @@ class FormatMetadata extends ContextSource {
 				$ref = 'W';
 			}
 		} else {
-			$nCoord = $coord;
+			$nCoord = (float)$coord;
 			if ( $type === 'latitude' ) {
 				$ref = 'N';
 			} elseif ( $type === 'longitude' ) {
@@ -1420,13 +1434,14 @@ class FormatMetadata extends ContextSource {
 		}
 
 		$deg = floor( $nCoord );
-		$min = floor( ( $nCoord - $deg ) * 60.0 );
-		$sec = round( ( ( $nCoord - $deg ) - $min / 60 ) * 3600, 2 );
+		$min = floor( ( $nCoord - $deg ) * 60 );
+		$sec = round( ( ( $nCoord - $deg ) * 60 - $min ) * 60, 2 );
 
 		$deg = $this->formatNum( $deg );
 		$min = $this->formatNum( $min );
 		$sec = $this->formatNum( $sec );
 
+		// Note the default message "$1° $2′ $3″ $4" ignores the 5th parameter
 		return $this->msg( 'exif-coordinate-format', $deg, $min, $sec, $ref, $coord )->text();
 	}
 
