@@ -26,6 +26,7 @@ use Wikimedia\Timestamp\TimestampException;
  */
 abstract class RangeChronologicalPager extends ReverseChronologicalPager {
 
+	/** @var string[] */
 	protected $rangeConds = [];
 
 	/**
@@ -93,14 +94,14 @@ abstract class RangeChronologicalPager extends ReverseChronologicalPager {
 	 *
 	 * @param string $offset Index offset, inclusive
 	 * @param int $limit Exact query limit
-	 * @param bool $descending Query direction, false for ascending, true for descending
+	 * @param bool $order IndexPager::QUERY_ASCENDING or IndexPager::QUERY_DESCENDING
 	 * @return array
 	 */
-	protected function buildQueryInfo( $offset, $limit, $descending ) {
+	protected function buildQueryInfo( $offset, $limit, $order ) {
 		list( $tables, $fields, $conds, $fname, $options, $join_conds ) = parent::buildQueryInfo(
 			$offset,
 			$limit,
-			$descending
+			$order
 		);
 
 		if ( $this->rangeConds ) {
