@@ -21,9 +21,11 @@
  * @ingroup SpecialPage
  */
 
+use MediaWiki\Block\DatabaseBlock;
 use MediaWiki\Block\Restriction\PageRestriction;
 use MediaWiki\Block\Restriction\NamespaceRestriction;
 use MediaWiki\MediaWikiServices;
+use MediaWiki\User\UserIdentity;
 
 /**
  * A special page that allows users with 'block' right to block users from
@@ -33,10 +35,11 @@ use MediaWiki\MediaWikiServices;
  */
 class SpecialBlock extends FormSpecialPage {
 	/** @var User|string|null User to be blocked, as passed either by parameter (url?wpTarget=Foo)
-	 * or as subpage (Special:Block/Foo) */
+	 * or as subpage (Special:Block/Foo)
+	 */
 	protected $target;
 
-	/** @var int Block::TYPE_ constant */
+	/** @var int DatabaseBlock::TYPE_ constant */
 	protected $type;
 
 	/** @var User|string The previous block target */
@@ -101,7 +104,7 @@ class SpecialBlock extends FormSpecialPage {
 		}
 
 		list( $this->previousTarget, /*...*/ ) =
-			Block::parseTarget( $request->getVal( 'wpPreviousTarget' ) );
+			DatabaseBlock::parseTarget( $request->getVal( 'wpPreviousTarget' ) );
 		$this->requestedHideUser = $request->getBool( 'wpHideUser' );
 	}
 
@@ -141,7 +144,9 @@ class SpecialBlock extends FormSpecialPage {
 	 * @return array
 	 */
 	protected function getFormFields() {
-		global $wgBlockAllowsUTEdit;
+		$conf = $this->getConfig();
+		$enablePartialBlocks = $conf->get( 'EnablePartialBlocks' );
+		$blockAllowsUTEdit = $conf->get( 'BlockAllowsUTEdit' );
 
 		$this->getOutput()->enableOOUI();
 
@@ -149,9 +154,6 @@ class SpecialBlock extends FormSpecialPage {
 
 		$suggestedDurations = self::getSuggestedDurations();
 
-		$conf = $this->getConfig();
-		$enablePartialBlocks = $conf->get( 'EnablePartialBlocks' );
-
 		$a = [];
 
 		$a['Target'] = [
@@ -232,7 +234,7 @@ class SpecialBlock extends FormSpecialPage {
 			];
 		}
 
-		if ( $wgBlockAllowsUTEdit ) {
+		if ( $blockAllowsUTEdit ) {
 			$a['DisableUTEdit'] = [
 				'type' => 'check',
 				'label-message' => 'ipb-disableusertalk',
@@ -268,7 +270,10 @@ class SpecialBlock extends FormSpecialPage {
 		];
 
 		# Allow some users to hide name from block log, blocklist and listusers
-		if ( $user->isAllowed( 'hideuser' ) ) {
+		if ( MediaWikiServices::getInstance()
+			->getPermissionManager()
+			->userHasRight( $user, 'hideuser' )
+		) {
 			$a['HideUser'] = [
 				'type' => 'check',
 				'label-message' => 'ipbhidename',
@@ -336,12 +341,12 @@ class SpecialBlock extends FormSpecialPage {
 		# This won't be
 		$fields['PreviousTarget']['default'] = (string)$this->target;
 
-		$block = Block::newFromTarget( $this->target );
+		$block = DatabaseBlock::newFromTarget( $this->target );
 
 		// Populate fields if there is a block that is not an autoblock; if it is a range
 		// block, only populate the fields if the range is the same as $this->target
-		if ( $block instanceof Block && $block->getType() !== Block::TYPE_AUTO
-			&& ( $this->type != Block::TYPE_RANGE
+		if ( $block instanceof DatabaseBlock && $block->getType() !== DatabaseBlock::TYPE_AUTO
+			&& ( $this->type != DatabaseBlock::TYPE_RANGE
 				|| $block->getTarget() == $this->target )
 		) {
 			$fields['HardBlock']['default'] = $block->isHardblock();
@@ -362,7 +367,10 @@ class SpecialBlock extends FormSpecialPage {
 
 			// If the username was hidden (ipb_deleted == 1), don't show the reason
 			// unless this user also has rights to hideuser: T37839
-			if ( !$block->getHideName() || $this->getUser()->isAllowed( 'hideuser' ) ) {
+			if ( !$block->getHideName() || MediaWikiServices::getInstance()
+					->getPermissionManager()
+					->userHasRight( $this->getUser(), 'hideuser' )
+			) {
 				$fields['Reason']['default'] = $block->getReason();
 			} else {
 				$fields['Reason']['default'] = '';
@@ -410,18 +418,20 @@ class SpecialBlock extends FormSpecialPage {
 		}
 
 		if ( $this->getConfig()->get( 'EnablePartialBlocks' ) ) {
-			if ( $block instanceof Block && !$block->isSitewide() ) {
+			if ( $block instanceof DatabaseBlock && !$block->isSitewide() ) {
 				$fields['EditingRestriction']['default'] = 'partial';
 			} else {
 				$fields['EditingRestriction']['default'] = 'sitewide';
 			}
 
-			if ( $block instanceof Block ) {
+			if ( $block instanceof DatabaseBlock ) {
 				$pageRestrictions = [];
 				$namespaceRestrictions = [];
 				foreach ( $block->getRestrictions() as $restriction ) {
 					switch ( $restriction->getType() ) {
 						case PageRestriction::TYPE:
+							/** @var PageRestriction $restriction */
+							'@phan-var PageRestriction $restriction';
 							if ( $restriction->getTitle() ) {
 								$pageRestrictions[] = $restriction->getTitle()->getPrefixedText();
 							}
@@ -542,7 +552,8 @@ class SpecialBlock extends FormSpecialPage {
 		$user = $this->getUser();
 
 		# Link to edit the block dropdown reasons, if applicable
-		if ( $user->isAllowed( 'editinterface' ) ) {
+		$permissionManager = MediaWikiServices::getInstance()->getPermissionManager();
+		if ( $permissionManager->userHasRight( $user, 'editinterface' ) ) {
 			$links[] = $linkRenderer->makeKnownLink(
 				$this->msg( 'ipbreason-dropdown' )->inContentLanguage()->getTitle(),
 				$this->msg( 'ipb-edit-dropdown' )->text(),
@@ -576,7 +587,7 @@ class SpecialBlock extends FormSpecialPage {
 			$text .= $out;
 
 			# Add suppression block entries if allowed
-			if ( $user->isAllowed( 'suppressionlog' ) ) {
+			if ( $permissionManager->userHasRight( $user, 'suppressionlog' ) ) {
 				LogEventsList::showLogExtract(
 					$out,
 					'suppress',
@@ -615,11 +626,11 @@ class SpecialBlock extends FormSpecialPage {
 
 	/**
 	 * Determine the target of the block, and the type of target
-	 * @todo Should be in Block.php?
+	 * @todo Should be in DatabaseBlock.php?
 	 * @param string $par Subpage parameter passed to setup, or data value from
 	 *     the HTMLForm
 	 * @param WebRequest|null $request Optionally try and get data from a request too
-	 * @return array [ User|string|null, Block::TYPE_ constant|null ]
+	 * @return array [ User|string|null, DatabaseBlock::TYPE_ constant|null ]
 	 * @phan-return array{0:User|string|null,1:int|null}
 	 */
 	public static function getTargetAndType( $par, WebRequest $request = null ) {
@@ -655,7 +666,7 @@ class SpecialBlock extends FormSpecialPage {
 					break 2;
 			}
 
-			list( $target, $type ) = Block::parseTarget( $target );
+			list( $target, $type ) = DatabaseBlock::parseTarget( $target );
 
 			if ( $type !== null ) {
 				return [ $target, $type ];
@@ -699,7 +710,7 @@ class SpecialBlock extends FormSpecialPage {
 		list( $target, $type ) = self::getTargetAndType( $value );
 		$status = Status::newGood( $target );
 
-		if ( $type == Block::TYPE_USER ) {
+		if ( $type == DatabaseBlock::TYPE_USER ) {
 			if ( $target->isAnon() ) {
 				$status->fatal(
 					'nosuchusershort',
@@ -711,7 +722,7 @@ class SpecialBlock extends FormSpecialPage {
 			if ( $unblockStatus !== true ) {
 				$status->fatal( 'badaccess', $unblockStatus );
 			}
-		} elseif ( $type == Block::TYPE_RANGE ) {
+		} elseif ( $type == DatabaseBlock::TYPE_RANGE ) {
 			list( $ip, $range ) = explode( '/', $target, 2 );
 
 			if (
@@ -737,7 +748,7 @@ class SpecialBlock extends FormSpecialPage {
 			if ( IP::isIPv6( $ip ) && $range < $wgBlockCIDRLimit['IPv6'] ) {
 				$status->fatal( 'ip_range_toolarge', $wgBlockCIDRLimit['IPv6'] );
 			}
-		} elseif ( $type == Block::TYPE_IP ) {
+		} elseif ( $type == DatabaseBlock::TYPE_IP ) {
 			# All is well
 		} else {
 			$status->fatal( 'badipaddress' );
@@ -751,27 +762,22 @@ class SpecialBlock extends FormSpecialPage {
 	 *
 	 * @param array $data
 	 * @param IContextSource $context
-	 * @return bool|string
+	 * @return bool|array
 	 */
 	public static function processForm( array $data, IContextSource $context ) {
-		global $wgBlockAllowsUTEdit, $wgHideUserContribLimit;
-
 		$performer = $context->getUser();
 		$enablePartialBlocks = $context->getConfig()->get( 'EnablePartialBlocks' );
 		$isPartialBlock = $enablePartialBlocks &&
 			isset( $data['EditingRestriction'] ) &&
 			$data['EditingRestriction'] === 'partial';
 
-		// Handled by field validator callback
-		// self::validateTargetField( $data['Target'] );
-
 		# This might have been a hidden field or a checkbox, so interesting data
 		# can come from it
 		$data['Confirm'] = !in_array( $data['Confirm'], [ '', '0', null, false ], true );
 
 		/** @var User $target */
 		list( $target, $type ) = self::getTargetAndType( $data['Target'] );
-		if ( $type == Block::TYPE_USER ) {
+		if ( $type == DatabaseBlock::TYPE_USER ) {
 			$user = $target;
 			$target = $user->getName();
 			$userId = $user->getId();
@@ -788,10 +794,10 @@ class SpecialBlock extends FormSpecialPage {
 			) {
 				return [ 'ipb-blockingself', 'ipb-confirmaction' ];
 			}
-		} elseif ( $type == Block::TYPE_RANGE ) {
+		} elseif ( $type == DatabaseBlock::TYPE_RANGE ) {
 			$user = null;
 			$userId = 0;
-		} elseif ( $type == Block::TYPE_IP ) {
+		} elseif ( $type == DatabaseBlock::TYPE_IP ) {
 			$user = null;
 			$target = $target->getName();
 			$userId = 0;
@@ -830,7 +836,10 @@ class SpecialBlock extends FormSpecialPage {
 		}
 
 		if ( $data['HideUser'] ) {
-			if ( !$performer->isAllowed( 'hideuser' ) ) {
+			if ( !MediaWikiServices::getInstance()
+				->getPermissionManager()
+				->userHasRight( $performer, 'hideuser' )
+			) {
 				# this codepath is unreachable except by a malicious user spoofing forms,
 				# or by race conditions (user has hideuser and block rights, loads block form,
 				# and loses hideuser rights before submission); so need to fail completely
@@ -843,31 +852,41 @@ class SpecialBlock extends FormSpecialPage {
 			}
 
 			# Recheck params here...
-			if ( $type != Block::TYPE_USER ) {
+			$hideUserContribLimit = $context->getConfig()->get( 'HideUserContribLimit' );
+			if ( $type != DatabaseBlock::TYPE_USER ) {
 				$data['HideUser'] = false; # IP users should not be hidden
 			} elseif ( !wfIsInfinity( $data['Expiry'] ) ) {
 				# Bad expiry.
 				return [ 'ipb_expiry_temp' ];
-			} elseif ( $wgHideUserContribLimit !== false
-				&& $user->getEditCount() > $wgHideUserContribLimit
+			} elseif ( $hideUserContribLimit !== false
+				&& $user->getEditCount() > $hideUserContribLimit
 			) {
 				# Typically, the user should have a handful of edits.
 				# Disallow hiding users with many edits for performance.
 				return [ [ 'ipb_hide_invalid',
-					Message::numParam( $wgHideUserContribLimit ) ] ];
+					Message::numParam( $hideUserContribLimit ) ] ];
 			} elseif ( !$data['Confirm'] ) {
 				return [ 'ipb-confirmhideuser', 'ipb-confirmaction' ];
 			}
 		}
 
+		$blockAllowsUTEdit = $context->getConfig()->get( 'BlockAllowsUTEdit' );
+		$userTalkEditAllowed = !$blockAllowsUTEdit || !$data['DisableUTEdit'];
+		if ( !$userTalkEditAllowed &&
+			$isPartialBlock &&
+			!in_array( NS_USER_TALK, explode( "\n", $data['NamespaceRestrictions'] ) )
+		) {
+			return [ 'ipb-prevent-user-talk-edit' ];
+		}
+
 		# Create block object.
-		$block = new Block();
+		$block = new DatabaseBlock();
 		$block->setTarget( $target );
 		$block->setBlocker( $performer );
 		$block->setReason( $data['Reason'][0] );
 		$block->setExpiry( $expiryTime );
 		$block->isCreateAccountBlocked( $data['CreateAccount'] );
-		$block->isUsertalkEditAllowed( !$wgBlockAllowsUTEdit || !$data['DisableUTEdit'] );
+		$block->isUsertalkEditAllowed( $userTalkEditAllowed );
 		$block->isEmailBlocked( $data['DisableEmail'] );
 		$block->isHardblock( $data['HardBlock'] );
 		$block->isAutoblocking( $data['AutoBlock'] );
@@ -924,13 +943,16 @@ class SpecialBlock extends FormSpecialPage {
 			} else {
 				# This returns direct blocks before autoblocks/rangeblocks, since we should
 				# be sure the user is blocked by now it should work for our purposes
-				$currentBlock = Block::newFromTarget( $target );
+				$currentBlock = DatabaseBlock::newFromTarget( $target );
 				if ( $block->equals( $currentBlock ) ) {
 					return [ [ 'ipb_already_blocked', $block->getTarget() ] ];
 				}
 				# If the name was hidden and the blocking user cannot hide
 				# names, then don't allow any block changes...
-				if ( $currentBlock->getHideName() && !$performer->isAllowed( 'hideuser' ) ) {
+				if ( $currentBlock->getHideName() && !MediaWikiServices::getInstance()
+						->getPermissionManager()
+						->userHasRight( $performer, 'hideuser' )
+				) {
 					return [ 'cant-see-hidden-user' ];
 				}
 
@@ -985,7 +1007,7 @@ class SpecialBlock extends FormSpecialPage {
 		}
 
 		# Can't watch a rangeblock
-		if ( $type != Block::TYPE_RANGE && $data['Watch'] ) {
+		if ( $type != DatabaseBlock::TYPE_RANGE && $data['Watch'] ) {
 			WatchAction::doWatch(
 				Title::makeTitle( NS_USER, $target ),
 				$performer,
@@ -993,7 +1015,7 @@ class SpecialBlock extends FormSpecialPage {
 			);
 		}
 
-		# Block constructor sanitizes certain block options on insert
+		# DatabaseBlock constructor sanitizes certain block options on insert
 		$data['BlockEmail'] = $block->isEmailBlocked();
 		$data['AutoBlock'] = $block->isAutoblocking();
 
@@ -1025,7 +1047,7 @@ class SpecialBlock extends FormSpecialPage {
 		$logId = $logEntry->insert();
 
 		if ( !empty( $data['Tags'] ) ) {
-			$logEntry->setTags( $data['Tags'] );
+			$logEntry->addTags( $data['Tags'] );
 		}
 
 		$logEntry->publish( $logId );
@@ -1077,7 +1099,7 @@ class SpecialBlock extends FormSpecialPage {
 	 *
 	 * @todo strtotime() only accepts English strings. This means the expiry input
 	 *       can only be specified in English.
-	 * @see https://secure.php.net/manual/en/function.strtotime.php
+	 * @see https://www.php.net/manual/en/function.strtotime.php
 	 *
 	 * @param string $expiry Whatever was typed into the form
 	 * @return string|bool Timestamp or 'infinity' or false on error.
@@ -1098,13 +1120,15 @@ class SpecialBlock extends FormSpecialPage {
 
 	/**
 	 * Can we do an email block?
-	 * @param User $user The sysop wanting to make a block
+	 * @param UserIdentity $user The sysop wanting to make a block
 	 * @return bool
 	 */
-	public static function canBlockEmail( $user ) {
-		global $wgEnableUserEmail, $wgSysopEmailBans;
+	public static function canBlockEmail( UserIdentity $user ) {
+		global $wgEnableUserEmail;
 
-		return ( $wgEnableUserEmail && $wgSysopEmailBans && $user->isAllowed( 'blockemail' ) );
+		return ( $wgEnableUserEmail && MediaWikiServices::getInstance()
+				->getPermissionManager()
+				->userHasRight( $user, 'blockemail' ) );
 	}
 
 	/**
@@ -1127,10 +1151,13 @@ class SpecialBlock extends FormSpecialPage {
 		} elseif ( is_string( $target ) ) {
 			$target = User::newFromName( $target );
 		}
-		if ( $performer->isBlocked() ) {
+		if ( $performer->getBlock() ) {
 			if ( $target instanceof User && $target->getId() == $performer->getId() ) {
 				# User is trying to unblock themselves
-				if ( $performer->isAllowed( 'unblockself' ) ) {
+				if ( MediaWikiServices::getInstance()
+					->getPermissionManager()
+					->userHasRight( $performer, 'unblockself' )
+				) {
 					return true;
 					# User blocked themselves and is now trying to reverse it
 				} elseif ( $performer->blockedBy() === $performer->getName() ) {
@@ -1140,7 +1167,7 @@ class SpecialBlock extends FormSpecialPage {
 				}
 			} elseif (
 				$target instanceof User &&
-				$performer->getBlock() instanceof Block &&
+				$performer->getBlock() instanceof DatabaseBlock &&
 				$performer->getBlock()->getBy() &&
 				$performer->getBlock()->getBy() === $target->getId()
 			) {
@@ -1164,7 +1191,7 @@ class SpecialBlock extends FormSpecialPage {
 	 * Return a comma-delimited list of "flags" to be passed to the log
 	 * reader for this block, to provide more information in the logs
 	 * @param array $data From HTMLForm data
-	 * @param int $type Block::TYPE_ constant (USER, RANGE, or IP)
+	 * @param int $type DatabaseBlock::TYPE_ constant (USER, RANGE, or IP)
 	 * @return string
 	 */
 	protected static function blockLogFlags( array $data, $type ) {
@@ -1176,7 +1203,7 @@ class SpecialBlock extends FormSpecialPage {
 
 		# when blocking a user the option 'anononly' is not available/has no effect
 		# -> do not write this into log
-		if ( !$data['HardBlock'] && $type != Block::TYPE_USER ) {
+		if ( !$data['HardBlock'] && $type != DatabaseBlock::TYPE_USER ) {
 			// For grepping: message block-log-flags-anononly
 			$flags[] = 'anononly';
 		}
@@ -1187,7 +1214,7 @@ class SpecialBlock extends FormSpecialPage {
 		}
 
 		# Same as anononly, this is not displayed when blocking an IP address
-		if ( !$data['AutoBlock'] && $type == Block::TYPE_USER ) {
+		if ( !$data['AutoBlock'] && $type == DatabaseBlock::TYPE_USER ) {
 			// For grepping: message block-log-flags-noautoblock
 			$flags[] = 'noautoblock';
 		}
