@@ -21,7 +21,8 @@
  * @ingroup SpecialPage
  */
 
-use MediaWiki\Widget\DateInputWidget;
+use MediaWiki\Block\DatabaseBlock;
+use MediaWiki\MediaWikiServices;
 
 /**
  * Special:Contributions, show user contributions in a paged list
@@ -41,11 +42,16 @@ class SpecialContributions extends IncludableSpecialPage {
 		$out = $this->getOutput();
 		// Modules required for viewing the list of contributions (also when included on other pages)
 		$out->addModuleStyles( [
+			'jquery.makeCollapsible.styles',
 			'mediawiki.interface.helpers.styles',
 			'mediawiki.special',
 			'mediawiki.special.changeslist',
 		] );
-		$out->addModules( 'mediawiki.special.recentchanges' );
+		$out->addModules( [
+			'mediawiki.special.recentchanges',
+			// Certain skins e.g. Minerva might have disabled this module.
+			'mediawiki.page.ready'
+		] );
 		$this->addHelpLink( 'Help:User contributions' );
 
 		$this->opts = [];
@@ -53,18 +59,11 @@ class SpecialContributions extends IncludableSpecialPage {
 
 		$target = $par ?? $request->getVal( 'target' );
 
-		if ( $request->getVal( 'contribs' ) == 'newbie' || $par === 'newbies' ) {
-			$target = 'newbies';
-			$this->opts['contribs'] = 'newbie';
-		} else {
-			$this->opts['contribs'] = 'user';
-		}
-
 		$this->opts['deletedOnly'] = $request->getBool( 'deletedOnly' );
 
 		if ( !strlen( $target ) ) {
 			if ( !$this->including() ) {
-				$out->addHTML( $this->getForm() );
+				$out->addHTML( $this->getForm( $this->opts ) );
 			}
 
 			return;
@@ -79,17 +78,10 @@ class SpecialContributions extends IncludableSpecialPage {
 		$this->opts['hideMinor'] = $request->getBool( 'hideMinor' );
 
 		$id = 0;
-		if ( $this->opts['contribs'] === 'newbie' ) {
-			$userObj = User::newFromName( $target ); // hysterical raisins
-			$out->addSubtitle( $this->msg( 'sp-contributions-newbies-sub' ) );
-			$out->setHTMLTitle( $this->msg(
-				'pagetitle',
-				$this->msg( 'sp-contributions-newbies-title' )->plain()
-			)->inContentLanguage() );
-		} elseif ( ExternalUserNames::isExternal( $target ) ) {
+		if ( ExternalUserNames::isExternal( $target ) ) {
 			$userObj = User::newFromName( $target, false );
 			if ( !$userObj ) {
-				$out->addHTML( $this->getForm() );
+				$out->addHTML( $this->getForm( $this->opts ) );
 				return;
 			}
 
@@ -101,12 +93,12 @@ class SpecialContributions extends IncludableSpecialPage {
 		} else {
 			$nt = Title::makeTitleSafe( NS_USER, $target );
 			if ( !$nt ) {
-				$out->addHTML( $this->getForm() );
+				$out->addHTML( $this->getForm( $this->opts ) );
 				return;
 			}
 			$userObj = User::newFromName( $nt->getText(), false );
 			if ( !$userObj ) {
-				$out->addHTML( $this->getForm() );
+				$out->addHTML( $this->getForm( $this->opts ) );
 				return;
 			}
 			$id = $userObj->getId();
@@ -126,19 +118,31 @@ class SpecialContributions extends IncludableSpecialPage {
 		}
 
 		$ns = $request->getVal( 'namespace', null );
-		if ( $ns !== null && $ns !== '' ) {
+		if ( $ns !== null && $ns !== '' && $ns !== 'all' ) {
 			$this->opts['namespace'] = intval( $ns );
 		} else {
 			$this->opts['namespace'] = '';
 		}
 
+		// Backwards compatibility: Before using OOUI form the old HTML form had
+		// fields for nsInvert and associated. These have now been replaced with the
+		// wpFilters query string parameters. These are retained to keep old URIs working.
 		$this->opts['associated'] = $request->getBool( 'associated' );
 		$this->opts['nsInvert'] = (bool)$request->getVal( 'nsInvert' );
+		$nsFilters = $request->getArray( 'wpfilters', null );
+		if ( $nsFilters !== null ) {
+			$this->opts['associated'] = in_array( 'associated', $nsFilters );
+			$this->opts['nsInvert'] = in_array( 'nsInvert', $nsFilters );
+		}
+
 		$this->opts['tagfilter'] = (string)$request->getVal( 'tagfilter' );
 
 		// Allows reverts to have the bot flag in recent changes. It is just here to
 		// be passed in the form at the top of the page
-		if ( $user->isAllowed( 'markbotedits' ) && $request->getBool( 'bot' ) ) {
+		if ( MediaWikiServices::getInstance()
+				 ->getPermissionManager()
+				 ->userHasRight( $user, 'markbotedits' ) && $request->getBool( 'bot' )
+		) {
 			$this->opts['bot'] = '1';
 		}
 
@@ -158,7 +162,7 @@ class SpecialContributions extends IncludableSpecialPage {
 				"<div class=\"mw-negative-namespace-not-supported error\">\n\$1\n</div>",
 				[ 'negative-namespace-not-supported' ]
 			);
-			$out->addHTML( $this->getForm() );
+			$out->addHTML( $this->getForm( $this->opts ) );
 			return;
 		}
 
@@ -210,12 +214,8 @@ class SpecialContributions extends IncludableSpecialPage {
 		$this->addFeedLinks( $feedParams );
 
 		if ( Hooks::run( 'SpecialContributionsBeforeMainOutput', [ $id, $userObj, $this ] ) ) {
-			if ( !$this->including() ) {
-				$out->addHTML( $this->getForm() );
-			}
 			$pager = new ContribsPager( $this->getContext(), [
 				'target' => $target,
-				'contribs' => $this->opts['contribs'],
 				'namespace' => $this->opts['namespace'],
 				'tagfilter' => $this->opts['tagfilter'],
 				'start' => $this->opts['start'],
@@ -226,7 +226,10 @@ class SpecialContributions extends IncludableSpecialPage {
 				'hideMinor' => $this->opts['hideMinor'],
 				'nsInvert' => $this->opts['nsInvert'],
 				'associated' => $this->opts['associated'],
-			] );
+			], $this->getLinkRenderer() );
+			if ( !$this->including() ) {
+				$out->addHTML( $this->getForm( $this->opts ) );
+			}
 
 			if ( IP::isValidRange( $target ) && !$pager->isQueryableRange( $target ) ) {
 				// Valid range, but outside CIDR limit.
@@ -254,9 +257,7 @@ class SpecialContributions extends IncludableSpecialPage {
 			$out->preventClickjacking( $pager->getPreventClickjacking() );
 
 			# Show the appropriate "footer" message - WHOIS tools, etc.
-			if ( $this->opts['contribs'] == 'newbie' ) {
-				$message = 'sp-contributions-footer-newbies';
-			} elseif ( IP::isValidRange( $target ) ) {
+			if ( IP::isValidRange( $target ) ) {
 				$message = 'sp-contributions-footer-anon-range';
 			} elseif ( IP::isIPAddress( $target ) ) {
 				$message = 'sp-contributions-footer-anon';
@@ -284,7 +285,7 @@ class SpecialContributions extends IncludableSpecialPage {
 	 */
 	protected function contributionsSub( $userObj ) {
 		if ( $userObj->isAnon() ) {
-			// Show a warning message that the user being searched for doesn't exists.
+			// Show a warning message that the user being searched for doesn't exist.
 			// User::isIP returns true for IP address and usemod IPs like '123.123.123.xxx',
 			// but returns false for IP ranges. We don't want to suggest either of these are
 			// valid usernames which we would with the 'contributions-userdoesnotexist' message.
@@ -319,16 +320,18 @@ class SpecialContributions extends IncludableSpecialPage {
 			// Do not expose the autoblocks, since that may lead to a leak of accounts' IPs,
 			// and also this will display a totally irrelevant log entry as a current block.
 			if ( !$this->including() ) {
-				// For IP ranges you must give Block::newFromTarget the CIDR string and not a user object.
+				// For IP ranges you must give DatabaseBlock::newFromTarget the CIDR string
+				// and not a user object.
 				if ( $userObj->isIPRange() ) {
-					$block = Block::newFromTarget( $userObj->getName(), $userObj->getName() );
+					$block = DatabaseBlock::newFromTarget( $userObj->getName(), $userObj->getName() );
 				} else {
-					$block = Block::newFromTarget( $userObj, $userObj );
+					$block = DatabaseBlock::newFromTarget( $userObj, $userObj );
 				}
 
-				if ( !is_null( $block ) && $block->getType() != Block::TYPE_AUTO ) {
-					if ( $block->getType() == Block::TYPE_RANGE ) {
-						$nt = MWNamespace::getCanonicalName( NS_USER ) . ':' . $block->getTarget();
+				if ( !is_null( $block ) && $block->getType() != DatabaseBlock::TYPE_AUTO ) {
+					if ( $block->getType() == DatabaseBlock::TYPE_RANGE ) {
+						$nt = MediaWikiServices::getInstance()->getNamespaceInfo()->
+							getCanonicalName( NS_USER ) . ':' . $block->getTarget();
 					}
 
 					$out = $this->getOutput(); // showLogExtract() wants first parameter by reference
@@ -372,71 +375,78 @@ class SpecialContributions extends IncludableSpecialPage {
 		$username = $target->getName();
 		$userpage = $target->getUserPage();
 		$talkpage = $target->getTalkPage();
+		$isIP = IP::isValid( $username );
+		$isRange = IP::isValidRange( $username );
 
 		$linkRenderer = $sp->getLinkRenderer();
 
+		$tools = [];
 		# No talk pages for IP ranges.
-		if ( !IP::isValidRange( $username ) ) {
+		if ( !$isRange ) {
 			$tools['user-talk'] = $linkRenderer->makeLink(
 				$talkpage,
 				$sp->msg( 'sp-contributions-talk' )->text()
 			);
 		}
 
-		if ( ( $id !== null ) || ( $id === null && IP::isIPAddress( $username ) ) ) {
-			if ( $sp->getUser()->isAllowed( 'block' ) ) { # Block / Change block / Unblock links
-				if ( $target->isBlocked() && $target->getBlock()->getType() != Block::TYPE_AUTO ) {
-					$tools['block'] = $linkRenderer->makeKnownLink( # Change block link
-						SpecialPage::getTitleFor( 'Block', $username ),
-						$sp->msg( 'change-blocklink' )->text()
-					);
-					$tools['unblock'] = $linkRenderer->makeKnownLink( # Unblock link
-						SpecialPage::getTitleFor( 'Unblock', $username ),
-						$sp->msg( 'unblocklink' )->text()
-					);
-				} else { # User is not blocked
-					$tools['block'] = $linkRenderer->makeKnownLink( # Block link
-						SpecialPage::getTitleFor( 'Block', $username ),
-						$sp->msg( 'blocklink' )->text()
-					);
-				}
-			}
-
-			# Block log link
-			$tools['log-block'] = $linkRenderer->makeKnownLink(
-				SpecialPage::getTitleFor( 'Log', 'block' ),
-				$sp->msg( 'sp-contributions-blocklog' )->text(),
-				[],
-				[ 'page' => $userpage->getPrefixedText() ]
-			);
-
-			# Suppression log link (T61120)
-			if ( $sp->getUser()->isAllowed( 'suppressionlog' ) ) {
-				$tools['log-suppression'] = $linkRenderer->makeKnownLink(
-					SpecialPage::getTitleFor( 'Log', 'suppress' ),
-					$sp->msg( 'sp-contributions-suppresslog', $username )->text(),
-					[],
-					[ 'offender' => $username ]
+		# Block / Change block / Unblock links
+		$permissionManager = MediaWikiServices::getInstance()->getPermissionManager();
+		if ( $permissionManager->userHasRight( $sp->getUser(), 'block' ) ) {
+			if ( $target->getBlock() && $target->getBlock()->getType() != DatabaseBlock::TYPE_AUTO ) {
+				$tools['block'] = $linkRenderer->makeKnownLink( # Change block link
+					SpecialPage::getTitleFor( 'Block', $username ),
+					$sp->msg( 'change-blocklink' )->text()
+				);
+				$tools['unblock'] = $linkRenderer->makeKnownLink( # Unblock link
+					SpecialPage::getTitleFor( 'Unblock', $username ),
+					$sp->msg( 'unblocklink' )->text()
+				);
+			} else { # User is not blocked
+				$tools['block'] = $linkRenderer->makeKnownLink( # Block link
+					SpecialPage::getTitleFor( 'Block', $username ),
+					$sp->msg( 'blocklink' )->text()
 				);
 			}
 		}
 
-		# Don't show some links for IP ranges
-		if ( !IP::isValidRange( $username ) ) {
-			# Uploads
-			$tools['uploads'] = $linkRenderer->makeKnownLink(
-				SpecialPage::getTitleFor( 'Listfiles', $username ),
-				$sp->msg( 'sp-contributions-uploads' )->text()
+		# Block log link
+		$tools['log-block'] = $linkRenderer->makeKnownLink(
+			SpecialPage::getTitleFor( 'Log', 'block' ),
+			$sp->msg( 'sp-contributions-blocklog' )->text(),
+			[],
+			[ 'page' => $userpage->getPrefixedText() ]
+		);
+
+		# Suppression log link (T61120)
+		if ( $permissionManager->userHasRight( $sp->getUser(), 'suppressionlog' ) ) {
+			$tools['log-suppression'] = $linkRenderer->makeKnownLink(
+				SpecialPage::getTitleFor( 'Log', 'suppress' ),
+				$sp->msg( 'sp-contributions-suppresslog', $username )->text(),
+				[],
+				[ 'offender' => $username ]
 			);
+		}
+
+		# Don't show some links for IP ranges
+		if ( !$isRange ) {
+			# Uploads: hide if IPs cannot upload (T220674)
+			if ( !$isIP || $permissionManager->userHasRight( $target, 'upload' ) ) {
+				$tools['uploads'] = $linkRenderer->makeKnownLink(
+					SpecialPage::getTitleFor( 'Listfiles', $username ),
+					$sp->msg( 'sp-contributions-uploads' )->text()
+				);
+			}
 
 			# Other logs link
+			# Todo: T146628
 			$tools['logs'] = $linkRenderer->makeKnownLink(
 				SpecialPage::getTitleFor( 'Log', $username ),
 				$sp->msg( 'sp-contributions-logs' )->text()
 			);
 
 			# Add link to deleted user contributions for priviledged users
-			if ( $sp->getUser()->isAllowed( 'deletedhistory' ) ) {
+			# Todo: T183457
+			if ( $permissionManager->userHasRight( $sp->getUser(), 'deletedhistory' ) ) {
 				$tools['deletedcontribs'] = $linkRenderer->makeKnownLink(
 					SpecialPage::getTitleFor( 'DeletedContributions', $username ),
 					$sp->msg( 'sp-contributions-deleted', $username )->text()
@@ -461,60 +471,12 @@ class SpecialContributions extends IncludableSpecialPage {
 
 	/**
 	 * Generates the namespace selector form with hidden attributes.
+	 * @param array $pagerOptions with keys contribs, user, deletedOnly, limit, target, topOnly,
+	 *  newOnly, hideMinor, namespace, associated, nsInvert, tagfilter, year, start, end
 	 * @return string HTML fragment
 	 */
-	protected function getForm() {
+	protected function getForm( array $pagerOptions ) {
 		$this->opts['title'] = $this->getPageTitle()->getPrefixedText();
-		if ( !isset( $this->opts['target'] ) ) {
-			$this->opts['target'] = '';
-		} else {
-			$this->opts['target'] = str_replace( '_', ' ', $this->opts['target'] );
-		}
-
-		if ( !isset( $this->opts['namespace'] ) ) {
-			$this->opts['namespace'] = '';
-		}
-
-		if ( !isset( $this->opts['nsInvert'] ) ) {
-			$this->opts['nsInvert'] = '';
-		}
-
-		if ( !isset( $this->opts['associated'] ) ) {
-			$this->opts['associated'] = false;
-		}
-
-		if ( !isset( $this->opts['contribs'] ) ) {
-			$this->opts['contribs'] = 'user';
-		}
-
-		if ( !isset( $this->opts['start'] ) ) {
-			$this->opts['start'] = '';
-		}
-
-		if ( !isset( $this->opts['end'] ) ) {
-			$this->opts['end'] = '';
-		}
-
-		if ( $this->opts['contribs'] == 'newbie' ) {
-			$this->opts['target'] = '';
-		}
-
-		if ( !isset( $this->opts['tagfilter'] ) ) {
-			$this->opts['tagfilter'] = '';
-		}
-
-		if ( !isset( $this->opts['topOnly'] ) ) {
-			$this->opts['topOnly'] = false;
-		}
-
-		if ( !isset( $this->opts['newOnly'] ) ) {
-			$this->opts['newOnly'] = false;
-		}
-
-		if ( !isset( $this->opts['hideMinor'] ) ) {
-			$this->opts['hideMinor'] = false;
-		}
-
 		// Modules required only for the form
 		$this->getOutput()->addModules( [
 			'mediawiki.userSuggest',
@@ -522,15 +484,7 @@ class SpecialContributions extends IncludableSpecialPage {
 		] );
 		$this->getOutput()->addModuleStyles( 'mediawiki.widgets.DateInputWidget.styles' );
 		$this->getOutput()->enableOOUI();
-
-		$form = Html::openElement(
-			'form',
-			[
-				'method' => 'get',
-				'action' => wfScript(),
-				'class' => 'mw-contributions-form'
-			]
-		);
+		$fields = [];
 
 		# Add hidden params for tracking except for parameters in $skipParameters
 		$skipParameters = [
@@ -538,7 +492,6 @@ class SpecialContributions extends IncludableSpecialPage {
 			'nsInvert',
 			'deletedOnly',
 			'target',
-			'contribs',
 			'year',
 			'month',
 			'start',
@@ -554,218 +507,171 @@ class SpecialContributions extends IncludableSpecialPage {
 			if ( in_array( $name, $skipParameters ) ) {
 				continue;
 			}
-			$form .= "\t" . Html::hidden( $name, $value ) . "\n";
+
+			$fields[$name] = [
+				'name' => $name,
+				'type' => 'hidden',
+				'default' => $value,
+			];
 		}
 
-		$tagFilter = ChangeTags::buildTagFilterSelector(
-			$this->opts['tagfilter'], false, $this->getContext() );
+		$target = $this->opts['target'] ?? null;
+		$fields['target'] = [
+			'type' => 'text',
+			'cssclass' => 'mw-autocomplete-user mw-ui-input-inline mw-input',
+			'default' => $target ?
+				str_replace( '_', ' ', $target ) : '' ,
+			'label' => $this->msg( 'sp-contributions-username' )->text(),
+			'name' => 'target',
+			'id' => 'mw-target-user-or-ip',
+			'size' => 40,
+			'autofocus' => !$target,
+			'section' => 'contribs-top',
+		];
 
-		if ( $tagFilter ) {
-			$filterSelection = Html::rawElement(
-				'div',
-				[],
-				implode( "\u{00A0}", $tagFilter )
-			);
-		} else {
-			$filterSelection = Html::rawElement( 'div', [], '' );
+		$ns = $this->opts['namespace'] ?? null;
+		$fields['namespace'] = [
+			'type' => 'namespaceselect',
+			'label' => $this->msg( 'namespace' )->text(),
+			'name' => 'namespace',
+			'cssclass' => 'namespaceselector',
+			'default' => $ns,
+			'id' => 'namespace',
+			'section' => 'contribs-top',
+		];
+		$request = $this->getRequest();
+		$nsFilters = $request->getArray( 'wpfilters' );
+		$fields['nsFilters'] = [
+			'class' => 'HTMLMultiSelectField',
+			'label' => '',
+			'name' => 'wpfilters',
+			'flatlist' => true,
+			// Only shown when namespaces are selected.
+			'cssclass' => $ns === '' ?
+				'contribs-ns-filters mw-input-with-label mw-input-hidden' :
+				'contribs-ns-filters mw-input-with-label',
+			// `contribs-ns-filters` class allows these fields to be toggled on/off by JavaScript.
+			// See resources/src/mediawiki.special.recentchanges.js
+			'infusable' => true,
+			'options' => [
+				$this->msg( 'invert' )->text() => 'nsInvert',
+				$this->msg( 'namespace_association' )->text() => 'associated',
+			],
+			'default' => $nsFilters,
+			'section' => 'contribs-top',
+		];
+		$fields['tagfilter'] = [
+			'type' => 'tagfilter',
+			'cssclass' => 'mw-tagfilter-input',
+			'id' => 'tagfilter',
+			'label-message' => [ 'tag-filter', 'parse' ],
+			'name' => 'tagfilter',
+			'size' => 20,
+			'section' => 'contribs-top',
+		];
+
+		if ( MediaWikiServices::getInstance()
+			->getPermissionManager()
+			->userHasRight( $this->getUser(), 'deletedhistory' )
+		) {
+			$fields['deletedOnly'] = [
+				'type' => 'check',
+				'id' => 'mw-show-deleted-only',
+				'label' => $this->msg( 'history-show-deleted' )->text(),
+				'name' => 'deletedOnly',
+				'section' => 'contribs-top',
+			];
 		}
 
-		$labelNewbies = Xml::radioLabel(
-			$this->msg( 'sp-contributions-newbies' )->text(),
-			'contribs',
-			'newbie',
-			'newbie',
-			$this->opts['contribs'] == 'newbie',
-			[ 'class' => 'mw-input' ]
-		);
-		$labelUsername = Xml::radioLabel(
-			$this->msg( 'sp-contributions-username' )->text(),
-			'contribs',
-			'user',
-			'user',
-			$this->opts['contribs'] == 'user',
-			[ 'class' => 'mw-input' ]
-		);
-		$input = Html::input(
-			'target',
-			$this->opts['target'],
-			'text',
-			[
-				'id' => 'mw-target-user-or-ip',
-				'size' => '40',
-				'class' => [
-					'mw-input',
-					'mw-ui-input-inline',
-					'mw-autocomplete-user', // used by mediawiki.userSuggest
-				],
-			] + (
-				// Only autofocus if target hasn't been specified or in non-newbies mode
-				( $this->opts['contribs'] === 'newbie' || $this->opts['target'] )
-					? [] : [ 'autofocus' => true ]
-				)
-		);
-
-		$targetSelection = Html::rawElement(
-			'div',
-			[],
-			$labelNewbies . '<br>' . $labelUsername . ' ' . $input . ' '
-		);
-
-		$hidden = $this->opts['namespace'] === '' ? ' mw-input-hidden' : '';
-		$namespaceSelection = Xml::tags(
-			'div',
-			[],
-			Xml::label(
-				$this->msg( 'namespace' )->text(),
-				'namespace',
-				''
-			) . "\u{00A0}" .
-			Html::namespaceSelector(
-				[ 'selected' => $this->opts['namespace'], 'all' => '', 'in-user-lang' => true ],
-				[
-					'name' => 'namespace',
-					'id' => 'namespace',
-					'class' => 'namespaceselector',
-				]
-			) . "\u{00A0}" .
-				Html::rawElement(
-					'span',
-					[ 'class' => 'mw-input-with-label' . $hidden ],
-					Xml::checkLabel(
-						$this->msg( 'invert' )->text(),
-						'nsInvert',
-						'nsinvert',
-						$this->opts['nsInvert'],
-						[
-							'title' => $this->msg( 'tooltip-invert' )->text(),
-							'class' => 'mw-input'
-						]
-					) . "\u{00A0}"
-				) .
-				Html::rawElement( 'span', [ 'class' => 'mw-input-with-label' . $hidden ],
-					Xml::checkLabel(
-						$this->msg( 'namespace_association' )->text(),
-						'associated',
-						'nsassociated',
-						$this->opts['associated'],
-						[
-							'title' => $this->msg( 'tooltip-namespace_association' )->text(),
-							'class' => 'mw-input'
-						]
-					) . "\u{00A0}"
-				)
-		);
-
-		$filters = [];
-
-		if ( $this->getUser()->isAllowed( 'deletedhistory' ) ) {
-			$filters[] = Html::rawElement(
-				'span',
-				[ 'class' => 'mw-input-with-label' ],
-				Xml::checkLabel(
-					$this->msg( 'history-show-deleted' )->text(),
-					'deletedOnly',
-					'mw-show-deleted-only',
-					$this->opts['deletedOnly'],
-					[ 'class' => 'mw-input' ]
-				)
-			);
-		}
-
-		$filters[] = Html::rawElement(
-			'span',
-			[ 'class' => 'mw-input-with-label' ],
-			Xml::checkLabel(
-				$this->msg( 'sp-contributions-toponly' )->text(),
-				'topOnly',
-				'mw-show-top-only',
-				$this->opts['topOnly'],
-				[ 'class' => 'mw-input' ]
-			)
-		);
-		$filters[] = Html::rawElement(
-			'span',
-			[ 'class' => 'mw-input-with-label' ],
-			Xml::checkLabel(
-				$this->msg( 'sp-contributions-newonly' )->text(),
-				'newOnly',
-				'mw-show-new-only',
-				$this->opts['newOnly'],
-				[ 'class' => 'mw-input' ]
-			)
-		);
-		$filters[] = Html::rawElement(
-			'span',
-			[ 'class' => 'mw-input-with-label' ],
-			Xml::checkLabel(
-				$this->msg( 'sp-contributions-hideminor' )->text(),
-				'hideMinor',
-				'mw-hide-minor-edits',
-				$this->opts['hideMinor'],
-				[ 'class' => 'mw-input' ]
-			)
-		);
+		$fields['topOnly'] = [
+			'type' => 'check',
+			'id' => 'mw-show-top-only',
+			'label' => $this->msg( 'sp-contributions-toponly' )->text(),
+			'name' => 'topOnly',
+			'section' => 'contribs-top',
+		];
+		$fields['newOnly'] = [
+			'type' => 'check',
+			'id' => 'mw-show-new-only',
+			'label' => $this->msg( 'sp-contributions-newonly' )->text(),
+			'name' => 'newOnly',
+			'section' => 'contribs-top',
+		];
+		$fields['hideMinor'] = [
+			'type' => 'check',
+			'cssclass' => 'mw-hide-minor-edits',
+			'id' => 'mw-show-new-only',
+			'label' => $this->msg( 'sp-contributions-hideminor' )->text(),
+			'name' => 'hideMinor',
+			'section' => 'contribs-top',
+		];
 
+		// Allow additions at this point to the filters.
+		$rawFilters = [];
 		Hooks::run(
 			'SpecialContributions::getForm::filters',
-			[ $this, &$filters ]
+			[ $this, &$rawFilters ]
 		);
+		foreach ( $rawFilters as $filter ) {
+			// Backwards compatibility support for previous hook function signature.
+			if ( is_string( $filter ) ) {
+				$fields[] = [
+					'type' => 'info',
+					'default' => $filter,
+					'raw' => true,
+					'section' => 'contribs-top',
+				];
+				wfDeprecated(
+					__METHOD__ .
+					' returning string[]',
+					'1.33'
+				);
+			} else {
+				// Preferred append method.
+				$fields[] = $filter;
+			}
+		}
 
-		$extraOptions = Html::rawElement(
-			'div',
-			[],
-			implode( '', $filters )
-		);
+		$fields['start'] = [
+			'type' => 'date',
+			'default' => '',
+			'id' => 'mw-date-start',
+			'label' => $this->msg( 'date-range-from' )->text(),
+			'name' => 'start',
+			'section' => 'contribs-date',
+		];
+		$fields['end'] = [
+			'type' => 'date',
+			'default' => '',
+			'id' => 'mw-date-end',
+			'label' => $this->msg( 'date-range-to' )->text(),
+			'name' => 'end',
+			'section' => 'contribs-date',
+		];
 
-		$dateRangeSelection = Html::rawElement(
-			'div',
-			[],
-			Xml::label( $this->msg( 'date-range-from' )->text(), 'mw-date-start' ) . ' ' .
-			new DateInputWidget( [
-				'infusable' => true,
-				'id' => 'mw-date-start',
-				'name' => 'start',
-				'value' => $this->opts['start'],
-				'longDisplayFormat' => true,
-			] ) . '<br>' .
-			Xml::label( $this->msg( 'date-range-to' )->text(), 'mw-date-end' ) . ' ' .
-			new DateInputWidget( [
-				'infusable' => true,
-				'id' => 'mw-date-end',
-				'name' => 'end',
-				'value' => $this->opts['end'],
-				'longDisplayFormat' => true,
-			] )
-		);
-
-		$submit = Xml::tags( 'div', [],
-			Html::submitButton(
-				$this->msg( 'sp-contributions-submit' )->text(),
-				[ 'class' => 'mw-submit' ], [ 'mw-ui-progressive' ]
+		$htmlForm = HTMLForm::factory( 'ooui', $fields, $this->getContext() );
+		$htmlForm
+			->setMethod( 'get' )
+			// When offset is defined, the user is paging through results
+			// so we hide the form by default to allow users to focus on browsing
+			// rather than defining search parameters
+			->setCollapsibleOptions(
+				( $pagerOptions['target'] ?? null ) ||
+				( $pagerOptions['start'] ?? null ) ||
+				( $pagerOptions['end'] ?? null )
 			)
-		);
-
-		$form .= Xml::fieldset(
-			$this->msg( 'sp-contributions-search' )->text(),
-			$targetSelection .
-			$namespaceSelection .
-			$filterSelection .
-			$extraOptions .
-			$dateRangeSelection .
-			$submit,
-			[ 'class' => 'mw-contributions-table' ]
-		);
+			->setAction( wfScript() )
+			->setSubmitText( $this->msg( 'sp-contributions-submit' )->text() )
+			->setWrapperLegend( $this->msg( 'sp-contributions-search' )->text() );
 
 		$explain = $this->msg( 'sp-contributions-explain' );
 		if ( !$explain->isBlank() ) {
-			$form .= Html::rawElement(
-				'p', [ 'id' => 'mw-sp-contributions-explain' ], $explain->parse()
-			);
+			$htmlForm->addFooterText( "<p id='mw-sp-contributions-explain'>{$explain->parse()}</p>" );
 		}
 
-		$form .= Xml::closeElement( 'form' );
+		$htmlForm->loadData();
 
-		return $form;
+		return $htmlForm->getHTML( false );
 	}
 
 	/**
