@@ -24,6 +24,8 @@
  * @author Soxred93 <soxred93@gmail.com>
  */
 
+use MediaWiki\MediaWikiServices;
+
 /**
  * Querypage that lists the most wanted files
  *
@@ -97,14 +99,14 @@ class WantedFilesPage extends WantedQueryPage {
 	/**
 	 * Does the file exist?
 	 *
-	 * Use wfFindFile so we still think file namespace pages without
-	 * files are missing, but valid file redirects and foreign files are ok.
+	 * Use findFile() so we still think file namespace pages without files
+	 * are missing, but valid file redirects and foreign files are ok.
 	 *
 	 * @param Title $title
 	 * @return bool
 	 */
 	protected function existenceCheck( Title $title ) {
-		return (bool)wfFindFile( $title );
+		return (bool)MediaWikiServices::getInstance()->getRepoGroup()->findFile( $title );
 	}
 
 	function getQueryInfo() {
