@@ -27,53 +27,54 @@ use Wikimedia\Rdbms\IDatabase;
  * user rights manipulation.
  */
 class UserRightsProxy {
+	/** @var IDatabase */
+	private $db;
+	/** @var string */
+	private $dbDomain;
+	/** @var string */
+	private $name;
+	/** @var int */
+	private $id;
+	/** @var array */
+	private $newOptions;
 
 	/**
 	 * @see newFromId()
 	 * @see newFromName()
 	 * @param IDatabase $db Db connection
-	 * @param string $database Database name
+	 * @param string $dbDomain Database name
 	 * @param string $name User name
 	 * @param int $id User ID
 	 */
-	private function __construct( $db, $database, $name, $id ) {
+	private function __construct( $db, $dbDomain, $name, $id ) {
 		$this->db = $db;
-		$this->database = $database;
+		$this->dbDomain = $dbDomain;
 		$this->name = $name;
 		$this->id = intval( $id );
 		$this->newOptions = [];
 	}
 
-	/**
-	 * Accessor for $this->database
-	 *
-	 * @return string Database name
-	 */
-	public function getDBName() {
-		return $this->database;
-	}
-
 	/**
 	 * Confirm the selected database name is a valid local interwiki database name.
 	 *
-	 * @param string $database Database name
+	 * @param string $dbDomain Database name
 	 * @return bool
 	 */
-	public static function validDatabase( $database ) {
+	public static function validDatabase( $dbDomain ) {
 		global $wgLocalDatabases;
-		return in_array( $database, $wgLocalDatabases );
+		return in_array( $dbDomain, $wgLocalDatabases );
 	}
 
 	/**
 	 * Same as User::whoIs()
 	 *
-	 * @param string $database Database name
+	 * @param string $dbDomain Database name
 	 * @param int $id User ID
-	 * @param bool $ignoreInvalidDB If true, don't check if $database is in $wgLocalDatabases
+	 * @param bool $ignoreInvalidDB If true, don't check if $dbDomain is in $wgLocalDatabases
 	 * @return string User name or false if the user doesn't exist
 	 */
-	public static function whoIs( $database, $id, $ignoreInvalidDB = false ) {
-		$user = self::newFromId( $database, $id, $ignoreInvalidDB );
+	public static function whoIs( $dbDomain, $id, $ignoreInvalidDB = false ) {
+		$user = self::newFromId( $dbDomain, $id, $ignoreInvalidDB );
 		if ( $user ) {
 			return $user->name;
 		} else {
@@ -84,35 +85,35 @@ class UserRightsProxy {
 	/**
 	 * Factory function; get a remote user entry by ID number.
 	 *
-	 * @param string $database Database name
+	 * @param string $dbDomain Database name
 	 * @param int $id User ID
-	 * @param bool $ignoreInvalidDB If true, don't check if $database is in $wgLocalDatabases
+	 * @param bool $ignoreInvalidDB If true, don't check if $dbDomain is in $wgLocalDatabases
 	 * @return UserRightsProxy|null If doesn't exist
 	 */
-	public static function newFromId( $database, $id, $ignoreInvalidDB = false ) {
-		return self::newFromLookup( $database, 'user_id', intval( $id ), $ignoreInvalidDB );
+	public static function newFromId( $dbDomain, $id, $ignoreInvalidDB = false ) {
+		return self::newFromLookup( $dbDomain, 'user_id', intval( $id ), $ignoreInvalidDB );
 	}
 
 	/**
 	 * Factory function; get a remote user entry by name.
 	 *
-	 * @param string $database Database name
+	 * @param string $dbDomain Database name
 	 * @param string $name User name
-	 * @param bool $ignoreInvalidDB If true, don't check if $database is in $wgLocalDatabases
+	 * @param bool $ignoreInvalidDB If true, don't check if $dbDomain is in $wgLocalDatabases
 	 * @return UserRightsProxy|null If doesn't exist
 	 */
-	public static function newFromName( $database, $name, $ignoreInvalidDB = false ) {
-		return self::newFromLookup( $database, 'user_name', $name, $ignoreInvalidDB );
+	public static function newFromName( $dbDomain, $name, $ignoreInvalidDB = false ) {
+		return self::newFromLookup( $dbDomain, 'user_name', $name, $ignoreInvalidDB );
 	}
 
 	/**
-	 * @param string $database
+	 * @param string $dbDomain
 	 * @param string $field
 	 * @param string $value
 	 * @param bool $ignoreInvalidDB
 	 * @return null|UserRightsProxy
 	 */
-	private static function newFromLookup( $database, $field, $value, $ignoreInvalidDB = false ) {
+	private static function newFromLookup( $dbDomain, $field, $value, $ignoreInvalidDB = false ) {
 		global $wgSharedDB, $wgSharedTables;
 		// If the user table is shared, perform the user query on it,
 		// but don't pass it to the UserRightsProxy,
@@ -120,10 +121,10 @@ class UserRightsProxy {
 		if ( $wgSharedDB && in_array( 'user', $wgSharedTables ) ) {
 			$userdb = self::getDB( $wgSharedDB, $ignoreInvalidDB );
 		} else {
-			$userdb = self::getDB( $database, $ignoreInvalidDB );
+			$userdb = self::getDB( $dbDomain, $ignoreInvalidDB );
 		}
 
-		$db = self::getDB( $database, $ignoreInvalidDB );
+		$db = self::getDB( $dbDomain, $ignoreInvalidDB );
 
 		if ( $db && $userdb ) {
 			$row = $userdb->selectRow( 'user',
@@ -132,9 +133,8 @@ class UserRightsProxy {
 				__METHOD__ );
 
 			if ( $row !== false ) {
-				return new UserRightsProxy( $db, $database,
-					$row->user_name,
-					intval( $row->user_id ) );
+				return new UserRightsProxy(
+					$db, $dbDomain, $row->user_name, intval( $row->user_id ) );
 			}
 		}
 		return null;
@@ -144,18 +144,17 @@ class UserRightsProxy {
 	 * Open a database connection to work on for the requested user.
 	 * This may be a new connection to another database for remote users.
 	 *
-	 * @param string $database
-	 * @param bool $ignoreInvalidDB If true, don't check if $database is in $wgLocalDatabases
+	 * @param string $dbDomain
+	 * @param bool $ignoreInvalidDB If true, don't check if $dbDomain is in $wgLocalDatabases
 	 * @return IDatabase|null If invalid selection
 	 */
-	public static function getDB( $database, $ignoreInvalidDB = false ) {
-		global $wgDBname;
-		if ( $ignoreInvalidDB || self::validDatabase( $database ) ) {
-			if ( $database == $wgDBname ) {
+	public static function getDB( $dbDomain, $ignoreInvalidDB = false ) {
+		if ( $ignoreInvalidDB || self::validDatabase( $dbDomain ) ) {
+			if ( WikiMap::isCurrentWikiId( $dbDomain ) ) {
 				// Hmm... this shouldn't happen though. :)
 				return wfGetDB( DB_MASTER );
 			} else {
-				return wfGetDB( DB_MASTER, [], $database );
+				return wfGetDB( DB_MASTER, [], $dbDomain );
 			}
 		}
 		return null;
@@ -181,7 +180,7 @@ class UserRightsProxy {
 	 * @return string
 	 */
 	public function getName() {
-		return $this->name . '@' . $this->database;
+		return $this->name . '@' . $this->dbDomain;
 	}
 
 	/**
