@@ -23,7 +23,7 @@
 
 require __DIR__ . '/../commandLine.inc';
 
-use Wikimedia\Rdbms\IMaintainableDatabase;
+use Wikimedia\Rdbms\Database;
 
 /**
  * Maintenance script that upgrade for log_id/log_deleted fields in a
@@ -34,14 +34,14 @@ use Wikimedia\Rdbms\IMaintainableDatabase;
 class UpdateLogging {
 
 	/**
-	 * @var IMaintainableDatabase
+	 * @var Database
 	 */
 	public $dbw;
 	public $batchSize = 1000;
 	public $minTs = false;
 
 	function execute() {
-		$this->dbw = $this->getDB( DB_MASTER );
+		$this->dbw = wfGetDB( DB_MASTER );
 		$logging = $this->dbw->tableName( 'logging' );
 		$logging_1_10 = $this->dbw->tableName( 'logging_1_10' );
 		$logging_pre_1_10 = $this->dbw->tableName( 'logging_pre_1_10' );
@@ -132,13 +132,13 @@ EOT;
 	 */
 	function sync( $srcTable, $dstTable ) {
 		$batchSize = 1000;
-		$minTs = $this->dbw->selectField( $srcTable, 'MIN(log_timestamp)', false, __METHOD__ );
+		$minTs = $this->dbw->selectField( $srcTable, 'MIN(log_timestamp)', '', __METHOD__ );
 		$minTsUnix = wfTimestamp( TS_UNIX, $minTs );
 		$numRowsCopied = 0;
 
 		while ( true ) {
-			$maxTs = $this->dbw->selectField( $srcTable, 'MAX(log_timestamp)', false, __METHOD__ );
-			$copyPos = $this->dbw->selectField( $dstTable, 'MAX(log_timestamp)', false, __METHOD__ );
+			$maxTs = $this->dbw->selectField( $srcTable, 'MAX(log_timestamp)', '', __METHOD__ );
+			$copyPos = $this->dbw->selectField( $dstTable, 'MAX(log_timestamp)', '', __METHOD__ );
 			$maxTsUnix = wfTimestamp( TS_UNIX, $maxTs );
 			$copyPosUnix = wfTimestamp( TS_UNIX, $copyPos );
 
