@@ -42,6 +42,9 @@ class CleanupImages extends TableCleanup {
 		'callback' => 'processRow',
 	];
 
+	/** @var LocalRepo|null */
+	private $repo;
+
 	public function __construct() {
 		parent::__construct();
 		$this->addDescription( 'Script to clean up broken, unparseable upload filenames' );
@@ -111,8 +114,12 @@ class CleanupImages extends TableCleanup {
 		}
 	}
 
+	/**
+	 * @param string $name
+	 * @return string
+	 */
 	private function filePath( $name ) {
-		if ( !isset( $this->repo ) ) {
+		if ( $this->repo === null ) {
 			$this->repo = RepoGroup::singleton()->getLocalRepo();
 		}
 
