@@ -44,8 +44,14 @@ class CompareParserCache extends Maintenance {
 		$withcache = 0;
 		$withdiff = 0;
 		$parserCache = MediaWikiServices::getInstance()->getParserCache();
+		$renderer = MediaWikiServices::getInstance()->getRevisionRenderer();
 		while ( $pages-- > 0 ) {
-			$row = $dbr->selectRow( 'page', '*',
+			$row = $dbr->selectRow( 'page',
+				// @todo Title::selectFields() or Title::getQueryInfo() or something
+				[
+					'page_namespace', 'page_title', 'page_id',
+					'page_len', 'page_is_redirect', 'page_latest',
+				],
 				[
 					'page_namespace' => $this->getOption( 'namespace' ),
 					'page_is_redirect' => 0,
@@ -64,17 +70,16 @@ class CompareParserCache extends Maintenance {
 
 			$title = Title::newFromRow( $row );
 			$page = WikiPage::factory( $title );
-			$revision = $page->getRevision();
-			$content = $revision->getContent( Revision::RAW );
-
+			$revision = $page->getRevision()->getRevisionRecord();
 			$parserOptions = $page->makeParserOptions( 'canonical' );
 
 			$parserOutputOld = $parserCache->get( $page, $parserOptions );
 
 			if ( $parserOutputOld ) {
 				$t1 = microtime( true );
-				$parserOutputNew = $content->getParserOutput(
-					$title, $revision->getId(), $parserOptions, false );
+				$parserOutputNew = $renderer->getRenderedRevision( $revision, $parserOptions )
+					->getRevisionParserOutput();
+
 				$sec = microtime( true ) - $t1;
 				$totalsec += $sec;
 
@@ -103,5 +108,5 @@ class CompareParserCache extends Maintenance {
 	}
 }
 
-$maintClass = "CompareParserCache";
+$maintClass = CompareParserCache::class;
 require_once RUN_MAINTENANCE_IF_MAIN;
