@@ -23,7 +23,7 @@
 
 require_once __DIR__ . '/Maintenance.php';
 
-use Wikimedia\Rdbms\ResultWrapper;
+use Wikimedia\Rdbms\IResultWrapper;
 
 /**
  * Maintenance script that sends purge requests for pages edited in a date
@@ -136,9 +136,9 @@ class PurgeChangedPages extends Maintenance {
 				}
 			}
 
-			// Send batch of purge requests out to squids
-			$squid = new CdnCacheUpdate( $urls, count( $urls ) );
-			$squid->doUpdate();
+			// Send batch of purge requests out to CDN servers
+			$cdn = new CdnCacheUpdate( $urls, count( $urls ) );
+			$cdn->doUpdate();
 
 			if ( $this->hasOption( 'sleep-per-batch' ) ) {
 				// sleep-per-batch is milliseconds, usleep wants micro seconds.
@@ -163,12 +163,12 @@ class PurgeChangedPages extends Maintenance {
 	 *
 	 * @todo move this elsewhere
 	 *
-	 * @param ResultWrapper $res Query result sorted by $column (ascending)
+	 * @param IResultWrapper $res Query result sorted by $column (ascending)
 	 * @param string $column
 	 * @param int $limit
 	 * @return array (array of rows, string column value)
 	 */
-	protected function pageableSortedRows( ResultWrapper $res, $column, $limit ) {
+	protected function pageableSortedRows( IResultWrapper $res, $column, $limit ) {
 		$rows = iterator_to_array( $res, false );
 
 		// Nothing to do
