@@ -103,11 +103,12 @@
 		 * @return {string} Parsed message
 		 */
 		parser: function () {
-			var text;
-			if ( mw.config.get( 'wgUserLanguage' ) === 'qqx' ) {
+			var text = this.map.get( this.key );
+			if (
+				mw.config.get( 'wgUserLanguage' ) === 'qqx' &&
+				( !text || text === '(' + this.key + ')' )
+			) {
 				text = '(' + this.key + '$*)';
-			} else {
-				text = this.map.get( this.key );
 			}
 			return mw.format.apply( null, [ text ].concat( this.parameters ) );
 		},
@@ -235,6 +236,18 @@
 	 * @singleton
 	 */
 
+	/**
+	 * Empty object for third-party libraries, for cases where you don't
+	 * want to add a new global, or the global is bad and needs containment
+	 * or wrapping.
+	 *
+	 * @property
+	 */
+	mw.libs = {};
+
+	// OOUI widgets specific to MediaWiki
+	mw.widgets = {};
+
 	/**
 	 * @inheritdoc mw.inspect#runReports
 	 * @method
@@ -698,6 +711,22 @@
 			} );
 	};
 
+	// Skeleton user object, extended by the 'mediawiki.user' module.
+	/**
+	 * @class mw.user
+	 * @singleton
+	 */
+	mw.user = {
+		/**
+		 * @property {mw.Map}
+		 */
+		options: new mw.Map(),
+		/**
+		 * @property {mw.Map}
+		 */
+		tokens: new mw.Map()
+	};
+
 	// Alias $j to jQuery for backwards compatibility
 	// @deprecated since 1.23 Use $ or jQuery instead
 	mw.log.deprecate( window, '$j', $, 'Use $ or jQuery instead.' );
