@@ -54,7 +54,7 @@ $( function () {
 	} );
 
 	// Turn logout to a POST action
-	$( '#pt-logout a' ).on( 'click', function ( e ) {
+	$( '#pt-logout a[data-mw="interface"]' ).on( 'click', function ( e ) {
 		var api = new mw.Api(),
 			url = this.href;
 		mw.notify(
