@@ -1,6 +1,5 @@
 <?php
 
-use MediaWiki\MediaWikiServices;
 use Wikimedia\TestingAccessWrapper;
 
 /**
@@ -405,11 +404,6 @@ class MessageTest extends MediaWikiLangTestCase {
 	 */
 	public function testRawHtmlInMsg() {
 		$this->setMwGlobals( 'wgRawHtml', true );
-		// We have to reset the core hook registration.
-		// to register the html hook
-		MessageCache::destroyInstance();
-		$this->setMwGlobals( 'wgParser',
-			MediaWikiServices::getInstance()->getParserFactory()->create() );
 
 		$msg = new RawMessage( '<html><script>alert("xss")</script></html>' );
 		$txt = '<span class="error">&lt;html&gt; tags cannot be' .
