@@ -1,5 +1,7 @@
 <?php
 
+use MediaWiki\Block\DatabaseBlock;
+
 /**
  * Tests for MediaWiki api.php?action=edit.
  *
@@ -408,10 +410,10 @@ class ApiEditPageTest extends ApiTestCase {
 		$count++;
 
 		/*
-		* T43990: if the target page has a newer revision than the redirect, then editing the
-		* redirect while specifying 'redirect' and *not* specifying 'basetimestamp' erroneously
-		* caused an edit conflict to be detected.
-		*/
+		 * T43990: if the target page has a newer revision than the redirect, then editing the
+		 * redirect while specifying 'redirect' and *not* specifying 'basetimestamp' erroneously
+		 * caused an edit conflict to be detected.
+		 */
 
 		// assume NS_HELP defaults to wikitext
 		$name = "Help:ApiEditPageTest_testEditConflict_redirect_T43990_$count";
@@ -1365,6 +1367,7 @@ class ApiEditPageTest extends ApiTestCase {
 		ChangeTags::defineTag( 'custom tag' );
 		$this->setMwGlobals( 'wgRevokePermissions',
 			[ 'user' => [ 'applychangetags' => true ] ] );
+
 		try {
 			$this->doApiRequestWithToken( [
 				'action' => 'edit',
@@ -1377,57 +1380,6 @@ class ApiEditPageTest extends ApiTestCase {
 		}
 	}
 
-	public function testEditAbortedByHook() {
-		$name = 'Help:' . ucfirst( __FUNCTION__ );
-
-		$this->setExpectedException( ApiUsageException::class,
-			'The modification you tried to make was aborted by an extension.' );
-
-		$this->hideDeprecated( 'APIEditBeforeSave hook (used in ' .
-			'hook-APIEditBeforeSave-closure)' );
-
-		$this->setTemporaryHook( 'APIEditBeforeSave',
-			function () {
-				return false;
-			}
-		);
-
-		try {
-			$this->doApiRequestWithToken( [
-				'action' => 'edit',
-				'title' => $name,
-				'text' => 'Some text',
-			] );
-		} finally {
-			$this->assertFalse( Title::newFromText( $name )->exists() );
-		}
-	}
-
-	public function testEditAbortedByHookWithCustomOutput() {
-		$name = 'Help:' . ucfirst( __FUNCTION__ );
-
-		$this->hideDeprecated( 'APIEditBeforeSave hook (used in ' .
-			'hook-APIEditBeforeSave-closure)' );
-
-		$this->setTemporaryHook( 'APIEditBeforeSave',
-			function ( $unused1, $unused2, &$r ) {
-				$r['msg'] = 'Some message';
-				return false;
-			} );
-
-		$result = $this->doApiRequestWithToken( [
-			'action' => 'edit',
-			'title' => $name,
-			'text' => 'Some text',
-		] );
-		Wikimedia\restoreWarnings();
-
-		$this->assertSame( [ 'msg' => 'Some message', 'result' => 'Failure' ],
-			$result[0]['edit'] );
-
-		$this->assertFalse( Title::newFromText( $name )->exists() );
-	}
-
 	public function testEditAbortedByEditPageHookWithResult() {
 		$name = 'Help:' . ucfirst( __FUNCTION__ );
 
@@ -1474,9 +1426,9 @@ class ApiEditPageTest extends ApiTestCase {
 	public function testEditWhileBlocked() {
 		$name = 'Help:' . ucfirst( __FUNCTION__ );
 
-		$this->assertNull( Block::newFromTarget( '127.0.0.1' ), 'Sanity check' );
+		$this->assertNull( DatabaseBlock::newFromTarget( '127.0.0.1' ), 'Sanity check' );
 
-		$block = new Block( [
+		$block = new DatabaseBlock( [
 			'address' => self::$users['sysop']->getUser()->getName(),
 			'by' => self::$users['sysop']->getUser()->getId(),
 			'reason' => 'Capriciousness',
@@ -1495,7 +1447,7 @@ class ApiEditPageTest extends ApiTestCase {
 			$this->fail( 'Expected exception not thrown' );
 		} catch ( ApiUsageException $ex ) {
 			$this->assertSame( 'You have been blocked from editing.', $ex->getMessage() );
-			$this->assertNotNull( Block::newFromTarget( '127.0.0.1' ), 'Autoblock spread' );
+			$this->assertNotNull( DatabaseBlock::newFromTarget( '127.0.0.1' ), 'Autoblock spread' );
 		} finally {
 			$block->delete();
 			self::$users['sysop']->getUser()->clearInstanceCache();
