@@ -29,8 +29,6 @@ class ApiOptionsTest extends MediaWikiLangTestCase {
 		// Set up groups and rights
 		$this->mUserMock->expects( $this->any() )
 			->method( 'getEffectiveGroups' )->will( $this->returnValue( [ '*', 'user' ] ) );
-		$this->mUserMock->expects( $this->any() )
-			->method( 'isAllowedAny' )->will( $this->returnValue( true ) );
 
 		// Set up callback for User::getOptionKinds
 		$this->mUserMock->expects( $this->any() )
@@ -44,11 +42,22 @@ class ApiOptionsTest extends MediaWikiLangTestCase {
 		$this->mUserMock->method( 'getOptions' )
 			->willReturn( [] );
 
+		// DefaultPreferencesFactory calls a ton of user methods, but we still want to list all of
+		// them in case bugs are caused by unexpected things returning null that shouldn't.
+		$this->mUserMock->expects( $this->never() )->method( $this->anythingBut(
+			'getEffectiveGroups', 'getOptionKinds', 'getInstanceForUpdate', 'getOptions', 'getId',
+			'isAnon', 'getRequest', 'isLoggedIn', 'getName', 'getGroupMemberships', 'getEditCount',
+			'getRegistration', 'isAllowed', 'getRealName', 'getOption', 'getStubThreshold',
+			'getBoolOption', 'getEmail', 'getDatePreference', 'useRCPatrol', 'useNPPatrol',
+			'setOption', 'saveSettings', 'resetOptions', 'isRegistered'
+		) );
+
 		// Create a new context
 		$this->mContext = new DerivativeContext( new RequestContext() );
 		$this->mContext->getContext()->setTitle( Title::newFromText( 'Test' ) );
 		$this->mContext->setUser( $this->mUserMock );
 
+		$this->overrideUserPermissions( $this->mUserMock, [ 'editmyoptions' ] );
 		$main = new ApiMain( $this->mContext );
 
 		// Empty session
@@ -154,6 +163,8 @@ class ApiOptionsTest extends MediaWikiLangTestCase {
 
 	private function executeQuery( $request ) {
 		$this->mContext->setRequest( new FauxRequest( $request, true, $this->mSession ) );
+		$this->mUserMock->method( 'getRequest' )->willReturn( $this->mContext->getRequest() );
+
 		$this->mTested->execute();
 
 		return $this->mTested->getResult()->getResultData( null, [ 'Strip' => 'all' ] );
