@@ -1,4 +1,5 @@
 <?php
+
 use MediaWiki\MediaWikiServices;
 use Wikimedia\TestingAccessWrapper;
 
@@ -180,7 +181,7 @@ class ContentHandlerTest extends MediaWikiTestCase {
 		$content = null;
 
 		$text = ContentHandler::getContentText( $content );
-		$this->assertEquals( '', $text );
+		$this->assertSame( '', $text );
 	}
 
 	public static function dataGetContentText_TextContent() {
@@ -519,7 +520,7 @@ class ContentHandlerTest extends MediaWikiTestCase {
 		$customContentHandler->expects( $this->any() )
 			->method( 'createDifferenceEngine' )
 			->willReturn( $customDifferenceEngine );
-		/** @var $customContentHandler ContentHandler */
+		/** @var ContentHandler $customContentHandler */
 		$slotDiffRenderer = $customContentHandler->getSlotDiffRenderer( RequestContext::getMain() );
 		$this->assertInstanceOf( DifferenceEngineSlotDiffRenderer::class, $slotDiffRenderer );
 		$this->assertSame(
@@ -553,7 +554,7 @@ class ContentHandlerTest extends MediaWikiTestCase {
 		$customContentHandler2->expects( $this->any() )
 			->method( 'getSlotDiffRendererInternal' )
 			->willReturn( $customSlotDiffRenderer );
-		/** @var $customContentHandler2 ContentHandler */
+		/** @var ContentHandler $customContentHandler2 */
 		$slotDiffRenderer = $customContentHandler2->getSlotDiffRenderer( RequestContext::getMain() );
 		$this->assertSame( $customSlotDiffRenderer, $slotDiffRenderer );
 	}
@@ -577,7 +578,7 @@ class ContentHandlerTest extends MediaWikiTestCase {
 		$customContentHandler->expects( $this->any() )
 			->method( 'createDifferenceEngine' )
 			->willReturn( $customDifferenceEngine );
-		/** @var $customContentHandler ContentHandler */
+		/** @var ContentHandler $customContentHandler */
 
 		$customSlotDiffRenderer = $this->getMockBuilder( SlotDiffRenderer::class )
 			->disableOriginalConstructor()
@@ -592,7 +593,7 @@ class ContentHandlerTest extends MediaWikiTestCase {
 		$customContentHandler2->expects( $this->any() )
 			->method( 'getSlotDiffRendererInternal' )
 			->willReturn( $customSlotDiffRenderer );
-		/** @var $customContentHandler2 ContentHandler */
+		/** @var ContentHandler $customContentHandler2 */
 
 		$customSlotDiffRenderer2 = $this->getMockBuilder( SlotDiffRenderer::class )
 			->disableOriginalConstructor()
