@@ -3,9 +3,7 @@
  * PHP Unit tests for MWMessagePack
  * @covers MWMessagePack
  */
-class MWMessagePackTest extends PHPUnit\Framework\TestCase {
-
-	use MediaWikiCoversValidator;
+class MWMessagePackTest extends MediaWikiTestCase {
 
 	/**
 	 * Provides test cases for MWMessagePackTest::testMessagePack
@@ -71,6 +69,7 @@ class MWMessagePackTest extends PHPUnit\Framework\TestCase {
 	 * @dataProvider providePacks
 	 */
 	public function testPack( $type, $value, $expected ) {
+		$this->hideDeprecated( MWMessagePack::class . '::pack' );
 		$actual = bin2hex( MWMessagePack::pack( $value ) );
 		$this->assertEquals( $expected, $actual, $type );
 	}
