@@ -119,15 +119,18 @@ class CommandTest extends PHPUnit\Framework\TestCase {
 	}
 
 	public function testT69870() {
-		$commandLine = wfIsWindows()
-			// 333 = 331 + CRLF
-			? ( 'for /l %i in (1, 1, 1001) do @echo ' . str_repeat( '*', 331 ) )
-			: 'printf "%-333333s" "*"';
+		if ( wfIsWindows() ) {
+			// T209159: Anonymous pipe under Windows does not support asynchronous read and write,
+			// and the default buffer is too small (~4K), it is easy to be blocked.
+			$this->markTestSkipped(
+				'T209159: Anonymous pipe under Windows cannot withstand such a large amount of data'
+			);
+		}
 
 		// Test several times because it involves a race condition that may randomly succeed or fail
 		for ( $i = 0; $i < 10; $i++ ) {
 			$command = new Command();
-			$output = $command->unsafeParams( $commandLine )
+			$output = $command->unsafeParams( 'printf "%-333333s" "*"' )
 				->execute()
 				->getStdout();
 			$this->assertEquals( 333333, strlen( $output ) );
