@@ -1,5 +1,8 @@
 <?php
 
+use MediaWiki\Permissions\PermissionManager;
+use MediaWiki\User\UserIdentityValue;
+use Wikimedia\Rdbms\IDatabase;
 use Wikimedia\Rdbms\LoadBalancer;
 use Wikimedia\TestingAccessWrapper;
 
@@ -8,8 +11,6 @@ use Wikimedia\TestingAccessWrapper;
  */
 class WatchedItemQueryServiceUnitTest extends MediaWikiTestCase {
 
-	use MediaWikiCoversValidator;
-
 	/**
 	 * @return PHPUnit_Framework_MockObject_MockObject|CommentStore
 	 */
@@ -66,24 +67,24 @@ class WatchedItemQueryServiceUnitTest extends MediaWikiTestCase {
 
 	/**
 	 * @param PHPUnit_Framework_MockObject_MockObject|Database $mockDb
+	 * @param PHPUnit_Framework_MockObject_MockObject|PermissionManager $mockPM
 	 * @return WatchedItemQueryService
 	 */
-	private function newService( $mockDb ) {
+	private function newService( $mockDb, $mockPM = null ) {
 		return new WatchedItemQueryService(
 			$this->getMockLoadBalancer( $mockDb ),
 			$this->getMockCommentStore(),
 			$this->getMockActorMigration(),
-			$this->getMockWatchedItemStore()
+			$this->getMockWatchedItemStore(),
+			$mockPM ?: $this->getMockPermissionManager()
 		);
 	}
 
 	/**
-	 * @return PHPUnit_Framework_MockObject_MockObject|Database
+	 * @return PHPUnit_Framework_MockObject_MockObject|IDatabase
 	 */
 	private function getMockDb() {
-		$mock = $this->getMockBuilder( Database::class )
-			->disableOriginalConstructor()
-			->getMock();
+		$mock = $this->createMock( IDatabase::class );
 
 		$mock->expects( $this->any() )
 			->method( 'makeList' )
@@ -126,7 +127,7 @@ class WatchedItemQueryServiceUnitTest extends MediaWikiTestCase {
 	}
 
 	/**
-	 * @param PHPUnit_Framework_MockObject_MockObject|Database $mockDb
+	 * @param PHPUnit_Framework_MockObject_MockObject|IDatabase $mockDb
 	 * @return PHPUnit_Framework_MockObject_MockObject|LoadBalancer
 	 */
 	private function getMockLoadBalancer( $mockDb ) {
@@ -141,7 +142,6 @@ class WatchedItemQueryServiceUnitTest extends MediaWikiTestCase {
 	}
 
 	/**
-	 * @param PHPUnit_Framework_MockObject_MockObject|Database $mockDb
 	 * @return PHPUnit_Framework_MockObject_MockObject|WatchedItemStore
 	 */
 	private function getMockWatchedItemStore() {
@@ -157,35 +157,50 @@ class WatchedItemQueryServiceUnitTest extends MediaWikiTestCase {
 	}
 
 	/**
-	 * @param int $id
-	 * @return PHPUnit_Framework_MockObject_MockObject|User
+	 * @param string $notAllowedAction
+	 * @return PHPUnit_Framework_MockObject_MockObject|PermissionManager
 	 */
-	private function getMockNonAnonUserWithId( $id ) {
-		$mock = $this->getMockBuilder( User::class )->getMock();
-		$mock->expects( $this->any() )
-			->method( 'isAnon' )
-			->will( $this->returnValue( false ) );
-		$mock->expects( $this->any() )
-			->method( 'getId' )
-			->will( $this->returnValue( $id ) );
+	private function getMockPermissionManager( $notAllowedAction = null ) {
+		$mock = $this->getMockBuilder( PermissionManager::class )
+			->disableOriginalConstructor()
+			->getMock();
+		$mock->method( 'userHasRight' )
+			->will( $this->returnCallback( function ( $user, $action ) use ( $notAllowedAction ) {
+				return $action !== $notAllowedAction;
+			} ) );
+		$mock->method( 'userHasAnyRight' )
+			->will( $this->returnCallback( function ( $user, ...$actions ) use ( $notAllowedAction ) {
+				return !in_array( $notAllowedAction, $actions );
+			} ) );
 		return $mock;
 	}
 
 	/**
 	 * @param int $id
+	 * @param string[] $extraMethods Extra methods that are expected might be called
 	 * @return PHPUnit_Framework_MockObject_MockObject|User
 	 */
-	private function getMockUnrestrictedNonAnonUserWithId( $id ) {
-		$mock = $this->getMockNonAnonUserWithId( $id );
-		$mock->expects( $this->any() )
-			->method( 'isAllowed' )
-			->will( $this->returnValue( true ) );
-		$mock->expects( $this->any() )
-			->method( 'isAllowedAny' )
-			->will( $this->returnValue( true ) );
-		$mock->expects( $this->any() )
-			->method( 'useRCPatrol' )
-			->will( $this->returnValue( true ) );
+	private function getMockNonAnonUserWithId( $id, array $extraMethods = [] ) {
+		$mock = $this->getMockBuilder( User::class )->getMock();
+		$mock->method( 'isRegistered' )->willReturn( true );
+		$mock->method( 'getId' )->willReturn( $id );
+		$methods = array_merge( [
+			'isRegistered',
+			'getId',
+		], $extraMethods );
+		$mock->expects( $this->never() )->method( $this->anythingBut( ...$methods ) );
+		return $mock;
+	}
+
+	/**
+	 * @param int $id
+	 * @param string[] $extraMethods Extra methods that are expected might be called
+	 * @return PHPUnit_Framework_MockObject_MockObject|User
+	 */
+	private function getMockUnrestrictedNonAnonUserWithId( $id, array $extraMethods = [] ) {
+		$mock = $this->getMockNonAnonUserWithId( $id,
+			array_merge( [ 'useRCPatrol' ], $extraMethods ) );
+		$mock->method( 'useRCPatrol' )->willReturn( true );
 		return $mock;
 	}
 
@@ -194,20 +209,11 @@ class WatchedItemQueryServiceUnitTest extends MediaWikiTestCase {
 	 * @param string $notAllowedAction
 	 * @return PHPUnit_Framework_MockObject_MockObject|User
 	 */
-	private function getMockNonAnonUserWithIdAndRestrictedPermissions( $id, $notAllowedAction ) {
-		$mock = $this->getMockNonAnonUserWithId( $id );
-
-		$mock->expects( $this->any() )
-			->method( 'isAllowed' )
-			->will( $this->returnCallback( function ( $action ) use ( $notAllowedAction ) {
-				return $action !== $notAllowedAction;
-			} ) );
-		$mock->expects( $this->any() )
-			->method( 'isAllowedAny' )
-			->will( $this->returnCallback( function () use ( $notAllowedAction ) {
-				$actions = func_get_args();
-				return !in_array( $notAllowedAction, $actions );
-			} ) );
+	private function getMockNonAnonUserWithIdAndRestrictedPermissions( $id ) {
+		$mock = $this->getMockNonAnonUserWithId( $id,
+			[ 'useRCPatrol', 'useNPPatrol' ] );
+		$mock->method( 'useRCPatrol' )->willReturn( false );
+		$mock->method( 'useNPPatrol' )->willReturn( false );
 
 		return $mock;
 	}
@@ -217,15 +223,8 @@ class WatchedItemQueryServiceUnitTest extends MediaWikiTestCase {
 	 * @return PHPUnit_Framework_MockObject_MockObject|User
 	 */
 	private function getMockNonAnonUserWithIdAndNoPatrolRights( $id ) {
-		$mock = $this->getMockNonAnonUserWithId( $id );
-
-		$mock->expects( $this->any() )
-			->method( 'isAllowed' )
-			->will( $this->returnValue( true ) );
-		$mock->expects( $this->any() )
-			->method( 'isAllowedAny' )
-			->will( $this->returnValue( true ) );
-
+		$mock = $this->getMockNonAnonUserWithId( $id,
+			[ 'useRCPatrol', 'useNPPatrol' ] );
 		$mock->expects( $this->any() )
 			->method( 'useRCPatrol' )
 			->will( $this->returnValue( false ) );
@@ -236,14 +235,6 @@ class WatchedItemQueryServiceUnitTest extends MediaWikiTestCase {
 		return $mock;
 	}
 
-	private function getMockAnonUser() {
-		$mock = $this->getMockBuilder( User::class )->getMock();
-		$mock->expects( $this->any() )
-			->method( 'isAnon' )
-			->will( $this->returnValue( true ) );
-		return $mock;
-	}
-
 	private function getFakeRow( array $rowValues ) {
 		$fakeRow = new stdClass();
 		foreach ( $rowValues as $valueName => $value ) {
@@ -1144,9 +1135,10 @@ class WatchedItemQueryServiceUnitTest extends MediaWikiTestCase {
 			)
 			->will( $this->returnValue( [] ) );
 
-		$user = $this->getMockNonAnonUserWithIdAndRestrictedPermissions( 1, $notAllowedAction );
+		$permissionManager = $this->getMockPermissionManager( $notAllowedAction );
+		$user = $this->getMockNonAnonUserWithIdAndRestrictedPermissions( 1 );
 
-		$queryService = $this->newService( $mockDb );
+		$queryService = $this->newService( $mockDb, $permissionManager );
 		$items = $queryService->getWatchedItemsWithRecentChangeInfo( $user, $options );
 
 		$this->assertEmpty( $items );
@@ -1385,7 +1377,7 @@ class WatchedItemQueryServiceUnitTest extends MediaWikiTestCase {
 
 		$queryService = $this->newService( $mockDb );
 		$user = $this->getMockUnrestrictedNonAnonUserWithId( 1 );
-		$otherUser = $this->getMockUnrestrictedNonAnonUserWithId( 2 );
+		$otherUser = $this->getMockUnrestrictedNonAnonUserWithId( 2, [ 'getOption' ] );
 		$otherUser->expects( $this->once() )
 			->method( 'getOption' )
 			->with( 'watchlisttoken' )
@@ -1416,7 +1408,7 @@ class WatchedItemQueryServiceUnitTest extends MediaWikiTestCase {
 
 		$queryService = $this->newService( $mockDb );
 		$user = $this->getMockUnrestrictedNonAnonUserWithId( 1 );
-		$otherUser = $this->getMockUnrestrictedNonAnonUserWithId( 2 );
+		$otherUser = $this->getMockUnrestrictedNonAnonUserWithId( 2, [ 'getOption' ] );
 		$otherUser->expects( $this->once() )
 			->method( 'getOption' )
 			->with( 'watchlisttoken' )
@@ -1716,7 +1708,8 @@ class WatchedItemQueryServiceUnitTest extends MediaWikiTestCase {
 
 		$queryService = $this->newService( $mockDb );
 
-		$items = $queryService->getWatchedItemsForUser( $this->getMockAnonUser() );
+		$items = $queryService->getWatchedItemsForUser(
+			new UserIdentityValue( 0, 'AnonUser', 0 ) );
 		$this->assertEmpty( $items );
 	}
 
