@@ -5,8 +5,12 @@ namespace MediaWiki\Tests\Maintenance;
 use DumpBackup;
 use Exception;
 use MediaWiki\MediaWikiServices;
+use MediaWiki\Revision\RevisionRecord;
+use MediaWiki\Revision\SlotRecord;
 use MediaWikiTestCase;
 use MWException;
+use RequestContext;
+use RevisionDeleter;
 use Title;
 use WikiExporter;
 use Wikimedia\Rdbms\IDatabase;
@@ -25,13 +29,14 @@ class BackupDumperPageTest extends DumpTestCase {
 
 	// We'll add several pages, revision and texts. The following variables hold the
 	// corresponding ids.
-	private $pageId1, $pageId2, $pageId3, $pageId4;
-	private $pageTitle1, $pageTitle2, $pageTitle3, $pageTitle4;
+	private $pageId1, $pageId2, $pageId3, $pageId4, $pageId5;
+	private $pageTitle1, $pageTitle2, $pageTitle3, $pageTitle4, $pageTitle5;
 	private $revId1_1, $textId1_1;
 	private $revId2_1, $textId2_1, $revId2_2, $textId2_2;
 	private $revId2_3, $textId2_3, $revId2_4, $textId2_4;
 	private $revId3_1, $textId3_1, $revId3_2, $textId3_2;
 	private $revId4_1, $textId4_1;
+	private $revId5_1, $textId5_1;
 	private $namespace, $talk_namespace;
 
 	/**
@@ -77,6 +82,17 @@ class BackupDumperPageTest extends DumpTestCase {
 				"BackupDumperTestP2Summary4 extra " );
 			$this->pageId2 = $page->getId();
 
+			$revDel = RevisionDeleter::createList(
+				'revision',
+				RequestContext::getMain(),
+				$this->pageTitle2,
+				[ $this->revId2_2 ]
+			);
+			$revDel->setVisibility( [
+				'value' => [ RevisionRecord::DELETED_TEXT => 1 ],
+				'comment' => 'testing!'
+			] );
+
 			$this->pageTitle3 = Title::newFromText( 'BackupDumperTestP3', $this->namespace );
 			$page = WikiPage::factory( $this->pageTitle3 );
 			list( $this->revId3_1, $this->textId3_1 ) = $this->addRevision( $page,
@@ -92,6 +108,15 @@ class BackupDumperPageTest extends DumpTestCase {
 				"Talk about BackupDumperTestP1 Text1",
 				"Talk BackupDumperTestP1 Summary1" );
 			$this->pageId4 = $page->getId();
+
+			$this->pageTitle5 = Title::newFromText( 'BackupDumperTestP5' );
+			$page = WikiPage::factory( $this->pageTitle5 );
+			list( $this->revId5_1, $this->textId5_1 ) = $this->addRevision( $page,
+				"BackupDumperTestP5 Text1",
+				"BackupDumperTestP5 Summary1" );
+			$this->pageId5 = $page->getId();
+
+			$this->corruptRevisionData( $page->getRevision()->getRevisionRecord() );
 		} catch ( Exception $e ) {
 			// We'd love to pass $e directly. However, ... see
 			// documentation of exceptionFromAddDBData in
@@ -100,6 +125,31 @@ class BackupDumperPageTest extends DumpTestCase {
 		}
 	}
 
+	/**
+	 * Corrupt the information about the given revision in the database.
+	 *
+	 * @param RevisionRecord $revision
+	 */
+	private function corruptRevisionData( RevisionRecord $revision ) {
+		global $wgMultiContentRevisionSchemaMigrationStage;
+
+		if ( ( $wgMultiContentRevisionSchemaMigrationStage & SCHEMA_COMPAT_WRITE_OLD ) ) {
+			$this->db->update(
+				'revision',
+				[ 'rev_text_id' => 0 ],
+				[ 'rev_id' => $revision->getId() ]
+			);
+		}
+
+		if ( ( $wgMultiContentRevisionSchemaMigrationStage & SCHEMA_COMPAT_WRITE_NEW ) ) {
+			$this->db->update(
+				'content',
+				[ 'content_address' => 'tt:0' ],
+				[ 'content_id' => $revision->getSlot( SlotRecord::MAIN )->getContentId() ]
+			);
+		}
+	}
+
 	protected function setUp() {
 		parent::setUp();
 
@@ -187,11 +237,14 @@ class BackupDumperPageTest extends DumpTestCase {
 		$dumper = $this->newDumpBackup(
 			[ '--full', '--quiet', '--output', 'file:' . $fname, '--schema-version', $schemaVersion ],
 			$this->pageId1,
-			$this->pageId4 + 1
+			$this->pageId5 + 1
 		);
 
-		// Performing the dump
+		// Performing the dump. Suppress warnings, since we want to test
+		// accessing broken revision data (page 5).
+		$this->setMwGlobals( 'wgDevelopmentWarnings', false );
 		$dumper->execute();
+		$this->setMwGlobals( 'wgDevelopmentWarnings', true );
 
 		// Checking the dumped data
 		$this->assertDumpSchema( $fname, $this->getXmlSchemaPath( $schemaVersion ) );
@@ -232,10 +285,10 @@ class BackupDumperPageTest extends DumpTestCase {
 		$asserter->assertRevision(
 			$this->revId2_2,
 			"BackupDumperTestP2Summary2",
-			$this->textId2_2,
-			23,
-			"b7vj5ks32po5m1z1t1br4o7scdwwy95",
-			"BackupDumperTestP2Text2",
+			null, // deleted!
+			false, // deleted!
+			null, // deleted!
+			false, // deleted!
 			$this->revId2_1
 		);
 		$asserter->assertRevision(
@@ -276,8 +329,26 @@ class BackupDumperPageTest extends DumpTestCase {
 			"Talk about BackupDumperTestP1 Text1",
 			false,
 			CONTENT_MODEL_WIKITEXT,
-			CONTENT_FORMAT_WIKITEXT,
-			$schemaVersion
+			CONTENT_FORMAT_WIKITEXT
+		);
+		$asserter->assertPageEnd();
+
+		// Page 5 (broken revision data)
+		$asserter->assertPageStart(
+			$this->pageId5,
+			$this->namespace,
+			$this->pageTitle5->getPrefixedText()
+		);
+		$asserter->assertRevision(
+			$this->revId5_1,
+			"BackupDumperTestP5 Summary1",
+			null,
+			24,
+			"d2vipufvkfs9wfruwjfj8eschxw0fbl",
+			false,
+			false,
+			CONTENT_MODEL_WIKITEXT,
+			CONTENT_FORMAT_WIKITEXT
 		);
 		$asserter->assertPageEnd();
 
@@ -303,11 +374,14 @@ class BackupDumperPageTest extends DumpTestCase {
 				'--schema-version', $schemaVersion,
 			],
 			$this->pageId1,
-			$this->pageId4 + 1
+			$this->pageId5 + 1
 		);
 
-		// Performing the dump
+		// Performing the dump. Suppress warnings, since we want to test
+		// accessing broken revision data (page 5).
+		$this->setMwGlobals( 'wgDevelopmentWarnings', false );
 		$dumper->execute();
+		$this->setMwGlobals( 'wgDevelopmentWarnings', true );
 
 		// Checking the dumped data
 		$this->assertDumpSchema( $fname, $this->getXmlSchemaPath( $schemaVersion ) );
@@ -346,10 +420,10 @@ class BackupDumperPageTest extends DumpTestCase {
 		$asserter->assertRevision(
 			$this->revId2_2,
 			"BackupDumperTestP2Summary2",
-			$this->textId2_2,
-			23,
-			"b7vj5ks32po5m1z1t1br4o7scdwwy95",
-			false,
+			null, // deleted!
+			false, // deleted!
+			null, // deleted!
+			false, // deleted!
 			$this->revId2_1
 		);
 		$asserter->assertRevision(
@@ -390,6 +464,21 @@ class BackupDumperPageTest extends DumpTestCase {
 		);
 		$asserter->assertPageEnd();
 
+		// Page 5 (broken revision data)
+		$asserter->assertPageStart(
+			$this->pageId5,
+			$this->namespace,
+			$this->pageTitle5->getPrefixedText()
+		);
+		$asserter->assertRevision(
+			$this->revId5_1,
+			"BackupDumperTestP5 Summary1",
+			null,
+			24,
+			"d2vipufvkfs9wfruwjfj8eschxw0fbl"
+		);
+		$asserter->assertPageEnd();
+
 		$asserter->assertDumpEnd();
 	}
 
@@ -622,10 +711,10 @@ class BackupDumperPageTest extends DumpTestCase {
 		$asserter->assertRevision(
 			$this->revId2_2,
 			"BackupDumperTestP2Summary2",
-			$this->textId2_2,
-			23,
-			"b7vj5ks32po5m1z1t1br4o7scdwwy95",
-			false,
+			null, // deleted!
+			false, // deleted!
+			null, // deleted!
+			false, // deleted!
 			$this->revId2_1
 		);
 		$asserter->assertRevision(
