@@ -1,9 +1,9 @@
 const Page = require( 'wdio-mediawiki/Page' );
 
 class RestorePage extends Page {
-	get reason() { return browser.element( '#wpComment' ); }
-	get submit() { return browser.element( '#mw-undelete-submit' ); }
-	get displayedContent() { return browser.element( '#mw-content-text' ); }
+	get reason() { return $( '#wpComment' ); }
+	get submit() { return $( '#mw-undelete-submit' ); }
+	get displayedContent() { return $( '#mw-content-text' ); }
 
 	open( subject ) {
 		super.openTitle( 'Special:Undelete/' + subject );
