@@ -1,5 +1,5 @@
-const assert = require( 'assert' ),
-	BlankPage = require( './../BlankPage' );
+const assert = require( 'assert' );
+const BlankPage = require( './../BlankPage' );
 
 describe( 'BlankPage', function () {
 	it( 'should have its title @daily', function () {
